@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +17,10 @@
 #ifndef COMMON_UTILS_HPP
 #define COMMON_UTILS_HPP
 
+#include <atomic>
 #include <cassert>
+#include <climits>
+#include <cmath>
 #include <cstddef>
 #include <cstdint>
 #include <cstdio>
@@ -26,6 +29,7 @@
 
 #include <memory>
 #include <string>
+#include <tuple>
 
 #define MSAN_ENABLED 0
 #define ATTR_NO_MSAN
@@ -161,11 +165,11 @@ std::unique_ptr<T> make_unique(Args &&... args) {
 }
 
 template <typename T, typename P>
-inline bool everyone_is(T val, P item) {
+constexpr bool everyone_is(T val, P item) {
     return val == item;
 }
 template <typename T, typename P, typename... Args>
-inline bool everyone_is(T val, P item, Args... item_others) {
+constexpr bool everyone_is(T val, P item, Args... item_others) {
     return val == item && everyone_is(val, item_others...);
 }
 
@@ -188,7 +192,7 @@ constexpr P map(T pat, P def, T item, P ival, Args... item_others) {
 }
 
 template <typename... Args>
-inline bool any_null(Args... ptrs) {
+constexpr bool any_null(Args... ptrs) {
     return one_of(nullptr, ptrs...);
 }
 
@@ -219,13 +223,13 @@ constexpr int product_impl(const T *arr, int2type<0>) {
 }
 
 template <typename T, size_t num>
-inline T product_impl(const T *arr, int2type<num>) {
+constexpr T product_impl(const T *arr, int2type<num>) {
     return arr[0] * product_impl(arr + 1, int2type<num - 1>());
 }
 } // namespace product_impl
 
 template <size_t num, typename T>
-inline T array_product(const T *arr) {
+constexpr T array_product(const T *arr) {
     return product_impl::product_impl(arr, product_impl::int2type<num - 1>());
 }
 
@@ -237,6 +241,18 @@ inline R array_product(const T *arr, size_t size) {
     return prod;
 }
 
+template <typename T, typename R = T>
+inline R array_min(const T *arr, size_t size) {
+    R min = std::numeric_limits<R>::max();
+    for (size_t i = 0; i < size; ++i)
+        min = std::min(min, arr[i]);
+    return min;
+}
+
+inline bool equal_with_nan(float v1, float v2) {
+    return (v1 == v2) || (std::isnan(v1) && std::isnan(v2));
+}
+
 /* Sorts an array of @p vals using @p comparator. Uses @p vals_2nd_level as a
  * second level comparing criteria in case comparator returns 0 (equal values)
  * for @p vals elements.
@@ -269,7 +285,7 @@ inline void simultaneous_sort(
 }
 
 template <typename T>
-inline const T &saturate(const T &low, const T &upper, const T &a) {
+constexpr const T &saturate(const T &low, const T &upper, const T &a) {
     return nstl::max(low, nstl::min(upper, a));
 }
 
@@ -285,10 +301,23 @@ inline typename remove_reference<T>::type rnd_up(const T a, const U b) {
 }
 
 template <typename T, typename U>
-inline typename remove_reference<T>::type rnd_dn(const T a, const U b) {
+constexpr typename remove_reference<T>::type rnd_dn(const T a, const U b) {
     return static_cast<typename remove_reference<T>::type>((a / b) * b);
 }
 
+template <typename T>
+inline typename remove_reference<T>::type rnd_up_pow2(const T a) {
+    using R = typename remove_reference<T>::type;
+    if (a <= 0)
+        return static_cast<R>(1);
+    else {
+        T b = a - 1;
+        for (size_t v = 1; v < sizeof(T) * CHAR_BIT; v <<= 1)
+            b |= (b >> v);
+        return static_cast<R>(b + 1);
+    }
+}
+
 template <typename T, typename U>
 inline typename remove_reference<T>::type max_div(const T a, const U b) {
     U div = b;
@@ -299,6 +328,11 @@ inline typename remove_reference<T>::type max_div(const T a, const U b) {
     return static_cast<typename remove_reference<T>::type>(div);
 }
 
+template <typename T>
+inline typename remove_reference<T>::type max_pow2_div(const T a) {
+    return static_cast<typename remove_reference<T>::type>(((a - 1) & ~a) + 1);
+}
+
 template <typename T>
 T *align_ptr(T *ptr, uintptr_t alignment) {
     return (T *)(((uintptr_t)ptr + alignment - 1) & ~(alignment - 1));
@@ -369,11 +403,11 @@ inline bool nd_iterator_jump(
 }
 
 template <typename T>
-inline T pick(size_t i, const T &x0) {
+constexpr T pick(size_t i, const T &x0) {
     return x0;
 }
 template <typename T, typename... Args>
-inline T pick(size_t i, const T &x0, Args &&... args) {
+constexpr T pick(size_t i, const T &x0, Args &&... args) {
     return i == 0 ? x0 : pick(i - 1, utils::forward<Args>(args)...);
 }
 
@@ -402,26 +436,31 @@ struct array_offset_calculator {
     array_offset_calculator(Telem *base, Targs... Fargs) : _dims {Fargs...} {
         _base_ptr = base;
     }
+
     template <typename... Targs>
-    inline Telem &operator()(Targs... Fargs) {
+    array_offset_calculator(std::nullptr_t, Targs... Fargs) = delete;
+
+    template <typename... Targs>
+    inline Telem &operator()(Targs... Fargs) const {
+        assert(static_cast<bool>(_base_ptr));
         return *(_base_ptr + _offset(1, Fargs...));
     }
 
 private:
     template <typename... Targs>
-    inline size_t _offset(size_t const dimension, size_t element) {
+    inline size_t _offset(size_t const dimension, size_t element) const {
         return element;
     }
 
     template <typename... Targs>
     inline size_t _offset(
-            size_t const dimension, size_t theta, size_t element) {
+            size_t const dimension, size_t theta, size_t element) const {
         return element + (_dims[dimension] * theta);
     }
 
     template <typename... Targs>
     inline size_t _offset(size_t const dimension, size_t theta, size_t element,
-            Targs... Fargs) {
+            Targs... Fargs) const {
         size_t t_prime = element + (_dims[dimension] * theta);
         return _offset(dimension + 1, t_prime, Fargs...);
     }
@@ -536,12 +575,21 @@ inline void yield_thread() {}
 int getenv(const char *name, char *buffer, int buffer_size);
 // Reads an integer from the environment
 int getenv_int(const char *name, int default_value = 0);
+// Various getter for profiling info
 bool get_jit_dump();
 unsigned get_jit_profiling_flags();
 std::string get_jit_profiling_jitdumpdir();
 FILE *fopen(const char *filename, const char *mode);
 int getpagesize();
 
+// return current library fpmath_mode
+fpmath_mode_t get_fpmath_mode();
+// checks if an fpmath_mode is valid
+status_t check_fpmath_mode(fpmath_mode_t mode);
+// Returns true if values reprensented by type sub_dt can all be
+// represented in dt. return false eotherwise
+bool is_fpsubtype(data_type_t sub_dt, data_type_t dt);
+
 constexpr int msan_enabled = MSAN_ENABLED;
 inline void msan_unpoison(void *ptr, size_t size) {
 #if MSAN_ENABLED
@@ -568,6 +616,93 @@ public:
     DNNL_DISALLOW_COPY_AND_ASSIGN(setting_t);
 };
 
+// The following code is derived from Boost C++ library
+// Copyright 2005-2014 Daniel James.
+// Distributed under the Boost Software License, Version 1.0. (See accompanying
+// file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+template <typename T>
+static size_t hash_combine(size_t seed, const T &v) {
+    return seed ^= std::hash<T> {}(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
+}
+
+inline int float2int(float x) {
+    return utils::bit_cast<int>(x);
+}
+
+// XXX: Currently SYCL doesn't provide an API to get device UUID but
+// we need to be able to distinguish OpenCL device from Level0 device.
+// As a temporary solution the compound ID will be used for that.
+// Below is a table explaning what the numbers are for different backends:
+//
+// -------------------------------------------------------------
+//  Backend      | Compound ID
+// -------------------------------------------------------------
+//  Host         | <backend_t::host, 0, 0>
+//  OpenCL       | <backend_t::opencl, cl_device, 0>
+//  NVIDIA       | <backend_t::nvidia, cuDevice, 0>
+//  Level0       | <backend_t::level0, uuid[0-63], uuid[64-127]>
+//  Pure CPU     | <0, 0, 0>
+//  Pure GPU     | <0, cl_device, 0>
+using device_id_t = std::tuple<int, uint64_t, uint64_t>;
+
+struct device_id_hash_t {
+    size_t operator()(const device_id_t &id) const {
+        size_t result = 0;
+        result = hash_combine(result, std::get<0>(id));
+        result = hash_combine(result, std::get<1>(id));
+        result = hash_combine(result, std::get<2>(id));
+        return result;
+    }
+};
+
+// A setting (basically a value) that can be set() multiple times until the
+// time first time the get() method is called. The set() method is expected to
+// be as expensive as a busy-waiting spinlock. The get() method is expected to
+// be asymptotically as expensive as a single lock-prefixed memory read. The
+// get() method also has a 'soft' mode when the setting is not locked for
+// re-setting. This is used for testing purposes.
+template <typename T>
+struct set_once_before_first_get_setting_t {
+private:
+    T value_;
+    std::atomic<unsigned> state_;
+    enum : unsigned { idle = 0, busy_setting = 1, locked = 2 };
+
+public:
+    set_once_before_first_get_setting_t(T init)
+        : value_ {init}, state_ {idle} {}
+
+    bool set(T new_value) {
+        if (state_.load() == locked) return false;
+
+        while (true) {
+            unsigned expected = idle;
+            if (state_.compare_exchange_weak(expected, busy_setting)) break;
+            if (expected == locked) return false;
+        }
+
+        value_ = new_value;
+        state_.store(locked);
+        return true;
+    }
+
+    T get(bool soft = false) {
+        if (!soft && state_.load() != locked) {
+            while (true) {
+                unsigned expected = idle;
+                if (state_.compare_exchange_weak(expected, locked)) break;
+                if (expected == locked) break;
+            }
+        }
+        return value_;
+    }
+};
+
+inline bool is_native_runtime(runtime_kind_t kind) {
+    return utils::one_of(kind, runtime_kind::seq, runtime_kind::omp,
+            runtime_kind::tbb, runtime_kind::threadpool);
+}
+
 } // namespace impl
 } // namespace dnnl
 
