@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 
 #include <atomic>
 
-#include "dnnl_types.h"
+#include "oneapi/dnnl/dnnl_types.h"
 
 #include "common/bfloat16.hpp"
 #include "common/c_types_map.hpp"
@@ -24,6 +24,7 @@
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 #include "cpu/x64/gemm_bf16_convolution.hpp"
+#include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -71,16 +72,11 @@ void cvt_acc_to_dst(const conv_gemm_conf_t &jcp, size_t g_start, size_t g_end,
 template <data_type_t dst_data_type>
 gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::pp_ker_t(const pd_t *pd)
     : jcp_(pd->jcp_)
-    , OC_(pd->jcp_.oc)
-    , do_bias_(false)
-    , do_eltwise_(false)
-    , do_sum_(false)
+    , do_sum_(dst_data_type != data_type::f32 && jcp_.with_sum)
     , max_data_reg_idx_(31)
     , max_unroll_(12)
     , compute_reg_step_(1)
-    , data_reg_base_idx_(0)
-    , bf16_emu_(nullptr)
-    , eltwise_injector_(nullptr) {
+    , data_reg_base_idx_(0) {
     using namespace types;
     using namespace Xbyak;
 
@@ -88,23 +84,37 @@ gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::pp_ker_t(const pd_t *pd)
         // bf16 is not supported
         return;
 
-    auto &post_ops = pd->attr()->post_ops_;
-    const int eltwise_ind = post_ops.find(primitive_kind::eltwise);
-    do_eltwise_ = eltwise_ind != -1;
-    if (do_eltwise_)
-        eltwise_injector_ = new jit_uni_eltwise_injector_f32<avx512_core>(this,
-                post_ops.entry_[eltwise_ind].eltwise, true,
-                reserved_eltwise_gpr, reserved_eltwise_maskr);
+    const auto &post_ops = jcp_.post_ops;
+    if (jcp_.with_eltwise || jcp_.with_binary) {
+#define PARAM_OFF(field) offsetof(ker_args, field)
+        static constexpr bool preserve_gpr = true;
+        static constexpr bool preserve_vmm = true;
+        static constexpr size_t helper_vmm_idx = 31;
+        static constexpr size_t tail_size = 1;
+        static constexpr bool use_exact_tail_scalar_bcast = false;
+        const binary_injector::rhs_arg_static_params_t rhs_arg_static_params {
+                helper_vmm_idx, reserved_eltwise_gpr, r14, preserve_gpr,
+                preserve_vmm, PARAM_OFF(post_ops_binary_rhs_arg_vec),
+                memory_desc_wrapper(pd->dst_md()), tail_size, kreg_rem_mask,
+                use_exact_tail_scalar_bcast};
+        const binary_injector::static_params_t binary_static_params {
+                this->reg_param, rhs_arg_static_params};
+        static constexpr bool save_state = true;
+        const eltwise_injector::static_params_t eltwise_static_params {
+                save_state, reserved_eltwise_gpr, reserved_eltwise_maskr};
+
+        postops_injector_ = utils::make_unique<
+                injector::jit_uni_postops_injector_t<avx512_core>>(
+                this, post_ops, binary_static_params, eltwise_static_params);
+#undef PARAM_OFF
+    }
 
-    do_sum_ = dst_data_type != data_type::f32
-            && post_ops.contain(primitive_kind::sum, 0);
     if (do_sum_) {
         compute_reg_step_ = 2;
         vreg_sum_scale = Zmm(data_reg_base_idx_++);
     }
 
-    do_bias_ = pd->with_bias();
-    if (do_bias_) vreg_bias = Zmm(data_reg_base_idx_++);
+    if (jcp_.with_bias) vreg_bias = Zmm(data_reg_base_idx_++);
 
     vlen_ = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
 
@@ -113,15 +123,50 @@ gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::pp_ker_t(const pd_t *pd)
 
     if (isa_ != avx512_core_bf16) {
         max_data_reg_idx_ = 26;
-        bf16_emu_ = new bf16_emulation_t(this, bf16_emu_reserv_1,
-                bf16_emu_reserv_2, bf16_emu_reserv_3, bf16_emu_reserv_4,
-                bf16_emu_reserv_5, bf16_emu_reserv_6);
+        bf16_emu_ = utils::make_unique<bf16_emulation_t>(this,
+                bf16_emu_reserv_1, bf16_emu_reserv_2, bf16_emu_reserv_3,
+                bf16_emu_reserv_4, bf16_emu_reserv_5, bf16_emu_reserv_6);
     }
 
     max_unroll_
             = (max_data_reg_idx_ - data_reg_base_idx_ + 1) / compute_reg_step_;
 }
 
+template <data_type_t dst_data_type>
+void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::apply_postops(
+        const bool apply_mask, const int out_offset, const int vmm_idx) {
+#define PARAM_OFF(x) offsetof(ker_args, x)
+    if (jcp_.with_eltwise || jcp_.with_binary) {
+        static constexpr int offset = 0;
+        if (jcp_.with_binary) {
+            binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
+            rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
+                    vmm_idx, ptr[reg_param + PARAM_OFF(g_oc_offset)]);
+            rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(vmm_idx, offset);
+            rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
+                    vmm_idx, oc_off_oprnd);
+            rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
+                    vmm_idx, out_offset);
+            rhs_arg_params.vmm_idx_to_out_off_oprnd.emplace(
+                    vmm_idx, out_off_oprnd);
+            if (apply_mask) rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
+
+            const injector_utils::register_preserve_guard_t register_guard(
+                    this, {oc_off_oprnd, out_off_oprnd});
+            mov(oc_off_oprnd,
+                    ptr[rsp + reg_binary_post_op_acc_off
+                            + register_guard.stack_space_occupied()]);
+            mov(out_off_oprnd, reg_dst);
+            sub(out_off_oprnd, ptr[reg_param + PARAM_OFF(dst_orig)]);
+            shr(out_off_oprnd, std::log2(sizeof(dst_data_t)));
+
+            postops_injector_->compute_vector(vmm_idx, rhs_arg_params);
+        } else
+            postops_injector_->compute_vector(vmm_idx);
+    }
+#undef PARAM_OFF
+}
+
 template <data_type_t dst_data_type>
 void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
     using namespace Xbyak;
@@ -129,15 +174,26 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
 
     preamble();
 
+#ifdef _WIN32
+    mov(reg_param, rcx);
+#endif
+
 #define PARAM_OFF(x) offsetof(ker_args, x)
     mov(reg_dst_base, ptr[reg_param + PARAM_OFF(dst)]);
     mov(reg_acc_base, ptr[reg_param + PARAM_OFF(acc)]);
-    if (do_bias_) mov(reg_bias, ptr[reg_param + PARAM_OFF(bias)]);
+    if (jcp_.with_bias) mov(reg_bias, ptr[reg_param + PARAM_OFF(bias)]);
     mov(reg_dst_str, ptr[reg_param + PARAM_OFF(dst_stride_in_bytes)]);
     mov(reg_acc_str, ptr[reg_param + PARAM_OFF(acc_stride_in_bytes)]);
     mov(reg_len, ptr[reg_param + PARAM_OFF(spatial_length)]);
     mov(reg_oc_iter, ptr[reg_param + PARAM_OFF(oc_work)]);
 
+    if (jcp_.with_binary) {
+        // zero initialize binary post_ops offset accumulator (store on stack)
+        const auto binary_post_op_acc_off_reg = reg_tmp;
+        xor_(binary_post_op_acc_off_reg, binary_post_op_acc_off_reg);
+        push(binary_post_op_acc_off_reg);
+    }
+
     if (do_sum_)
         vbroadcastss(vreg_sum_scale, ptr[reg_param + PARAM_OFF(sum_scale)]);
 #undef PARAM_OFF
@@ -154,7 +210,7 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
         if (apply_mask) vreg_dst_ = vreg_dst_ | kreg_rem_mask;
         vmovups(vreg_dst_, acc_addr);
 
-        if (do_bias_) vaddps(vreg_dst(idx), vreg_dst(idx), vreg_bias);
+        if (jcp_.with_bias) vaddps(vreg_dst(idx), vreg_dst(idx), vreg_bias);
 
         auto dst_addr = ptr[reg_dst + offset * sizeof(dst_data_t)];
         if (do_sum_) {
@@ -177,7 +233,7 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
             vfmadd231ps(vreg_dst(idx), vreg_prev_dst(idx), vreg_sum_scale);
         }
 
-        if (do_eltwise_) eltwise_injector_->compute_vector(vreg_dst_idx(idx));
+        apply_postops(apply_mask, offset, vreg_dst_idx(idx));
 
         if (dst_data_type == data_type::bf16) {
             // TODO: implement store by zmm registers for bf16
@@ -213,7 +269,7 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
     mov(reg_dst, reg_dst_base);
     mov(reg_acc, reg_acc_base);
 
-    if (do_bias_) vbroadcastss(vreg_bias, ptr[reg_bias]);
+    if (jcp_.with_bias) vbroadcastss(vreg_bias, ptr[reg_bias]);
 
     constexpr int n_unroll = default_unroll_2_pow_; // unroll by powers of 2
             // from 2^n to 2^0
@@ -249,23 +305,29 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::generate() {
 
     add(reg_dst_base, reg_dst_str);
     add(reg_acc_base, reg_acc_str);
-    if (do_bias_) add(reg_bias, sizeof(acc_data_t));
+    if (jcp_.with_bias) add(reg_bias, sizeof(acc_data_t));
+    if (jcp_.with_binary)
+        inc(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off));
 
     dec(reg_oc_iter);
     jnz(oc_loop, T_NEAR); // oc_loop end
 
     L(oc_loop_end);
 
+    if (jcp_.with_binary) add(rsp, stack_space_needed);
+
     postamble();
 
-    if (do_eltwise_) eltwise_injector_->prepare_table();
+    if (jcp_.with_eltwise) postops_injector_->prepare_table();
 }
 
 // operator () specialized for nspc format
 template <data_type_t dst_data_type>
 void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::operator()(
         dst_data_t *dst, const acc_data_t *acc, const acc_data_t *bias,
-        float sum_scale, size_t oc_work) {
+        float sum_scale, size_t oc_work,
+        const void *post_ops_binary_rhs_arg_vec, const void *dst_orig,
+        const size_t g_oc_offset) {
 
     ker_args args;
     args.acc = acc;
@@ -276,6 +338,10 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::operator()(
     args.acc_stride_in_bytes = sizeof(acc_data_t);
     args.spatial_length = 1;
     args.oc_work = oc_work;
+
+    args.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
+    args.dst_orig = dst_orig;
+    args.g_oc_offset = g_oc_offset;
     jit_generator::operator()(&args);
 }
 
@@ -284,7 +350,9 @@ template <data_type_t dst_data_type>
 void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::operator()(
         dst_data_t *dst, const acc_data_t *acc, const acc_data_t *bias,
         float sum_scale, size_t dst_stride_in_elements,
-        size_t acc_stride_in_elements, size_t sp_len, size_t oc_len) {
+        size_t acc_stride_in_elements, size_t sp_len, size_t oc_len,
+        const void *post_ops_binary_rhs_arg_vec, const void *dst_orig,
+        const size_t g_oc_offset) {
     if (sp_len == 0) return;
 
     ker_args args;
@@ -296,6 +364,10 @@ void gemm_bf16_convolution_fwd_t<dst_data_type>::pp_ker_t::operator()(
     args.acc_stride_in_bytes = acc_stride_in_elements * sizeof(acc_data_t);
     args.spatial_length = sp_len;
     args.oc_work = oc_len;
+
+    args.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
+    args.dst_orig = dst_orig;
+    args.g_oc_offset = g_oc_offset;
     jit_generator::operator()(&args);
 }
 
@@ -305,6 +377,9 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_nspc(
     auto src_base = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto wei_base = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto dst_base = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(
+                    this->pd()->attr()->post_ops_, ctx);
 
     auto scratchpad = ctx.get_scratchpad_grantor();
     const conv_gemm_conf_t &jcp = pd()->jcp_;
@@ -325,8 +400,9 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_nspc(
 
     std::atomic<status_t> st(status::success);
     parallel(jcp.nthr, [&](const int ithr, const int nthr) {
-        status_t st_thr = execute_forward_thr_nspc(
-                ithr, nthr, src_base, wei_base, bia_base, dst_base, scratchpad);
+        status_t st_thr = execute_forward_thr_nspc(ithr, nthr, src_base,
+                wei_base, bia_base, dst_base, scratchpad,
+                post_ops_binary_rhs_arg_vec.data());
         if (st_thr != status::success) st = st_thr;
     });
 
@@ -337,7 +413,8 @@ template <data_type_t dst_data_type>
 status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_thr_nspc(
         const int ithr, const int nthr, const src_data_t *src_base,
         const wei_data_t *wei_base, const float *bia_base, dst_data_t *dst_base,
-        const memory_tracking::grantor_t &scratchpad) const {
+        const memory_tracking::grantor_t &scratchpad,
+        const void *post_ops_binary_rhs_arg_vec) const {
     const conv_gemm_conf_t &jcp = pd()->jcp_;
 
     // Src Format: mb-spatial-groups-input_channels
@@ -364,17 +441,17 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_thr_nspc(
     const bool do_sum = post_ops.contain(primitive_kind::sum, 0);
     const float sum_scale = do_sum ? post_ops.entry_[0].sum.scale : 0;
 
-    int g {0}, n {0}, ohb {0}, owb {0};
-    size_t start = 0, end = 0;
+    dim_t g {0}, n {0}, ohb {0}, owb {0};
+    dim_t start = 0, end = 0;
 
     const bool is_problem_3d = pd()->ndims() == 5;
     assert(IMPLICATION(is_problem_3d,
             jcp.oh_block == jcp.oh && jcp.ow_block == jcp.ow
                     && jcp.ic_block == jcp.ic));
 
-    const int nb_oh = div_up(jcp.oh, jcp.oh_block);
-    const int nb_ow = div_up(jcp.ow, jcp.ow_block);
-    const size_t work_amount = (size_t)jcp.ngroups * jcp.mb * nb_oh * nb_ow;
+    const dim_t nb_oh = div_up(jcp.oh, jcp.oh_block);
+    const dim_t nb_ow = div_up(jcp.ow, jcp.ow_block);
+    const dim_t work_amount = jcp.ngroups * jcp.mb * nb_oh * nb_ow;
     balance211(work_amount, nthr, ithr, start, end);
     nd_iterator_init(start, n, jcp.mb, g, jcp.ngroups, ohb, nb_oh, owb, nb_ow);
 
@@ -390,7 +467,7 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_thr_nspc(
         for (ptrdiff_t i = 0; i < jcp.im2col_sz; i++)
             col_r[i] = zero_val;
     }
-    for (size_t iwork = start; iwork < end; ++iwork) {
+    for (dim_t iwork = start; iwork < end; ++iwork) {
         int oh = ohb * jcp.oh_block;
         int ow = owb * jcp.ow_block;
         const src_data_t *__restrict src
@@ -447,7 +524,9 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_thr_nspc(
 
                             (*pp_ker_)(dst_arr,
                                     acc_needed ? acc_arr : (float *)dst_arr,
-                                    bia_arr, sum_scale, jcp.oc);
+                                    bia_arr, sum_scale, jcp.oc,
+                                    post_ops_binary_rhs_arg_vec, dst_base,
+                                    g * jcp.oc);
                         });
             }
         }
@@ -462,6 +541,9 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(
+                    this->pd()->attr()->post_ops_, ctx);
 
     bool is_bf16_dst = dst_data_type == data_type::bf16;
 
@@ -498,7 +580,7 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
     const size_t weights_oc_size = jcp.ic * jcp.ks;
 
     const dim_t LDB = weights_oc_size;
-    const size_t work_amount
+    const dim_t work_amount
             = (size_t)jcp.ngroups * jcp.mb * jcp.od * jcp.os_nb_block;
     const bool is_problem_3d = pd()->ndims() == 5;
     std::atomic<status_t> st(status::success);
@@ -506,8 +588,8 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
     auto inner_ker = [&](const int ic, const int oc, const int groups,
                              const int od, const int spatial,
                              const src_data_t *src, const wei_data_t *weights,
-                             src_data_t *col, dst_data_t *dst, acc_data_t *acc,
-                             int ic_block, int oc_block) {
+                             src_data_t *col, dst_data_t *dst_im,
+                             acc_data_t *acc, int ic_block, int oc_block) {
         const dim_t os_block = nstl::min(
                 (dim_t)jcp.os_block, (dim_t)jcp.os - spatial * jcp.os_block);
 
@@ -530,7 +612,7 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
         const dim_t LDC = is_bf16_dst ? m : M;
         const float beta = (ic == 0) ? this->beta_ : one;
         auto out_off = spatial * jcp.os_block + od * jcp.os;
-        dst_data_t *dst_local = dst + out_off;
+        dst_data_t *dst_local = dst_im + out_off;
 
         status_t st_thr = gemm_bf16bf16f32("N", "N", &m, &N, &K, &one,
                 jcp.im2col_sz ? col : src + ic * M + out_off, &LDA, weights,
@@ -541,11 +623,13 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
             return;
         }
 
-        if (this->pd()->is_postprocess_required()) {
+        if (this->pd()->is_postprocess_required() && ic + ic_block >= jcp.ic) {
             size_t acc_str = LDC;
             size_t dst_str = M;
-            (*pp_ker_)(dst_local, acc, bias + groups * jcp.oc + oc, sum_scale,
-                    dst_str, acc_str, m, oc_block);
+            float *bias_ptr = bias ? bias + groups * jcp.oc + oc : nullptr;
+            (*pp_ker_)(dst_local, acc, bias_ptr, sum_scale, dst_str, acc_str, m,
+                    oc_block, post_ops_binary_rhs_arg_vec.data(), dst,
+                    groups * jcp.oc + oc);
         }
     };
 
@@ -557,19 +641,20 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
             for (ptrdiff_t i = 0; i < jcp.im2col_sz; i++)
                 _col[i] = (src_data_t)0;
         }
-        int g {0}, n {0}, od {0}, nb_os {0};
-        size_t start = 0, end = 0;
-        size_t oc_start = 0, oc_end = 0;
+        dim_t g {0}, n {0}, od {0}, nb_os {0};
+        dim_t start = 0, end = 0;
+        dim_t oc_start = 0, oc_end = 0;
 
         assert(jcp.loop_order == gemm_loop_lbr);
-        balance2D(nthr, ithr, work_amount, start, end, (size_t)jcp.oc, oc_start,
-                oc_end, (size_t)jcp.nthr_oc);
+        balance2D(nthr, ithr, work_amount, start, end, jcp.oc, oc_start, oc_end,
+                dim_t(jcp.nthr_oc));
 
         nd_iterator_init(start, g, jcp.ngroups, n, jcp.mb, od, jcp.od, nb_os,
                 jcp.os_nb_block);
-        for (size_t iwork = start; iwork < end; ++iwork) {
-            for_(int oc = (int)oc_start; oc < (int)oc_end; oc += jcp.oc_block)
-            for (int ic = 0; ic < jcp.ic; ic += jcp.ic_block) {
+        for (dim_t iwork = start; iwork < end; ++iwork) {
+            for_(dim_t oc = (dim_t)oc_start; oc < (dim_t)oc_end;
+                    oc += jcp.oc_block)
+            for (dim_t ic = 0; ic < jcp.ic; ic += jcp.ic_block) {
                 const src_data_t *_src = src + (n * jcp.ngroups + g) * src_step;
                 const wei_data_t *_weights = weights + g * weights_g_size
                         + oc * weights_oc_size + ic * jcp.ks;
@@ -577,15 +662,16 @@ status_t gemm_bf16_convolution_fwd_t<dst_data_type>::execute_forward_ncsp(
                         = dst + (n * jcp.ngroups + g) * dst_step + oc * M;
                 auto out_off = nb_os * jcp.os_block + od * jcp.os;
                 dst_data_t *dst_local = _dst_im + out_off;
-                const int sizeof_cacheline_float = 16;
+                const dim_t sizeof_cacheline_float = 16;
                 acc_data_t *_acc = is_bf16_dst ? acc_base
                                 + ithr
                                         * rnd_up(jcp.oc_block * jcp.os_block,
                                                 sizeof_cacheline_float)
                                                : (acc_data_t *)dst_local;
 
-                const int ic_block = nstl::min(jcp.ic - ic, jcp.ic_block);
-                const int oc_block = nstl::min(int(oc_end) - oc, jcp.oc_block);
+                const dim_t ic_block = nstl::min(jcp.ic - ic, jcp.ic_block);
+                const dim_t oc_block
+                        = nstl::min(dim_t(oc_end) - oc, jcp.oc_block);
 
                 inner_ker(ic, oc, g, od, nb_os, _src, _weights, _col, _dst_im,
                         _acc, ic_block, oc_block);
@@ -643,20 +729,20 @@ status_t gemm_bf16_convolution_bwd_data_t<
     const size_t diff_src_os_stride = jcp.ngroups * jcp.ic;
 
     // threads share work across mini-batch and groups
-    const size_t work_amount = jcp.ngroups * jcp.mb;
+    const dim_t work_amount = jcp.ngroups * jcp.mb;
 
     acc_data_t *__restrict col = scratchpad.get<acc_data_t>(key_conv_gemm_col)
             + (ptrdiff_t)ithr * jcp.im2col_sz;
     acc_data_t *__restrict acc = scratchpad.get<acc_data_t>(key_conv_gemm_acc)
             + (ptrdiff_t)ithr * jcp.is * jcp.id * jcp.ic;
 
-    int n {0}, g {0};
-    size_t start = 0, end = 0;
+    dim_t n {0}, g {0};
+    dim_t start = 0, end = 0;
 
     balance211(work_amount, nthr, ithr, start, end);
     nd_iterator_init(start, n, jcp.mb, g, jcp.ngroups);
 
-    for (size_t iwork = start; iwork < end; ++iwork) {
+    for (dim_t iwork = start; iwork < end; ++iwork) {
         const diff_dst_data_t *__restrict diff_dst = diff_dst_base
                 + n * diff_dst_mb_stride + g * diff_dst_g_stride;
         const wei_data_t *__restrict wei = wei_base + g * wei_g_stride;
@@ -737,7 +823,7 @@ status_t gemm_bf16_convolution_bwd_data_t<diff_src_data_type>::
     const dim_t K = jcp.oc;
     const dim_t N = jcp.ic * jcp.ks;
 
-    const size_t work_amount = (size_t)jcp.ngroups * jcp.mb;
+    const dim_t work_amount = (size_t)jcp.ngroups * jcp.mb;
     const bool is_problem_3d = pd()->ndims() == 5;
 
     std::atomic<status_t> st(status::success);
@@ -745,11 +831,11 @@ status_t gemm_bf16_convolution_bwd_data_t<diff_src_data_type>::
     parallel(jcp.nthr, [&](const int ithr, const int nthr) {
         acc_data_t *_col = col + (ptrdiff_t)ithr * jcp.im2col_sz;
 
-        int g {0}, n {0};
-        size_t start = 0, end = 0;
+        dim_t g {0}, n {0};
+        dim_t start = 0, end = 0;
         balance211(work_amount, nthr, ithr, start, end);
         nd_iterator_init(start, g, jcp.ngroups, n, jcp.mb);
-        for (size_t iwork = start; iwork < end; ++iwork) {
+        for (dim_t iwork = start; iwork < end; ++iwork) {
 
             diff_src_data_t *diff_src_local
                     = diff_src + (n * jcp.ngroups + g) * src_step;
@@ -815,10 +901,9 @@ void gemm_bf16_convolution_bwd_weights_t<
     assert(nthr_mb > 1); // no reduction for nthr_mb == 1
 
     const bool is_bf16_out = diff_wei_data_type == data_type::bf16;
-    const size_t weights_g_size = jcp.oc;
-    size_t weights_start {0}, weights_end {0};
-    balance211(size_t(jcp.ks) * jcp.ic, nthr_mb, ithr_mb, weights_start,
-            weights_end);
+    const dim_t weights_g_size = jcp.oc;
+    dim_t weights_start {0}, weights_end {0};
+    balance211(jcp.ks * jcp.ic, nthr_mb, ithr_mb, weights_start, weights_end);
 
     for (auto tidx = 1; tidx < nthr_mb; ++tidx) {
         const acc_data_t *ws_base
@@ -1061,20 +1146,20 @@ status_t gemm_bf16_convolution_bwd_weights_t<diff_wei_data_type>::
     }
 
     if (jcp.with_bias) {
-        parallel_nd(jcp.ngroups, jcp.oc, [&](int g, int oc) {
+        parallel_nd(jcp.ngroups, jcp.oc, [&](dim_t g, dim_t oc) {
             acc_data_t db = 0;
-            const size_t offset_base = g * jcp.oc + oc;
-            for_(int mb = 0; mb < jcp.mb; ++mb)
-            for_(int od = 0; od < jcp.od; ++od)
-            for (int oh = 0; oh < jcp.oh; ++oh) {
-                const int width_stride = jcp.ngroups * jcp.oc;
+            const dim_t offset_base = g * jcp.oc + oc;
+            for_(dim_t mb = 0; mb < jcp.mb; ++mb)
+            for_(dim_t od = 0; od < jcp.od; ++od)
+            for (dim_t oh = 0; oh < jcp.oh; ++oh) {
+                const dim_t width_stride = jcp.ngroups * jcp.oc;
                 const diff_dst_data_t *__restrict diff_dst_arr = diff_dst
                         + offset_base
                         + ((mb * jcp.od + od) * jcp.oh + oh) * jcp.ow
                                 * width_stride;
 
                 PRAGMA_OMP_SIMD(reduction(+ : db))
-                for (int ow = 0; ow < jcp.ow; ++ow) {
+                for (dim_t ow = 0; ow < jcp.ow; ++ow) {
                     db += diff_dst_arr[ow * width_stride];
                 }
             }
@@ -1263,17 +1348,18 @@ status_t gemm_bf16_convolution_bwd_weights_t<diff_wei_data_type>::
     }
 
     if (jcp.with_bias) {
-        parallel_nd(jcp.ngroups, jcp.oc, [&](int g, int oc) {
+        parallel_nd(jcp.ngroups, jcp.oc, [&](size_t g, size_t oc) {
             acc_data_t db = 0;
-            size_t offset_ = (size_t)g * dst_step + (size_t)oc * K;
-            for (int mb = 0; mb < jcp.mb; ++mb) {
-                size_t offset = offset_ + (size_t)mb * jcp.ngroups * dst_step;
-                for_(int od = 0; od < jcp.od; ++od)
-                for (int oh = 0; oh < jcp.oh; ++oh)
+            dim_t offset_ = g * dst_step + oc * K;
+            for (dim_t mb = 0; mb < jcp.mb; ++mb) {
+                dim_t offset = offset_ + mb * jcp.ngroups * dst_step;
+                for_(dim_t od = 0; od < jcp.od; ++od)
+                for (dim_t oh = 0; oh < jcp.oh; ++oh) {
                     PRAGMA_OMP_SIMD(reduction(+ : db))
-                for (int ow = 0; ow < jcp.ow; ++ow) {
-                    db += diff_dst[offset];
-                    offset++;
+                    for (dim_t ow = 0; ow < jcp.ow; ++ow) {
+                        db += diff_dst[offset];
+                        offset++;
+                    }
                 }
             }
             diff_bias[g * jcp.oc + oc] = db;
