@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -48,6 +48,12 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward(
             const data_t *, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
     auto bias_dw = CTX_IN_MEM(
             const data_t *, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
+    const auto post_ops_binary_rhs_arg_vec_dw = pd()->jcp_dw_
+            ? binary_injector::prepare_binary_args(pd()->jcp_dw_->post_ops, ctx,
+                    pd()->jcp_.post_ops.entry_.size() + 1)
+            : std::vector<const void *> {};
 
     auto scratchpad = ctx.get_scratchpad_grantor();
 
@@ -65,17 +71,19 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward(
 
     parallel(0, [&](const int ithr, const int nthr) {
         execute_forward_thr(ithr, nthr, src, weights, bias, weights_dw, bias_dw,
-                dst, scratchpad);
+                dst, scratchpad, post_ops_binary_rhs_arg_vec.data(),
+                post_ops_binary_rhs_arg_vec_dw.data());
     });
 
-    if (pd()->wants_zero_pad_dst())
-        ctx.memory(DNNL_ARG_DST)->zero_pad(ctx.stream());
+    if (pd()->wants_zero_pad_dst()) ctx.zero_pad_output(DNNL_ARG_DST);
 }
 
 void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         const int nthr, const data_t *src, const data_t *weights,
         const data_t *bias, const data_t *weights_dw, const data_t *bias_dw,
-        data_t *dst, const memory_tracking::grantor_t &scratchpad) const {
+        data_t *dst, const memory_tracking::grantor_t &scratchpad,
+        const void *post_ops_binary_rhs_arg_vec,
+        const void *post_ops_binary_rhs_arg_vec_dw) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper dst_d(pd()->dst_md());
@@ -153,8 +161,11 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
 
     auto init_load = [&](int ocb, int ocb_end, int &load_step) {
         load_step = step(nb_load_blocking, ocb_end - ocb, nb_load_blocking_max);
+        // binary postop injector may override zero-padded areas, so proper
+        // output masking needs to be performed base on exact number of channels
+        const auto oc = jcp.with_binary ? jcp.oc_without_padding : jcp.oc;
         p.load_dim = this_block_size(
-                ocb * jcp.oc_block, jcp.oc, load_step * jcp.oc_block);
+                ocb * jcp.oc_block, oc, load_step * jcp.oc_block);
     };
 
     auto ker_1x1 = [&](int ocb, int icb, int ocb_start, int n, int g, int od,
@@ -202,6 +213,10 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         } else
             p.bcast_data = src + data_blk_off(src_d, n, ic_off_idx, id, ih, iw);
 
+        p.oc_l_off = ocb * jcp.oc_block;
+        p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
+        p.dst_orig = dst;
+
         (*kernel_)(&p);
     };
 
@@ -268,7 +283,13 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
 
             par_conv_dw.kh_padding = (size_t)nstl::max(0, kh_padding);
 
-            par_conv_dw.ch_blocks = nstl::min(ch + ch_num, jcp_dw->nb_ch) - ch;
+            par_conv_dw.load_work = (nstl::min(ch + ch_num, jcp_dw->nb_ch) - ch)
+                    * jcp_dw->ch_block;
+
+            par_conv_dw.oc_l_off = ch * jcp_dw->ch_block;
+            par_conv_dw.post_ops_binary_rhs_arg_vec
+                    = post_ops_binary_rhs_arg_vec_dw;
+            par_conv_dw.dst_orig = dst;
 
             (*dw_jit_ker)(&par_conv_dw);
 
@@ -467,7 +488,8 @@ void jit_avx2_1x1_convolution_bwd_data_t::execute_backward_data(
 
 status_t jit_avx2_1x1_convolution_bwd_weights_t::init(engine_t *engine) {
     CHECK(safe_ptr_assign(kernel_,
-            new jit_avx2_1x1_conv_kernel_f32(pd()->jcp_, *pd()->attr())));
+            new jit_avx2_1x1_conv_kernel_f32(
+                    pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
     CHECK(kernel_->create_kernel());
 
     CHECK(safe_ptr_assign(reducer_weights_,
@@ -634,6 +656,29 @@ void jit_avx2_1x1_convolution_bwd_weights_t::execute_backward_weights(
         }
     };
 
+    auto maybe_zero_icpad = [&](const int g_start, const int g_end,
+                                    const int ocb_start, const int ocb_end) {
+        // write zeros to IC padded region.
+        const int ic_tail = jcp.ic_without_padding % jcp.ic_block;
+        if (is_ddst_layout_nxc && ic_tail != 0) {
+            for_(int g = g_start; g < g_end; ++g)
+            for (int z_ocb = ocb_start; z_ocb < ocb_end; ++z_ocb) {
+                const int z_icb = nb_ic - 1;
+                const size_t off = pd()->with_groups()
+                        ? diff_weights_d.blk_off(g, z_ocb, z_icb)
+                        : diff_weights_d.blk_off(z_ocb, z_icb);
+                data_t *z_wei = diff_weights + off + ic_tail * jcp.oc_block;
+                const int zero_work
+                        = (nb_ic * jcp.ic_block - jcp.ic_without_padding)
+                        * jcp.oc_block;
+                PRAGMA_OMP_SIMD()
+                for (int o = 0; o < zero_work; ++o) {
+                    z_wei[o] = 0;
+                }
+            }
+        }
+    };
+
     auto ker = [&](const int ithr, const int nthr) {
         assert(nthr == rw->balancer().nthr_);
 
@@ -707,6 +752,11 @@ void jit_avx2_1x1_convolution_bwd_weights_t::execute_backward_weights(
                 img += 1;
             }
 
+            if (rw->balancer().nthr_per_group_ == 1
+                    && bcast_i + 1 >= bcast_work)
+                maybe_zero_icpad(g, g + 1, oc_b,
+                        nstl::min(nb_oc, oc_b + nb_oc_blocking));
+
             nd_iterator_step(
                     g, jcp.ngroups, load_i, load_work, bcast_i, bcast_work);
         }
