@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -29,8 +29,8 @@
 #include "gpu/ocl/rnn/ref_rnn.hpp"
 
 #include "common/c_types_map.hpp"
-#include "common/dnnl_thread.hpp"
 #include "common/dnnl_traits.hpp"
+#include "common/gemm_utils.hpp"
 #include "common/math_utils.hpp"
 #include "common/type_helpers.hpp"
 #include "gpu/gemm/gpu_gemm.hpp"
@@ -125,7 +125,7 @@ static status_t init_conf(rnn_conf_t &conf, const rnn_pd_t *rnn_pd,
     conf.is_training = rnn.is_training;
 
     conf.states_ws_ld = rnn.states_ws_ld;
-    conf.diff_states_ws_ld = rnn.diff_states_ws_ld;
+    conf.scratch_diff_states_ld = rnn.scratch_diff_states_ld;
     conf.gates_ws_ld = rnn.gates_ws_ld;
     conf.scratch_gates_ld = rnn.scratch_gates_ld;
 
@@ -178,10 +178,11 @@ static status_t init_conf(rnn_conf_t &conf, const rnn_pd_t *rnn_pd,
     }
 
     rnn_utils::set_offsets(rnn, conf.ws_gates_offset, conf.ws_states_offset,
-            conf.ws_c_state_offset, conf.ws_diff_states_offset,
-            conf.ws_grid_comp_offset, conf.scratch_cell_offset,
-            conf.ws_dhG1_offset, conf.ws_bias_offset, conf.scratch_gates_offset,
-            conf.scratchpad_size, conf.workspace_size);
+            conf.ws_c_state_offset, conf.ws_grid_comp_offset,
+            conf.ws_bias_offset, conf.scratch_diff_states_offset,
+            conf.scratch_cell_offset, conf.scratch_dhG1_offset,
+            conf.scratch_gates_offset, conf.scratchpad_size,
+            conf.workspace_size);
 
     conf.cell_kind = rnn_pd->cell_kind();
     conf.activation_kind = rnn_pd->activation_kind();
@@ -219,7 +220,6 @@ static status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("WS_GATES", rnn_utils::gates);
     kernel_ctx.define_int("WS_STATES", rnn_utils::states);
     kernel_ctx.define_int("WS_C_STATES", rnn_utils::c_states);
-    kernel_ctx.define_int("WS_DIFF_STATES", rnn_utils::diff_states);
     kernel_ctx.define_int("WS_BIAS", rnn_utils::bias);
 
     kernel_ctx.define_int("L2R", dnnl_unidirectional_left2right);
@@ -288,14 +288,13 @@ static status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("WS_GATES_OFFSET", conf.ws_gates_offset);
     kernel_ctx.define_int("WS_STATES_OFFSET", conf.ws_states_offset);
     kernel_ctx.define_int("WS_C_STATE_OFFSET", conf.ws_c_state_offset);
-    kernel_ctx.define_int("WS_DIFF_STATES_OFFSET", conf.ws_diff_states_offset);
     kernel_ctx.define_int("WS_GRID_COMP_OFFSET", conf.ws_grid_comp_offset);
-    kernel_ctx.define_int("SCRATCH_HDG1_OFFSET", conf.ws_dhG1_offset);
     kernel_ctx.define_int("WS_BIAS_OFFSET", conf.ws_bias_offset);
-    kernel_ctx.define_int("SCRATCH_GATES_OFFSET", conf.scratch_gates_offset);
+
     kernel_ctx.define_int("STATES_WS_LD", conf.states_ws_ld);
-    kernel_ctx.define_int("DIFF_STATES_WS_LD", conf.diff_states_ws_ld);
     kernel_ctx.define_int("GATES_WS_LD", conf.gates_ws_ld);
+    kernel_ctx.define_int(
+            "SCRATCH_DIFF_STATES_LD", conf.scratch_diff_states_ld);
     kernel_ctx.define_int("SCRATCH_GATES_LD", conf.scratch_gates_ld);
 
     if (conf.src_dt == data_type::f16) {
@@ -584,28 +583,25 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
     status_t status = init_conf<aprop>(conf, rnn_conf, this, this->off);
     if (status != status::success) { return status; }
 
+    // The inputs of create_gemm_pd describe a gemm in column major.
+    // Below, we have to transpose the a and b descriptor to describe
+    // the GEMM as a row major problem.
     auto create_gemm_pd
-            = [&](std::unique_ptr<primitive_desc_t> &gemm_pd, int m, int n,
+            = [&](std::shared_ptr<primitive_desc_t> &gemm_pd, int m, int n,
                       int k, int lda, int ldb, int ldc, data_type_t a_dt,
                       data_type_t b_dt, data_type_t c_dt, bool is_B_trans,
                       float beta) -> status_t {
         auto gemm_desc = gemm_desc_t();
+        memory_desc_t a_md, b_md, c_md;
+        create_2d_desc(&b_md, k, m, a_dt, transpose::notrans, lda);
+        create_2d_desc(&a_md, n, k, b_dt,
+                is_B_trans ? transpose::trans : transpose::notrans, ldb);
+        create_2d_desc(&c_md, n, m, c_dt, transpose::notrans, ldc);
         gemm_desc.primitive_kind = primitive_kind::gemm;
-        gemm_desc.transa = transpose::notrans;
-        gemm_desc.transb = is_B_trans ? transpose::trans : transpose::notrans;
-        gemm_desc.batch = 1;
-        gemm_desc.m = m;
-        gemm_desc.n = n;
-        gemm_desc.k = k;
-        gemm_desc.lda = lda;
-        gemm_desc.ldb = ldb;
-        gemm_desc.ldc = ldc;
-        gemm_desc.stride_a = lda;
-        gemm_desc.stride_b = ldb;
-        gemm_desc.stride_c = ldc;
-        gemm_desc.a_type = a_dt;
-        gemm_desc.b_type = b_dt;
-        gemm_desc.c_type = c_dt;
+        gemm_desc.a_desc = a_md;
+        gemm_desc.b_desc = b_md;
+        gemm_desc.c_desc = c_md;
+        gemm_desc.bias_desc = glob_zero_md;
         gemm_desc.acc_type = c_dt;
 
         primitive_attr_t attr;
@@ -613,8 +609,8 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
         dnnl_primitive_desc_iterator it(
                 engine, (op_desc_t *)&gemm_desc, &attr, nullptr);
         if (!it.is_initialized()) return status::out_of_memory;
-        ++it;
-        gemm_pd.reset(it.fetch_once());
+
+        gemm_pd = *(++it);
         if (!gemm_pd) return status::unimplemented;
         return status::success;
     };
@@ -668,41 +664,42 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
         case prop_kind::backward:
             gemm_ok = true
                     && utils::everyone_is(status::success,
-                            (rnn_conf.is_vanilla_gru
-                                            ? (create_gemm_pd(gemm_iter_bwd_pd_,
-                                                       sic, batch,
-                                                       (n_gates - 1) * dhc,
-                                                       rnn_conf.weights_iter_ld,
-                                                       rnn_conf.scratch_gates_ld,
-                                                       rnn_conf.diff_states_ws_ld,
-                                                       weights_type, src_type,
-                                                       rnn_conf.acc_data_type,
-                                                       false, 1.0f),
-                                                    create_gemm_pd(
-                                                            gemm_iter_bwd_2_pd_,
-                                                            sic, batch, dhc,
-                                                            rnn_conf.weights_iter_ld,
-                                                            rnn_conf.scratch_gates_ld,
-                                                            rnn_conf.diff_states_ws_ld,
-                                                            weights_type,
-                                                            src_type,
-                                                            rnn_conf.acc_data_type,
-                                                            false, 0.0f))
-                                            : create_gemm_pd(gemm_iter_bwd_pd_,
-                                                    sic, batch, n_gates * dhc,
-                                                    rnn_conf.weights_iter_ld,
-                                                    rnn_conf.scratch_gates_ld,
-                                                    rnn_conf.diff_states_ws_ld,
-                                                    weights_type, src_type,
-                                                    rnn_conf.acc_data_type,
-                                                    false, gemm_iter_bwd_beta)),
+                            (rnn_conf.is_vanilla_gru ? (
+                                     create_gemm_pd(gemm_iter_bwd_pd_, sic,
+                                             batch, (n_gates - 1) * dhc,
+                                             rnn_conf.weights_iter_ld,
+                                             rnn_conf.scratch_gates_ld,
+                                             rnn_conf.scratch_diff_states_ld,
+                                             weights_type, src_type,
+                                             rnn_conf.acc_data_type, false,
+                                             1.0f),
+                                     create_gemm_pd(gemm_iter_bwd_2_pd_, sic,
+                                             batch, dhc,
+                                             rnn_conf.weights_iter_ld,
+                                             rnn_conf.scratch_gates_ld,
+                                             rnn_conf.scratch_diff_states_ld,
+                                             weights_type, src_type,
+                                             rnn_conf.acc_data_type, false,
+                                             0.0f))
+                                                     : create_gemm_pd(
+                                                             gemm_iter_bwd_pd_,
+                                                             sic, batch,
+                                                             n_gates * dhc,
+                                                             rnn_conf.weights_iter_ld,
+                                                             rnn_conf.scratch_gates_ld,
+                                                             rnn_conf.scratch_diff_states_ld,
+                                                             weights_type,
+                                                             src_type,
+                                                             rnn_conf.acc_data_type,
+                                                             false,
+                                                             gemm_iter_bwd_beta)),
                             create_gemm_pd(gemm_layer_bwd_pd_, slc,
                                     layer_merged_size, n_gates * dhc,
                                     rnn_conf.weights_layer_ld,
                                     rnn_conf.scratch_gates_ld,
-                                    rnn_conf.diff_states_ws_ld, weights_type,
-                                    src_type, rnn_conf.acc_data_type, false,
-                                    0.0f),
+                                    rnn_conf.scratch_diff_states_ld,
+                                    weights_type, src_type,
+                                    rnn_conf.acc_data_type, false, 0.0f),
                             create_gemm_pd(gemm_diff_wei_layer_pd_,
                                     n_gates * dhc, slc, layer_merged_size,
                                     rnn_conf.scratch_gates_ld,
@@ -750,8 +747,56 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
 
 template <prop_kind_t aprop>
 status_t _ref_rnn_common_t<aprop>::init(engine_t *engine) {
-    compute::kernel_ctx_t kernel_ctx;
+    using namespace rnn_utils;
+    auto assign_funcs = [](gemm_t &g, weights_assign_t &p) {
+        g = &class_name::gemm_primitive;
+        p = &class_name::assign_weights;
+    };
 
+    assign_funcs(gemm_iter_func, weights_iter_assign_func);
+    assign_funcs(gemm_layer_func, weights_layer_assign_func);
+
+    switch (pd()->cell_kind()) {
+        case dnnl_vanilla_lstm:
+            cell_func = &class_name::cell_execution;
+            elemwise_common = pd()->src_type == data_type::u8
+                            && pd()->weights_type == data_type::s8
+                    ? &class_name::lstm_elemwise_u8s8
+                    : &class_name::lstm_elemwise;
+            break;
+        case dnnl_vanilla_rnn:
+            cell_func = &class_name::cell_execution;
+            elemwise_common = &class_name::rnn_elemwise;
+            break;
+        case dnnl_vanilla_gru:
+            cell_func = &class_name::cell_execution_gru;
+            elemwise_gru = &class_name::gru_elemwise;
+            break;
+        case dnnl_lbr_gru:
+            cell_func = &class_name::cell_execution_gru_lbr;
+            elemwise_gru_lbr = &class_name::gru_lbr_elemwise;
+            break;
+        default: break;
+    }
+
+    grid_computation = &class_name::linear_execution;
+
+    size_t scratchpad_size, workspace_size;
+    rnn_utils::set_offsets(pd()->rnn_conf, ws_gates_offset_, ws_states_offset_,
+            ws_c_states_offset_, ws_grid_comp_offset_, ws_bias_offset_,
+            scratch_diff_states_offset_, scratch_cell_offset_,
+            scratch_dhG1_offset_, scratch_gates_offset_, scratchpad_size,
+            workspace_size);
+    int max_nparts = (pd()->cell_kind() == alg_kind::vanilla_gru) ? 2 : 1;
+    int wei_offsets_iter_sz = pd()->L() * pd()->D() * max_nparts;
+    int wei_offsets_layer_sz = pd()->L() * pd()->D();
+
+    wei_layer_offset_ptr
+            = (size_t *)malloc(sizeof(size_t) * wei_offsets_layer_sz, 64);
+    wei_iter_offset_ptr
+            = (size_t *)malloc(sizeof(size_t) * wei_offsets_iter_sz, 64);
+
+    compute::kernel_ctx_t kernel_ctx;
     status_t status = init_kernel_ctx(kernel_ctx, pd()->conf, pd()->off);
     CHECK(status);
 
@@ -834,6 +879,7 @@ status_t _ref_rnn_common_t<aprop>::init(engine_t *engine) {
 
     return status::success;
 }
+
 template <prop_kind_t aprop>
 status_t _ref_rnn_common_t<aprop>::init_res_storage(
         engine_t *engine, gpu_resource_t *r) const {
@@ -845,7 +891,8 @@ status_t _ref_rnn_common_t<aprop>::init_res_storage(
         // copy bias to memory storage
         std::unique_ptr<memory_storage_t> tmp_mem_storage(tmp_mem_storage_ptr);
         void *scales_ptr = nullptr;
-        CHECK(tmp_mem_storage->map_data(&scales_ptr, nullptr));
+        CHECK(tmp_mem_storage->map_data(&scales_ptr, nullptr,
+                sizeof(float) * pd()->rnn_conf.n_gates * pd()->rnn_conf.dhc));
         utils::array_copy((float *)scales_ptr,
                 pd()->attr()->rnn_weights_qparams_.scales_,
                 pd()->rnn_conf.n_gates * pd()->rnn_conf.dhc);
@@ -865,7 +912,8 @@ status_t _ref_rnn_common_t<aprop>::init_res_storage(
 
         std::unique_ptr<memory_storage_t> tmp_mem_storage(tmp_mem_storage_ptr);
         void *tm_scales_ptr = nullptr;
-        CHECK(tmp_mem_storage->map_data(&tm_scales_ptr, nullptr));
+        CHECK(tmp_mem_storage->map_data(&tm_scales_ptr, nullptr,
+                sizeof(float) * pd()->attr()->rnn_tparams_.ngates_));
         utils::array_copy((float *)tm_scales_ptr,
                 pd()->attr()->rnn_tparams_.scales_,
                 pd()->attr()->rnn_tparams_.ngates_);
@@ -884,44 +932,49 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
     // different number of GEMMs.
     bool is_lbr = this->pd()->is_lbr();
     bool is_vanilla_gru = this->pd()->rnn_conf.is_vanilla_gru;
-
-    void *scratchpad_ptr {nullptr}, *scratch_gates_ptr {nullptr},
-            *scratch_cell_ptr {nullptr};
+    bool is_training = !this->pd()->rnn_conf.is_fwd;
 
     memory_t *workspace = (aprop == prop_kind::forward)
             ? ctx.output(DNNL_ARG_WORKSPACE)
             : ctx.input(DNNL_ARG_WORKSPACE);
 
-    std::unique_ptr<memory_storage_t> scratchpad;
-    if (pd()->rnn_conf.use_workspace) {
-        workspace->memory_storage()->get_data_handle(&scratchpad_ptr);
-    } else {
-        scratchpad = ctx.get_scratchpad_grantor().get_memory_storage(
-                key_rnn_space);
-        scratchpad->get_data_handle(&scratchpad_ptr);
-    }
-
-    auto scratchpad_gates
-            = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_gates);
-    scratchpad_gates->get_data_handle(&scratch_gates_ptr);
-
-    auto scratch_cell
-            = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_cell);
-    if (is_lbr || (is_vanilla_gru && gemm_kind == gemm_diff_wei_iter_2))
-        scratch_cell->get_data_handle(&scratch_cell_ptr);
-
-    void *weights_ptr {nullptr};
     memory_t *weights {nullptr};
 
     // These memory storages provide a mechanism to reuse existing memory
     // storage with an offset. These memory storages don't own attached memory
-    memory_storage_t *gemm_A_ = nullptr;
-    memory_storage_t *gemm_B_ = nullptr;
-    memory_storage_t *gemm_C_ = nullptr;
+    std::unique_ptr<memory_storage_t> gemm_A_;
+    std::unique_ptr<memory_storage_t> gemm_B_;
+    std::unique_ptr<memory_storage_t> gemm_C_;
+    std::unique_ptr<memory_storage_t> scratchpad;
+    std::unique_ptr<memory_storage_t> scratchpad_gates;
+    std::unique_ptr<memory_storage_t> scratchpad_cell;
+    std::unique_ptr<memory_storage_t> scratchpad_dhG1;
+    std::unique_ptr<memory_storage_t> scratchpad_diff;
 
-    engine->create_memory_storage(&gemm_A_, use_runtime_ptr, 0, nullptr);
-    engine->create_memory_storage(&gemm_B_, use_runtime_ptr, 0, nullptr);
-    engine->create_memory_storage(&gemm_C_, use_runtime_ptr, 0, nullptr);
+    scratchpad_gates
+            = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_gates);
+
+    if (pd()->rnn_conf.use_workspace) {
+        scratchpad = workspace->memory_storage()->clone();
+    } else {
+        scratchpad = ctx.get_scratchpad_grantor().get_memory_storage(
+                key_rnn_space);
+    }
+
+    if (is_lbr || (is_vanilla_gru && gemm_kind == gemm_diff_wei_iter_2)) {
+        scratchpad_cell
+                = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_cell);
+    }
+
+    if (is_vanilla_gru && gemm_kind == gemm_iter_bwd_2) {
+        scratchpad_dhG1 = ctx.get_scratchpad_grantor().get_memory_storage(
+                key_rnn_diff_ht);
+    }
+
+    if (is_training) {
+        scratchpad_diff = ctx.get_scratchpad_grantor().get_memory_storage(
+                key_rnn_diff_states);
+    }
 
     switch (gemm_kind) {
         case gemm_iter_fwd:
@@ -930,14 +983,12 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
             weights = (gemm_kind == gemm_layer_fwd)
                     ? ctx.input(DNNL_ARG_WEIGHTS_LAYER)
                     : ctx.input(DNNL_ARG_WEIGHTS_ITER);
-            weights->memory_storage()->get_data_handle(&weights_ptr);
-
-            gemm_A_->set_data_handle(weights_ptr);
-            gemm_B_->set_data_handle(scratchpad_ptr);
+            gemm_A_ = weights->memory_storage()->clone();
+            gemm_B_ = scratchpad->clone();
             if (is_lbr && gemm_kind == gemm_iter_fwd) {
-                gemm_C_->set_data_handle(scratch_cell_ptr);
+                gemm_C_ = scratchpad_cell->clone();
             } else {
-                gemm_C_->set_data_handle(scratch_gates_ptr);
+                gemm_C_ = scratchpad_gates->clone();
             }
             break;
         case gemm_iter_bwd:
@@ -946,37 +997,34 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
             weights = (gemm_kind == gemm_layer_bwd)
                     ? ctx.input(DNNL_ARG_WEIGHTS_LAYER)
                     : ctx.input(DNNL_ARG_WEIGHTS_ITER);
-            weights->memory_storage()->get_data_handle(&weights_ptr);
-
-            gemm_A_->set_data_handle(weights_ptr);
+            gemm_A_ = weights->memory_storage()->clone();
             if (is_lbr && gemm_kind == gemm_iter_bwd) {
-                gemm_B_->set_data_handle(scratch_cell_ptr);
+                gemm_B_ = scratchpad_cell->clone();
             } else {
-                gemm_B_->set_data_handle(scratch_gates_ptr);
+                gemm_B_ = scratchpad_gates->clone();
             }
-            gemm_C_->set_data_handle(scratchpad_ptr);
+            gemm_C_ = (gemm_kind == gemm_iter_bwd_2) ? scratchpad_dhG1->clone()
+                                                     : scratchpad_diff->clone();
             break;
         case gemm_diff_wei_iter:
         case gemm_diff_wei_layer:
             weights = (gemm_kind == gemm_diff_wei_iter)
                     ? ctx.output(DNNL_ARG_DIFF_WEIGHTS_ITER)
                     : ctx.output(DNNL_ARG_DIFF_WEIGHTS_LAYER);
-            weights->memory_storage()->get_data_handle(&weights_ptr);
             if (is_lbr && gemm_kind == gemm_diff_wei_iter) {
-                gemm_A_->set_data_handle(scratch_cell_ptr);
+                gemm_A_ = scratchpad_cell->clone();
             } else {
-                gemm_A_->set_data_handle(scratch_gates_ptr);
+                gemm_A_ = scratchpad_gates->clone();
             }
-            gemm_B_->set_data_handle(scratchpad_ptr);
-            gemm_C_->set_data_handle(weights_ptr);
+            gemm_B_ = scratchpad->clone();
+            gemm_C_ = weights->memory_storage()->clone();
             break;
         case gemm_diff_wei_iter_2:
             weights = ctx.output(DNNL_ARG_DIFF_WEIGHTS_ITER);
-            weights->memory_storage()->get_data_handle(&weights_ptr);
 
-            gemm_A_->set_data_handle(scratch_gates_ptr);
-            gemm_B_->set_data_handle(scratch_cell_ptr);
-            gemm_C_->set_data_handle(weights_ptr);
+            gemm_A_ = scratchpad_gates->clone();
+            gemm_B_ = scratchpad_cell->clone();
+            gemm_C_ = weights->memory_storage()->clone();
             break;
         default: assert(!"unknown gemm_kind");
     }
@@ -985,10 +1033,12 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
     gemm_B_->set_offset(off_b);
     gemm_C_->set_offset(off_c);
 
+    // We flip A and B here since the GEMM API is row major but the
+    // RNN code describes GEMM in column major fashion
     gemm_exec_args_t gemm_args;
-    gemm_args.a = gemm_A_;
-    gemm_args.b = gemm_B_;
-    gemm_args.c = gemm_C_;
+    gemm_args.a = gemm_B_.get();
+    gemm_args.b = gemm_A_.get();
+    gemm_args.c = gemm_C_.get();
 
     auto gemm_ctx = gemm_exec_ctx_t(ctx, gemm_args);
 
@@ -1041,10 +1091,6 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
             break;
         default: assert(!"unknown gemm_kind");
     }
-
-    delete gemm_A_;
-    delete gemm_B_;
-    delete gemm_C_;
 }
 
 template <prop_kind_t aprop>
@@ -1085,7 +1131,7 @@ grid_execution_sig((_ref_rnn_common_t<aprop>::linear_execution)) {
             cl_ulong offset_ws_layer, offset_wei_layer, offset_ws_iter;
             // offsets for bwd rnn gemm grid computation
             cl_ulong offset_diff_wei_iter, offset_diff_wei_lay,
-                    offset_diff_ws_lay;
+                    offset_scratch_diff_lay;
 
             set_offsets_fwd_gemm(rnn, dir, lay, src_t, wei_layer_offset_ptr,
                     ws_states_offset_, offset_ws_layer, offset_wei_layer,
@@ -1093,8 +1139,8 @@ grid_execution_sig((_ref_rnn_common_t<aprop>::linear_execution)) {
             if (aprop == prop_kind::backward) {
                 int start_diff_src_iter_idx = 0;
                 set_offsets_bwd_gemm(rnn, start_diff_src_iter_idx, dir, lay,
-                        ws_diff_states_offset_, offset_diff_wei_iter,
-                        offset_diff_wei_lay, offset_diff_ws_lay);
+                        offset_diff_wei_iter, offset_diff_wei_lay,
+                        offset_scratch_diff_lay);
             }
 
             if (aprop == prop_kind::forward && rnn.merge_gemm_layer) {
@@ -1107,15 +1153,15 @@ grid_execution_sig((_ref_rnn_common_t<aprop>::linear_execution)) {
                 int iter = (aprop == prop_kind::forward) ? i : n_iter - i - 1;
                 (this->*cell_func)(engine, ctx, dir, lay, iter,
                         &offset_wei_layer, wei_iter_offset_ptr, bias, workspace,
-                        scratch_gates, scratch_cell, wei_layer, wei_iter,
-                        diff_weights_layer, diff_weights_iter, diff_bias,
-                        scales, tm_scales);
+                        scratch_gates, scratch_cell, scratch_diff_states,
+                        scratch_dhG1, wei_layer, wei_iter, diff_weights_layer,
+                        diff_weights_iter, diff_bias, scales, tm_scales);
             }
 
             if (aprop == prop_kind::backward && rnn.merge_gemm_layer) {
                 gemm_primitive(engine, ctx, wei_layer, offset_wei_layer,
-                        scratch_gates, 0, workspace, offset_diff_ws_lay,
-                        gemm_layer_bwd);
+                        scratch_gates, 0, scratch_diff_states,
+                        offset_scratch_diff_lay, gemm_layer_bwd);
                 gemm_primitive(engine, ctx, scratch_gates, 0, workspace,
                         offset_ws_layer, diff_weights_layer,
                         offset_diff_wei_lay, gemm_diff_wei_layer);
@@ -1158,6 +1204,7 @@ template <prop_kind_t aprop>
 void _ref_rnn_common_t<aprop>::copy_init_layer(const exec_ctx_t &ctx,
         compute::compute_stream_t *compute_stream, bool lr, bool rl, int n_iter,
         int batch, int slc, const memory_storage_t &ws,
+        const memory_storage_t &scratch_diff_states,
         const memory_storage_t &input,
         const memory_storage_t &diff_dst_layer) const {
 
@@ -1165,8 +1212,9 @@ void _ref_rnn_common_t<aprop>::copy_init_layer(const exec_ctx_t &ctx,
         compute::kernel_arg_list_t arg_list;
         arg_list.set(0, ws);
         arg_list.set(1, input);
-        arg_list.set(2, (cl_int)lr);
-        arg_list.set(3, (cl_int)rl);
+        arg_list.set(2, scratch_diff_states);
+        arg_list.set(3, (cl_int)lr);
+        arg_list.set(4, (cl_int)rl);
 
         parallel_for(ctx, compute::nd_range_t({slc, batch, n_iter}),
                 copy_init_layer_kernel_, arg_list);
@@ -1174,8 +1222,9 @@ void _ref_rnn_common_t<aprop>::copy_init_layer(const exec_ctx_t &ctx,
         compute::kernel_arg_list_t arg_list;
         arg_list.set(0, ws);
         arg_list.set(1, diff_dst_layer);
-        arg_list.set(2, (cl_int)0);
+        arg_list.set(2, scratch_diff_states);
         arg_list.set(3, (cl_int)0);
+        arg_list.set(4, (cl_int)0);
 
         parallel_for(ctx, compute::nd_range_t({batch, n_iter}),
                 copy_init_layer_kernel_, arg_list);
@@ -1186,6 +1235,7 @@ template <prop_kind_t aprop>
 void _ref_rnn_common_t<aprop>::copy_init_iter(const exec_ctx_t &ctx,
         compute::compute_stream_t *compute_stream, int n_layer, int n_dir,
         int batch, int sic, int dhc, const memory_storage_t &ws,
+        const memory_storage_t &scratch_diff_states,
         const memory_storage_t &firstit_states,
         const memory_storage_t &firstit_c_states,
         const memory_storage_t &diff_dst_iter,
@@ -1198,9 +1248,10 @@ void _ref_rnn_common_t<aprop>::copy_init_iter(const exec_ctx_t &ctx,
         arg_list.set(0, ws);
         arg_list.set(1, firstit_states);
         arg_list.set(2, firstit_c_states);
-        arg_list.set(3, shift);
-        arg_list.set(4, scale);
-        arg_list.set(5, (int)quantize);
+        arg_list.set(3, scratch_diff_states);
+        arg_list.set(4, shift);
+        arg_list.set(5, scale);
+        arg_list.set(6, (int)quantize);
         parallel_for(ctx, compute::nd_range_t({max_d, batch, n_layer * n_dir}),
                 copy_init_iter_kernel_, arg_list);
     } else {
@@ -1208,6 +1259,7 @@ void _ref_rnn_common_t<aprop>::copy_init_iter(const exec_ctx_t &ctx,
         arg_list.set(0, ws);
         arg_list.set(1, diff_dst_iter);
         arg_list.set(2, diff_dst_iter_c);
+        arg_list.set(3, scratch_diff_states);
         parallel_for(ctx, compute::nd_range_t({dhc, batch, n_layer * n_dir}),
                 copy_init_iter_kernel_, arg_list);
     }
@@ -1216,7 +1268,9 @@ void _ref_rnn_common_t<aprop>::copy_init_iter(const exec_ctx_t &ctx,
 template <prop_kind_t aprop>
 void _ref_rnn_common_t<aprop>::copy_res_layer(const exec_ctx_t &ctx,
         compute::compute_stream_t *compute_stream, bool lr, bool rl, int n_iter,
-        int batch, int slc, int dhc, const memory_storage_t &dst_last_layer,
+        int batch, int slc, int dhc,
+        const memory_storage_t &scratch_diff_states,
+        const memory_storage_t &dst_last_layer,
         const memory_storage_t &diff_src_layer, const memory_storage_t &ws,
         const float shift, const float scale, const bool dequantize) const {
 
@@ -1224,19 +1278,21 @@ void _ref_rnn_common_t<aprop>::copy_res_layer(const exec_ctx_t &ctx,
         compute::kernel_arg_list_t arg_list;
         arg_list.set(0, ws);
         arg_list.set(1, dst_last_layer);
-        arg_list.set(2, (cl_int)lr);
-        arg_list.set(3, (cl_int)rl);
-        arg_list.set(4, shift);
-        arg_list.set(5, scale);
-        arg_list.set(6, (int)dequantize);
+        arg_list.set(2, scratch_diff_states);
+        arg_list.set(3, (cl_int)lr);
+        arg_list.set(4, (cl_int)rl);
+        arg_list.set(5, shift);
+        arg_list.set(6, scale);
+        arg_list.set(7, (int)dequantize);
         parallel_for(ctx, compute::nd_range_t({dhc, batch, n_iter}),
                 copy_res_layer_kernel_, arg_list);
     } else {
         compute::kernel_arg_list_t arg_list;
         arg_list.set(0, ws);
         arg_list.set(1, diff_src_layer);
-        arg_list.set(2, (cl_int)lr);
-        arg_list.set(3, (cl_int)rl);
+        arg_list.set(2, scratch_diff_states);
+        arg_list.set(3, (cl_int)lr);
+        arg_list.set(4, (cl_int)rl);
         parallel_for(ctx, compute::nd_range_t({slc, batch, n_iter}),
                 copy_res_layer_kernel_, arg_list);
     }
@@ -1245,7 +1301,9 @@ void _ref_rnn_common_t<aprop>::copy_res_layer(const exec_ctx_t &ctx,
 template <prop_kind_t aprop>
 void _ref_rnn_common_t<aprop>::copy_res_iter(const exec_ctx_t &ctx,
         compute::compute_stream_t *compute_stream, int n_layer, int n_dir,
-        int batch, int sic, int dhc, const memory_storage_t &dst_last_iter,
+        int batch, int sic, int dhc,
+        const memory_storage_t &scratch_diff_states,
+        const memory_storage_t &dst_last_iter,
         const memory_storage_t &dst_last_iter_c,
         const memory_storage_t &diff_src_iter,
         const memory_storage_t &diff_src_iter_c, const memory_storage_t &ws,
@@ -1256,9 +1314,10 @@ void _ref_rnn_common_t<aprop>::copy_res_iter(const exec_ctx_t &ctx,
         arg_list.set(0, ws);
         arg_list.set(1, dst_last_iter);
         arg_list.set(2, dst_last_iter_c);
-        arg_list.set(3, shift);
-        arg_list.set(4, scale);
-        arg_list.set(5, (int)dequantize);
+        arg_list.set(3, scratch_diff_states);
+        arg_list.set(4, shift);
+        arg_list.set(5, scale);
+        arg_list.set(6, (int)dequantize);
         parallel_for(ctx, compute::nd_range_t({dhc, batch, n_layer * n_dir}),
                 copy_res_iter_kernel_, arg_list);
     } else {
@@ -1267,6 +1326,7 @@ void _ref_rnn_common_t<aprop>::copy_res_iter(const exec_ctx_t &ctx,
         arg_list.set(0, ws);
         arg_list.set(1, diff_src_iter);
         arg_list.set(2, diff_src_iter_c);
+        arg_list.set(3, scratch_diff_states);
         parallel_for(ctx, compute::nd_range_t({max_d, batch, n_layer * n_dir}),
                 copy_res_iter_kernel_, arg_list);
     }
@@ -1382,11 +1442,20 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
     auto scratchpad_cell
             = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_cell);
 
-    auto &scratch_cell
-            = this->pd()->is_lbr() || this->pd()->rnn_conf.is_vanilla_gru
+    auto &scratch_cell = this->pd()->is_lbr() || is_vanilla_gru
             ? *scratchpad_cell
             : empty_mem;
 
+    auto scratchpad_diff_states
+            = ctx.get_scratchpad_grantor().get_memory_storage(
+                    key_rnn_diff_states);
+    auto &scratch_diff_states = is_fwd ? empty_mem : *scratchpad_diff_states;
+
+    auto scratchpad_dhG1
+            = ctx.get_scratchpad_grantor().get_memory_storage(key_rnn_diff_ht);
+    auto &scratch_dhG1
+            = (!is_fwd && is_vanilla_gru) ? *scratchpad_dhG1 : empty_mem;
+
     auto &diff_src_layer_native_ = CTX_OUT_STORAGE(DNNL_ARG_DIFF_SRC_LAYER);
     auto &diff_src_iter_native_ = CTX_OUT_STORAGE(DNNL_ARG_DIFF_SRC_ITER);
     auto &diff_src_iter_c_native_ = CTX_OUT_STORAGE(DNNL_ARG_DIFF_SRC_ITER_C);
@@ -1407,10 +1476,10 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
         DPRINT("  n_gates         = %d\n", n_gates);
         DPRINT("  n_bias          = %d\n", n_bias);
         DPRINT("  n_states        = %d\n", rnn.n_states);
-        DPRINT("  n_weights_layer = %d\n", rnn_pd()->SLC);
-        DPRINT("  n_weights_iter  = %d\n", rnn_pd()->SIC);
+        DPRINT("  n_weights_layer = %d\n", rnn_pd->SLC());
+        DPRINT("  n_weights_iter  = %d\n", rnn_pd->SIC());
         DPRINT("  batch           = %d\n", batch);
-        DPRINT("  slc             = %d\n", nlc);
+        DPRINT("  slc             = %d\n", slc);
         DPRINT("  sic             = %d\n", sic);
         DPRINT("  dhc             = %d\n", dhc);
         DPRINT("  dlc             = %d\n", dlc);
@@ -1460,14 +1529,6 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
     }
 #endif
 
-    // initialize diff_state to 0
-    if (aprop == prop_kind::backward) {
-        ws_set(ctx, compute_stream, workspace_, ws_dhG1_offset_,
-                rnn_utils::dhG1_gru, 0.0f, rnn.ws_dhG1_size);
-        ws_set(ctx, compute_stream, workspace_, ws_diff_states_offset_,
-                rnn_utils::diff_states, 0.0f, rnn.ws_diff_states_size);
-    }
-
     DPRINT("\n%s(%d) WS before bias prepare\n\n", __FUNCTION__, __LINE__);
     WS_PRINT(ctx, compute_stream, workspace_);
 
@@ -1504,13 +1565,14 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
 
     // we first need to copy the initial states and input into ws
     copy_init_layer(ctx, compute_stream, is_lr, is_rl, n_iter, batch, slc,
-            workspace_, src_layer_native_, diff_dst_layer_native_);
+            workspace_, scratch_diff_states, src_layer_native_,
+            diff_dst_layer_native_);
     const bool quantize = pd()->with_src_iter()
             && pd()->src_md(1)->data_type == data_type::f32 && rnn.is_int8;
     copy_init_iter(ctx, compute_stream, n_layer, n_dir, batch, sic, dhc,
-            workspace_, src_iter_native_, src_c_iter_native_,
-            diff_dst_iter_native_, diff_dst_iter_c_native_, shift, scale,
-            quantize);
+            workspace_, scratch_diff_states, src_iter_native_,
+            src_c_iter_native_, diff_dst_iter_native_, diff_dst_iter_c_native_,
+            shift, scale, quantize);
 
     DPRINT("\n%s(%d) WS before grid\n\n", __FUNCTION__, __LINE__);
     WS_PRINT(ctx, compute_stream, workspace_);
@@ -1522,9 +1584,10 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
 
     // run the execution on the grid
     (this->*grid_computation)(engine, ctx, bias_native_, workspace_,
-            scratch_gates, scratch_cell, wei_layer_native_, wei_iter_native_,
-            diff_weights_layer_native_, diff_weights_iter_native_,
-            diff_bias_native_, scales_buf, tm_scales_buf);
+            scratch_gates, scratch_cell, scratch_diff_states, scratch_dhG1,
+            wei_layer_native_, wei_iter_native_, diff_weights_layer_native_,
+            diff_weights_iter_native_, diff_bias_native_, scales_buf,
+            tm_scales_buf);
 
     DPRINT("\n%s(%d) WS before copy res\n\n", __FUNCTION__, __LINE__);
     WS_PRINT(ctx, compute_stream, workspace_);
@@ -1534,18 +1597,17 @@ status_t _ref_rnn_common_t<aprop>::execute_(const exec_ctx_t &ctx) const {
     const bool dequantize_l
             = pd()->dst_md(0)->data_type == data_type::f32 && rnn.is_int8;
     copy_res_layer(ctx, compute_stream, is_lr, is_rl, n_iter, batch, slc, dhc,
-            dst_last_layer_native_, diff_src_layer_native_, workspace_, shift,
-            scale, dequantize_l);
+            scratch_diff_states, dst_last_layer_native_, diff_src_layer_native_,
+            workspace_, shift, scale, dequantize_l);
     const bool dequantize_i = pd()->with_dst_iter()
             && pd()->dst_md(1)->data_type == data_type::f32 && rnn.is_int8;
     copy_res_iter(ctx, compute_stream, n_layer, n_dir, batch, sic, dhc,
-            dst_last_iter_native_, dst_last_iter_c_native_,
+            scratch_diff_states, dst_last_iter_native_, dst_last_iter_c_native_,
             diff_src_iter_native_, diff_src_iter_c_native_, workspace_, shift,
             scale, dequantize_i);
 
     return status::success;
 };
-
 // Fix for MSVS warning C4661.
 template <>
 cell_execution_sig(ref_rnn_fwd_t::cell_execution);
@@ -1572,13 +1634,13 @@ elemwise_sig(ref_rnn_fwd_t::lstm_elemwise_u8s8);
 template <>
 elemwise_sig(ref_rnn_bwd_t::lstm_elemwise_u8s8);
 template <>
-elemwise_sig(ref_rnn_fwd_t::gru_lbr_elemwise);
+elemwise_sig_gru_lbr(ref_rnn_fwd_t::gru_lbr_elemwise);
 template <>
-elemwise_sig(ref_rnn_bwd_t::gru_lbr_elemwise);
+elemwise_sig_gru_lbr(ref_rnn_bwd_t::gru_lbr_elemwise);
 template <>
-elemwise_sig(ref_rnn_fwd_t::gru_elemwise);
+elemwise_sig_gru(ref_rnn_fwd_t::gru_elemwise);
 template <>
-elemwise_sig(ref_rnn_bwd_t::gru_elemwise);
+elemwise_sig_gru(ref_rnn_bwd_t::gru_elemwise);
 
 template struct _ref_rnn_common_t<prop_kind::forward>;
 template struct _ref_rnn_common_t<prop_kind::backward>;
