@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 #include <limits.h>
 #include <stdint.h>
 
+#include <cctype>
 #include <fstream>
 #include <functional>
 #include <string>
@@ -27,6 +28,9 @@
 #include "dnnl.h"
 
 #include "common.hpp"
+#include "utils/parser.hpp"
+
+#include "tests/test_thread.hpp"
 
 // BENCHDNN_MEMORY_CHECK macro enables guarding mechanism for memory allocation:
 // memory block is allocated on a page boundary and the page after the block is
@@ -43,110 +47,13 @@
 #include <sys/mman.h>
 #endif
 
-const char *bench_mode2str(bench_mode_t mode) {
-    const char *modes[] = {"MODE_UNDEF", "CORR", "PERF", "CORR+PERF", "LIST"};
-    assert((int)mode < sizeof(modes) / sizeof(*modes));
-    return modes[(int)mode];
-}
+bench_mode_t RUN {0x1}; // Run mode.
+bench_mode_t CORR {0x2}; // Correctness mode. The default one.
+bench_mode_t PERF {0x4}; // Performance mode. May be combined with CORR.
+bench_mode_t LIST {0x8}; // Listing mode. Standalone mode to only create prb.
 
-bench_mode_t str2bench_mode(const char *str) {
-    bench_mode_t mode = MODE_UNDEF;
-    if (strchr(str, 'c') || strchr(str, 'C'))
-        mode = (bench_mode_t)((int)mode | (int)CORR);
-    if (strchr(str, 'p') || strchr(str, 'P'))
-        mode = (bench_mode_t)((int)mode | (int)PERF);
-    if (strchr(str, 'l') || strchr(str, 'L')) {
-        // list mode is exclusive
-        assert(mode == MODE_UNDEF);
-        mode = (bench_mode_t)((int)mode | (int)LIST);
-    }
-    if (mode == MODE_UNDEF)
-        []() {
-            SAFE(FAIL, CRIT);
-            return 0;
-        }();
-    return mode;
-}
-
-/* perf */
-#include <chrono>
-
-static inline double ms_now() {
-    auto timePointTmp
-            = std::chrono::high_resolution_clock::now().time_since_epoch();
-    return std::chrono::duration<double, std::milli>(timePointTmp).count();
-}
-
-#if !defined(BENCHDNN_USE_RDPMC) || defined(_WIN32)
-unsigned long long ticks_now() {
-    return (unsigned long long)0;
-}
-#else
-unsigned long long ticks_now() {
-    unsigned eax, edx, ecx;
-
-    ecx = (1 << 30) + 1;
-    __asm__ volatile("rdpmc" : "=a"(eax), "=d"(edx) : "c"(ecx));
-
-    return (unsigned long long)eax | (unsigned long long)edx << 32;
-}
-#endif
-
-void benchdnn_timer_t::reset() {
-    times_ = 0;
-    for (int i = 0; i < n_modes; ++i)
-        ticks_[i] = 0;
-    ticks_start_ = 0;
-    for (int i = 0; i < n_modes; ++i)
-        ms_[i] = 0;
-    ms_start_ = 0;
-
-    start();
-}
-
-void benchdnn_timer_t::start() {
-    ticks_start_ = ticks_now();
-    ms_start_ = ms_now();
-}
-
-void benchdnn_timer_t::stop(int add_times) {
-    if (add_times == 0) return;
-
-    unsigned long long d_ticks = ticks_now() - ticks_start_;
-    double d_ms = ms_now() - ms_start_;
-
-    ticks_start_ += d_ticks;
-    ms_start_ += d_ms;
-
-    ms_[benchdnn_timer_t::avg] += d_ms;
-    ticks_[benchdnn_timer_t::avg] += d_ticks;
-
-    d_ticks /= add_times;
-    d_ms /= add_times;
-
-    ms_[benchdnn_timer_t::min]
-            = times_ ? MIN2(ms_[benchdnn_timer_t::min], d_ms) : d_ms;
-    ms_[benchdnn_timer_t::max]
-            = times_ ? MAX2(ms_[benchdnn_timer_t::max], d_ms) : d_ms;
-
-    ticks_[benchdnn_timer_t::min]
-            = times_ ? MIN2(ticks_[benchdnn_timer_t::min], d_ticks) : d_ticks;
-    ticks_[benchdnn_timer_t::max]
-            = times_ ? MAX2(ticks_[benchdnn_timer_t::max], d_ticks) : d_ticks;
-
-    times_ += add_times;
-}
-
-benchdnn_timer_t &benchdnn_timer_t::operator=(const benchdnn_timer_t &rhs) {
-    if (this == &rhs) return *this;
-    times_ = rhs.times_;
-    for (int i = 0; i < n_modes; ++i)
-        ticks_[i] = rhs.ticks_[i];
-    ticks_start_ = rhs.ticks_start_;
-    for (int i = 0; i < n_modes; ++i)
-        ms_[i] = rhs.ms_[i];
-    ms_start_ = rhs.ms_start_;
-    return *this;
+bool is_bench_mode(bench_mode_t user_mode) {
+    return !(bench_mode & user_mode).none();
 }
 
 /* result structure */
@@ -172,8 +79,10 @@ const char *skip_reason2str(skip_reason_t skip_reason) {
     CASE(CASE_NOT_SUPPORTED);
     CASE(DATA_TYPE_NOT_SUPPORTED);
     CASE(INVALID_CASE);
+    CASE(KNOWN_LIMITATION);
     CASE(NOT_ENOUGH_RAM);
     CASE(SKIP_IMPL_HIT);
+    CASE(SKIP_START);
 #undef CASE
     return "SKIP_UNKNOWN";
 }
@@ -185,8 +94,17 @@ void parse_result(
 
     switch (res.state) {
         case UNTESTED:
-            if (!(bench_mode & CORR)) {
+            if (is_bench_mode(PERF)) {
                 want_perf_report = true;
+                if (status == FAIL)
+                    bs.failed++;
+                else
+                    bs.passed++;
+                break;
+            } else if (is_bench_mode(RUN)) {
+                assert(status == OK);
+                BENCHDNN_PRINT(0, "%d:EXECUTED __REPRO: %s\n", bs.tests, pstr);
+                want_perf_report = false;
                 break;
             }
         case FAILED:
@@ -226,18 +144,11 @@ void parse_result(
             want_perf_report = false;
             bs.listed++;
             break;
-        default:
-            assert(!"unknown state");
-            {
-                []() {
-                    SAFE(FAIL, CRIT);
-                    return 0;
-                }();
-            }
+        default: assert(!"unknown state"); SAFE_V(FAIL);
     }
 
-    if (bench_mode & PERF) {
-        using bt = benchdnn_timer_t;
+    if (is_bench_mode(PERF)) {
+        using bt = timer::timer_t;
         for (int mode = 0; mode < (int)bt::n_modes; ++mode)
             bs.ms[mode] += res.timer.ms((bt::mode_t)mode);
     }
@@ -249,7 +160,7 @@ void parse_result(
 static void *zmalloc_protect(size_t size) {
     const size_t page_sz = getpagesize();
 
-    const size_t block_sz = size + 2 * sizeof(void *);
+    const size_t block_sz = size + 3 * sizeof(void *);
     const size_t total_sz = div_up(block_sz, page_sz) * page_sz + page_sz;
 
     void *mem_ptr;
@@ -276,29 +187,36 @@ static void *zmalloc_protect(size_t size) {
         return nullptr;
     }
 
+    // Align down `ptr` on 8 bytes before storing addresses to make behavior
+    // defined.
+    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
+    void *ptr_aligned_8 = ptr - to_align;
     // Save pointers for zfree_protect
-    ((void **)ptr)[-2] = ptr_start;
-    ((void **)ptr)[-1] = ptr_protect;
+    ((void **)ptr_aligned_8)[-2] = ptr_start;
+    ((void **)ptr_aligned_8)[-1] = ptr_protect;
 
     return ptr;
 }
 
 static void zfree_protect(void *ptr) {
-    const size_t page_sz = getpagesize();
+    // Get aligned ptr before obtaining addresses
+    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
+    void *ptr_aligned_8 = reinterpret_cast<uint8_t *>(ptr) - to_align;
 
     // Restore read-write access for the protected region
-    void *ptr_protect = ((void **)ptr)[-1];
+    void *ptr_protect = ((void **)ptr_aligned_8)[-1];
+    const size_t page_sz = getpagesize();
     mprotect(ptr_protect, page_sz, PROT_READ | PROT_WRITE);
 
     // Deallocate the whole region
-    void *ptr_start = ((void **)ptr)[-2];
+    void *ptr_start = ((void **)ptr_aligned_8)[-2];
     ::free(ptr_start);
 }
 #endif
 
 void *zmalloc(size_t size, size_t align) {
 #ifdef BENCHDNN_MEMORY_CHECK
-    if (bench_mode & CORR) { return zmalloc_protect(size); }
+    if (is_bench_mode(CORR)) { return zmalloc_protect(size); }
 #endif
 
     void *ptr;
@@ -313,7 +231,7 @@ void *zmalloc(size_t size, size_t align) {
 
     // TODO. Heuristics: Increasing the size to alignment increases
     // the stability of performance results.
-    if ((bench_mode & PERF) && (size < align)) size = align;
+    if (is_bench_mode(PERF) && (size < align)) size = align;
     int rc = ::posix_memalign(&ptr, align, size);
 #endif /* _WIN32 */
     return rc == 0 ? ptr : nullptr;
@@ -321,7 +239,7 @@ void *zmalloc(size_t size, size_t align) {
 
 void zfree(void *ptr) {
 #ifdef BENCHDNN_MEMORY_CHECK
-    if (bench_mode & CORR) {
+    if (is_bench_mode(CORR)) {
         zfree_protect(ptr);
         return;
     }
@@ -376,23 +294,14 @@ bool match_regex(const char *str, const char *pattern) {
 bool maybe_skip(const std::string &impl_str) {
     if (skip_impl.empty()) return false;
 
-    size_t start_pos = 0, end_pos = 0;
-    if (skip_impl[0] == '"' || skip_impl[0] == '\'') start_pos++;
-
-    do {
-        const size_t delim_pos = skip_impl.find_first_of(':', start_pos);
-        // rows below to identify quotes at the end and deal with them
-        end_pos = MIN2(skip_impl.size(), delim_pos);
-        size_t len = end_pos - start_pos;
-        if (skip_impl[end_pos - 1] == '"' || skip_impl[end_pos - 1] == '\'')
-            len--;
-        std::string sub_skip_impl = skip_impl.substr(start_pos, len);
-        // even incomplete match leads to skipping
-        if (!sub_skip_impl.empty()
-                && impl_str.find(sub_skip_impl) != std::string::npos)
-            return true;
-        start_pos = end_pos + 1;
-    } while (end_pos < skip_impl.size());
+    size_t start_pos = 0;
+    // Iterate over impls in skip list.
+    while (start_pos != std::string::npos) {
+        const auto skip_impl_item
+                = parser::get_substr(skip_impl, start_pos, ',');
+        if (skip_impl_item.empty()) continue;
+        if (impl_str.find(skip_impl_item) != std::string::npos) return true;
+    }
 
     return false;
 }
@@ -458,12 +367,14 @@ std::string locate_batch_file(const std::string &fname) {
     if (ifs.is_open()) return fname;
 
     for (int n = 0; n < n_paths; ++n) {
-        const std::string fullname = search_paths[n] + "/" + fname;
+        std::string fullname = search_paths[n] + "/" + fname;
         ifs.open(fullname);
         if (ifs.is_open()) {
             BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+            ifs.close();
             return fullname;
         }
+        ifs.close();
     }
 
     // Search in default inputs directory
@@ -480,13 +391,15 @@ std::string locate_batch_file(const std::string &fname) {
                         + std::string(driver_name);
             }
             // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
-            const std::string fullname = fdir + "/" + fname;
+            std::string fullname = fdir + "/" + fname;
             ifs.open(fullname);
             if (ifs.is_open()) {
                 search_paths[n_paths++] = std::move(fdir);
                 BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+                ifs.close();
                 return fullname;
             }
+            ifs.close();
         }
     }
 
@@ -496,7 +409,7 @@ std::string locate_batch_file(const std::string &fname) {
 
 int batch(const char *fname, bench_f bench) {
     std::ifstream ifs(locate_batch_file(std::string(fname)));
-    SAFE_V(ifs.is_open() ? OK : FAIL);
+    SAFE(ifs.is_open() ? OK : FAIL, CRIT);
 
     std::vector<std::string> opts;
     std::string str;
@@ -583,6 +496,7 @@ void gemm(const char *layout, const char *transa, const char *transb, int64_t m,
         int64_t n, int64_t k, const float alpha, const float *a,
         const int64_t lda, const float *b, const int64_t ldb, const float beta,
         float *c, const int64_t ldc) {
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
     if (*layout == 'C') {
         dnnl_sgemm(
                 *transa, *transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
@@ -590,6 +504,30 @@ void gemm(const char *layout, const char *transa, const char *transb, int64_t m,
         dnnl_sgemm(
                 *transb, *transa, n, m, k, alpha, b, ldb, a, lda, beta, c, ldc);
     }
+#else
+    if (std::toupper(*layout) != 'C') {
+        gemm("C", transb, transa, n, m, k, alpha, b, ldb, a, lda, beta, c, ldc);
+        return;
+    }
+
+    auto a_accessor = [&](int64_t i, int64_t j) {
+        if (std::toupper(*transa) == 'N') return a[i * lda + j];
+        return a[j * lda + i];
+    };
+
+    auto b_accessor = [&](int64_t i, int64_t j) {
+        if (std::toupper(*transb) == 'N') return b[i * ldb + j];
+        return b[j * ldb + i];
+    };
+
+    dnnl::impl::parallel_nd(m, n, [&](int64_t i, int64_t j) {
+        float ab = 0.0f;
+        for (int64_t _k = 0; _k < k; ++_k)
+            ab += a_accessor(i, _k) * b_accessor(_k, j);
+        float cij = (beta == 0) ? 0.0f : beta * c[i * ldc + j];
+        c[i * ldc + j] = alpha * ab + cij;
+    });
+#endif
 }
 
 int sanitize_desc(int &ndims, std::vector<std::reference_wrapper<int64_t>> d,
