@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
+* Copyright 2021 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,22 +15,30 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <cstring>
+
 #include <float.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+#include "tests/test_isa_common.hpp"
+#endif
 
 #include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
 #include "norm.hpp"
+#include "utils/compare.hpp"
 
 #include "binary/binary.hpp"
 #include "conv/conv_common.hpp"
 #include "eltwise/eltwise.hpp"
+#include "prelu/prelu.hpp"
 
 namespace conv {
 
@@ -140,7 +149,6 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
     // Update trh with the largest value from all eltwise post-ops
     const auto &po = prb->attr.post_ops;
     bool has_eltwise = po.eltwise_index() != -1;
-    const bool has_binary = po.binary_index() != -1;
     using pk_t = attr_t::post_ops_t::kind_t;
 
     int sum_ind = po.find(pk_t::SUM);
@@ -169,24 +177,26 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
             // the library for integral target data types.
             not_a_num += 1;
             not_a_num_ok += true;
-        } else if (fp < f_min) {
+        } else if (fp0 < f_min) {
             diff_norm.update(f_min, dt);
             ok = dt == f_min;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
             below += 1;
             below_ok += ok;
-        } else if (fp > f_max) {
+        } else if (fp0 > f_max) {
             diff_norm.update(f_max, dt);
             ok = dt == f_max;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
+            // Hack for (TODO: Intel only?) CPU saturation check.
+            if (is_cpu() && f_dt == dnnl_s32)
+                ok = dt >= BENCHDNN_S32_TO_F32_SAT_CONST && dt < f_max;
             above += 1;
             above_ok += ok;
+        } else if (fp == dt) {
+            // Quick check.
+            diff_norm.update(fp, dt);
+            in += 1;
+            in_ok += ok;
         } else {
             diff_norm.update(fp, dt);
             float trh = get_eps(prb, kind);
@@ -200,10 +210,7 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
                 }
             }
             ok = (fabs(fp) > 1e-5 ? rel_diff : diff) <= trh;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
             in += 1;
             in_ok += ok;
         }
@@ -213,8 +220,7 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
         bool dump = (!ok
                             && ((!dont_complain && res->errors < 10)
                                     || verbose >= 10))
-                || (final_compare
-                        && ((verbose >= 50 && i < 30) || (verbose >= 99)));
+                || (final_compare && (verbose >= 99));
 
         if (dump) {
             int64_t mb_or_g = 0, g_or_oc = 0, c = 0, d = 0, h = 0, w = 0;
@@ -318,22 +324,40 @@ int compare_dst(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
     return compare_dat(prb, DST, mem_dt, mem_fp, res, final_compare);
 }
 
+bool need_src_init(const prb_t *prb) {
+    return !(prb->dir == BWD_D);
+}
+
+bool need_wei_init(const prb_t *prb) {
+    return !(prb->dir & FLAG_BWD && prb->dir & FLAG_WEI);
+}
+
+bool need_bia_init(const prb_t *prb) {
+    return need_wei_init(prb);
+}
+
+bool need_dst_init(const prb_t *prb) {
+    return !(prb->dir & FLAG_FWD)
+            || (prb->attr.post_ops.find(attr_t::post_ops_t::SUM) >= 0);
+}
+
 int fill_src(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (is_bench_mode(CORR)) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem) {
+    if (check_reorder) {
         extra_mem
                 = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::abx, get_test_engine());
     }
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
 
     const auto &c = prb->cfg[SRC];
     const int range = c.f_max - c.f_min + 1;
 
     dnnl::impl::parallel_nd(prb->mb, prb->ic, prb->id, prb->ih, prb->iw,
-            [&](int mb, int ic, int id, int ih, int iw) {
-                const int gen
+            [&](int64_t mb, int64_t ic, int64_t id, int64_t ih, int64_t iw) {
+                const int64_t gen
                         = 101 * id + 103 * ih + 107 * iw + 109 * mb + 113 * ic;
                 const bool non_base = flip_coin(gen, c.f_sparsity);
                 float value = non_base ? c.f_min + gen * c.f_step % range
@@ -347,7 +371,7 @@ int fill_src(
             });
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_src(prb, mem_fp, mem_00, res), WARN);
     }
@@ -358,12 +382,31 @@ int fill_src(
 int fill_wei(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
     const bool wino_s8 = prb->alg == WINO && prb->cfg[WEI].dt == dnnl_s8;
-    const bool s8_s8
-            = prb->cfg[WEI].dt == dnnl_s8 && prb->cfg[SRC].dt == dnnl_s8;
     const bool is_def_zp = prb->attr.zero_points.is_def(DNNL_ARG_SRC);
     const bool diff_data_type = mem_dt.dt() != mem_fp.dt();
-    const bool check_reorder
-            = diff_data_type && !wino_s8 && !s8_s8 && is_def_zp;
+
+    dnnl_data_type_t dt_check = dnnl_s8;
+#if DNNL_AARCH64
+    /* Note for x64:
+    Both data types of src and weight are s8, oneDNN addds 128 to one of the s8
+    input to make it of type u8 instead, as explained in
+    https://oneapi-src.github.io/oneDNN/dev_guide_int8_computations.html or
+    doc/advanced/int8_computations.md
+    It is because `VPDPBUSD` instruction uses the combination of s8 and u8 as
+    input.
+
+    Note for AArch64:
+    Because dot product instructions of AArch64 "SDOT" receives s8 input
+    for both src and weight, the addition (and its counterpart of subtraction)
+    is not required for AArch64.
+    */
+    if (res->impl_name.find("jit", 0) == 0) dt_check = dnnl_u8;
+#endif
+
+    const bool wei_x8x8
+            = prb->cfg[WEI].dt == dnnl_s8 && prb->cfg[SRC].dt == dt_check;
+    const bool check_reorder = (is_bench_mode(CORR)) && diff_data_type
+            && !wino_s8 && !wei_x8x8 && is_def_zp;
 
     dnn_mem_t extra_mem;
     if (check_reorder) {
@@ -377,8 +420,9 @@ int fill_wei(
 
     dnnl::impl::parallel_nd(prb->g, prb->oc / prb->g, prb->ic / prb->g, prb->kd,
             prb->kh, prb->kw,
-            [&](int g, int oc, int ic, int kd, int kh, int kw) {
-                const int gen
+            [&](int64_t g, int64_t oc, int64_t ic, int64_t kd, int64_t kh,
+                    int64_t kw) {
+                const int64_t gen
                         = 127 * kd + 131 * kh + 137 * kw + 139 * oc + 149 * ic;
                 const bool non_base = flip_coin(gen, c.f_sparsity);
                 const float value = non_base ? c.f_min + gen * c.f_step % range
@@ -393,17 +437,29 @@ int fill_wei(
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_wei(prb, mem_fp, mem_00, res), WARN);
     }
+    if ((wei_x8x8 || !is_def_zp) && is_cpu()) {
+        // Check that s8 -> s8_comp exists in the library since users may have
+        // already quantized data.
+        dnn_mem_t mem_fp_s8(mem_fp.md_, dnnl_s8, get_test_engine());
+        dnn_mem_t mem_dt_s8(mem_dt.md_, dnnl_s8, get_test_engine());
+        SAFE(mem_fp_s8.reorder(mem_fp), WARN);
+        SAFE(mem_dt_s8.reorder(mem_fp_s8), WARN);
+        SAFE(mem_dt.size() == mem_dt_s8.size() ? OK : FAIL, WARN);
+        int rc = std::memcmp((void *)mem_dt, (void *)mem_dt_s8, mem_dt.size());
+        SAFE(rc == 0 ? OK : FAIL, WARN);
+    }
 
     return OK;
 }
 
 int fill_bia(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (is_bench_mode(CORR)) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem)
+    if (check_reorder)
         extra_mem = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::x, get_test_engine());
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
 
     const size_t nelems = mem_00.nelems();
     if (nelems == 0) return OK;
@@ -421,7 +477,7 @@ int fill_bia(
     }
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_bia(prb, mem_fp, mem_00, res), WARN);
     }
@@ -431,19 +487,20 @@ int fill_bia(
 int fill_dst_with_params(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
         dnnl_data_type_t dt, double sparsity, int min, int max, int base,
         int step, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (is_bench_mode(CORR)) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem) {
+    if (check_reorder) {
         extra_mem
                 = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::abx, get_test_engine());
     }
 
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
     const int range = max - min + 1;
 
     dnnl::impl::parallel_nd(prb->mb, prb->oc, prb->od, prb->oh, prb->ow,
-            [&](int mb, int oc, int od, int oh, int ow) {
-                const int gen
+            [&](int64_t mb, int64_t oc, int64_t od, int64_t oh, int64_t ow) {
+                const int64_t gen
                         = 157 * od + 163 * oh + 167 * ow + 173 * mb + 179 * oc;
                 const bool non_base = flip_coin(gen, sparsity);
                 const float value = non_base ? min + gen * step % range : base;
@@ -453,7 +510,7 @@ int fill_dst_with_params(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
             });
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_dst(prb, mem_fp, mem_00, res), WARN);
     }
@@ -469,10 +526,15 @@ int fill_dst(
                                   : dnnl_data_type_undef;
     bool diff_sum_dst_types
             = sum_dt != dnnl_data_type_undef && sum_dt != dst_dt;
+    bool sum_dt_is_int8 = sum_dt == dnnl_s8 || sum_dt == dnnl_u8;
 
     const auto &c = prb->cfg[DST];
-    float f_min = (diff_sum_dst_types) ? lowest_dt(sum_dt) : c.f_min;
-    float f_max = (diff_sum_dst_types) ? max_dt(sum_dt) : c.f_max;
+    float f_min = c.f_min;
+    float f_max = c.f_max;
+    if (diff_sum_dst_types && sum_dt_is_int8) {
+        f_min = lowest_dt(sum_dt);
+        f_max = max_dt(sum_dt);
+    }
 
     // Change mem dt to sum dt, so we can save sum data properly.
     if (diff_sum_dst_types) { mem_dt.set_dt(sum_dt); }
@@ -485,15 +547,8 @@ int fill_dst(
     return OK;
 }
 
-inline int init_pd_custom(dnnl_engine_t engine, const prb_t *prb,
-        dnnl_primitive_desc_t &cpd, res_t *res,
-        dnnl_data_type_t src_dt = dnnl_data_type_undef,
-        dnnl_data_type_t wei_dt = dnnl_data_type_undef,
-        dnnl_data_type_t bia_dt = dnnl_data_type_undef,
-        dnnl_data_type_t dst_dt = dnnl_data_type_undef,
-        dnnl_data_type_t acc_dt = dnnl_data_type_undef,
-        std::string src_tag = tag::undef, std::string wei_tag = tag::undef,
-        std::string bia_tag = tag::undef, std::string dst_tag = tag::undef) {
+int init_pd(dnnl_engine_t engine, const prb_t *prb, dnnl_primitive_desc_t &cpd,
+        res_t *res, dir_t dir, const_dnnl_primitive_desc_t hint) {
     dnnl_convolution_desc_t cd;
     dnnl_memory_desc_t src_d, wei_d, bia_d, dst_d;
 
@@ -524,25 +579,17 @@ inline int init_pd_custom(dnnl_engine_t engine, const prb_t *prb,
             ? dst_3d_dims
             : prb->ndims == 4 ? dst_2d_dims : dst_1d_dims;
 
-    if (src_dt == dnnl_data_type_undef) src_dt = prb->cfg[SRC].dt;
-    if (wei_dt == dnnl_data_type_undef) wei_dt = prb->cfg[WEI].dt;
-    if (bia_dt == dnnl_data_type_undef) bia_dt = prb->cfg[BIA].dt;
-    if (dst_dt == dnnl_data_type_undef) dst_dt = prb->cfg[DST].dt;
-    if (acc_dt == dnnl_data_type_undef) acc_dt = prb->cfg[ACC].dt;
-    if (src_tag == tag::undef) src_tag = normalize_tag(prb->stag, prb->ndims);
-    if (wei_tag == tag::undef) wei_tag = normalize_tag(prb->wtag, prb->ndims);
-    if (bia_tag == tag::undef) bia_tag = tag::any;
-    if (dst_tag == tag::undef) dst_tag = normalize_tag(prb->dtag, prb->ndims);
-
-    SAFE(init_md(&src_d, prb->ndims, src_dims, src_dt, src_tag), WARN);
-
-    SAFE(init_md(&wei_d, prb->ndims + prb->has_groups, wei_dims, wei_dt,
-                 wei_tag),
+    SAFE(init_md(&src_d, prb->ndims, src_dims, prb->cfg[SRC].dt,
+                 normalize_tag(prb->stag, prb->ndims)),
+            WARN);
+    SAFE(init_md(&wei_d, prb->ndims + prb->has_groups, wei_dims,
+                 prb->cfg[WEI].dt,
+                 normalize_tag(prb->wtag, prb->ndims + prb->has_groups)),
+            WARN);
+    SAFE(init_md(&bia_d, 1, bia_dims, prb->cfg[BIA].dt, tag::any), WARN);
+    SAFE(init_md(&dst_d, prb->ndims, dst_dims, prb->cfg[DST].dt,
+                 normalize_tag(prb->dtag, prb->ndims)),
             WARN);
-
-    SAFE(init_md(&bia_d, 1, bia_dims, bia_dt, bia_tag), WARN);
-
-    SAFE(init_md(&dst_d, prb->ndims, dst_dims, dst_dt, dst_tag), WARN);
 
     dnnl_dim_t strides_nd[] = {prb->sd, prb->sh, prb->sw};
     dnnl_dim_t dilates_nd[] = {prb->dd, prb->dh, prb->dw};
@@ -587,19 +634,19 @@ inline int init_pd_custom(dnnl_engine_t engine, const prb_t *prb,
         default: DNN_SAFE(dnnl_invalid_arguments, CRIT);
     }
 
-    DNN_SAFE(cd.accum_data_type == acc_dt ? dnnl_success : dnnl_unimplemented,
+    DNN_SAFE(cd.accum_data_type == prb->cfg[ACC].dt ? dnnl_success
+                                                    : dnnl_unimplemented,
             CRIT);
 
     attr_args_t attr_args;
     attr_args.prepare_output_scales(prb->attr, prb->scales, prb->oc);
-    attr_args.prepare_binary_post_op_mds(prb->attr, prb->ndims, dst_dims);
-    auto dnnl_attr = create_dnnl_attr(prb->attr, attr_args);
+    attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, dst_dims);
+    auto dnnl_attr = make_benchdnn_dnnl_wrapper(
+            create_dnnl_attr(prb->attr, attr_args));
 
     dnnl_status_t init_status
             = dnnl_primitive_desc_create(&cpd, &cd, dnnl_attr, engine, nullptr);
 
-    dnnl_primitive_attr_destroy(dnnl_attr);
-
     if (!res) return OK;
 
     if (init_status == dnnl_unimplemented)
@@ -610,13 +657,42 @@ inline int init_pd_custom(dnnl_engine_t engine, const prb_t *prb,
     if (maybe_skip(res->impl_name)) {
         BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
                 res->impl_name.c_str());
-        DNN_SAFE(dnnl_primitive_desc_destroy(cpd), WARN);
         return res->state = SKIPPED, res->reason = SKIP_IMPL_HIT, OK;
     } else {
         BENCHDNN_PRINT(
                 5, "oneDNN implementation: %s\n", res->impl_name.c_str());
     }
 
+    SAFE(check_pd_w_and_wo_attr(res, prb->attr, cd), WARN);
+
+    return OK;
+}
+
+int init_prim_ref(
+        benchdnn_dnnl_wrapper_t<dnnl_primitive_t> &prim_ref, const prb_t *prb) {
+    if (!(is_bench_mode(CORR) && is_gpu() && fast_ref_gpu)) return OK;
+
+    // Create a new copy of prb to avoid potentially corrupting the test by
+    // modifying prb in place.
+    // DIRECT algorithm is used to prevent fallback  to the slow benchdnn
+    // reference implementation.
+    auto cpu_attr = prb->attr;
+    update_cpu_ref_attrs(cpu_attr);
+    prb_t prb_cpu {*prb, prb->dir, conf_f32, tag::abx, tag::abx, tag::abx,
+            DIRECT, cpu_attr, prb->mb, prb->is_deconv};
+    dnnl_primitive_desc_t pd_ref_ {};
+    SAFE(init_pd(get_cpu_engine(), &prb_cpu, pd_ref_, nullptr, prb->dir,
+                 nullptr),
+            WARN);
+    auto pd_ref = make_benchdnn_dnnl_wrapper(pd_ref_);
+
+    dnnl_primitive_t prim_ref_ {};
+    if (pd_ref) {
+        DNN_SAFE(dnnl_primitive_create(&prim_ref_, pd_ref), WARN);
+        BENCHDNN_PRINT(
+                5, "%s\n", "benchdnn: use CPU primitive as the reference");
+    }
+    prim_ref.reset(prim_ref_);
     return OK;
 }
 
@@ -625,13 +701,74 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
             {prb->cfg[SRC].dt, prb->cfg[WEI].dt, prb->cfg[DST].dt}, prb->dir,
             res);
     if (res->state == SKIPPED) return;
+    // GPU only case
+    bool is_f32_wei = prb->cfg[WEI].dt == dnnl_f32;
+    bool is_f32_src = prb->cfg[SRC].dt == dnnl_f32;
+    bool is_int8_dst = prb->cfg[DST].dt == dnnl_s8;
+    const bool f32_s8_conv = is_f32_src && is_f32_wei && is_int8_dst;
+
+    if (is_cpu() && f32_s8_conv) {
+        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
+
+    if (is_nvidia_gpu()) {
+        const int64_t ID = prb->id, IH = prb->ih, IW = prb->iw;
+        const int64_t OD = prb->od, OH = prb->oh, OW = prb->ow;
+        const int64_t KD = prb->kd, KH = prb->kh, KW = prb->kw;
+        const int64_t SD = prb->sd, SH = prb->sh, SW = prb->sw;
+        const int64_t PD = prb->pd, PH = prb->ph, PW = prb->pw;
+        const int64_t PD_R = prb->pd_r, PH_R = prb->ph_r, PW_R = prb->pw_r;
+        const bool pad_ok = PD >= PD_R && PH >= PH_R && PW >= PW_R;
+        // copy-pasted from str2desc, dilation is not supported for Nvidia
+        const auto compute_out
+                = [](int64_t i, int64_t k, int64_t s, int64_t p) {
+                      return (i - k + 2 * p) / s + 1;
+                  };
+        const bool out_ok = OD == compute_out(ID, KD, SD, PD)
+                && OH == compute_out(IH, KH, SH, PH)
+                && OW == compute_out(IW, KW, SW, PW);
+
+        const auto &po = prb->attr.post_ops;
+        bool post_ops_ok = true;
+        for (int i = 0; i < po.len(); ++i) {
+            const auto &e = po.entry[i];
+            if (e.is_sum_kind())
+                continue;
+            else if (e.is_eltwise_kind())
+                post_ops_ok = post_ops_ok && is_nvidia_eltwise_ok(prb->dir, e);
+            else if (e.is_binary_kind() || e.is_convolution_kind())
+                post_ops_ok = false;
+            else
+                assert(!"unknown post-op type");
+        }
+
+        const auto dtag = normalize_tag(prb->dtag, prb->ndims);
+        const bool dtag_is_axb = dtag == normalize_tag(tag::axb, prb->ndims);
+        const bool tag_ok = !((prb->dir & FLAG_BWD) && dtag_is_axb);
+
+        const bool is_f16_src = prb->cfg[SRC].dt == dnnl_f16;
+        const bool is_f16_wei = prb->cfg[WEI].dt == dnnl_f16;
+        const bool f16_s8_conv = is_f16_src && is_f16_wei && is_int8_dst;
+
+        // TODO: specified wtag (even for supported formats) is not working?
+        if (!pad_ok || !out_ok || !post_ops_ok || !tag_ok || f32_s8_conv
+                || f16_s8_conv) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
 
     // Winograd implementation limitations.
     if (prb->alg == WINO) {
-        if (engine_tgt_kind == dnnl_cpu) {
+        if (is_cpu()) {
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
             static auto isa = dnnl_get_effective_cpu_isa();
-            static bool has_avx512_common = isa >= dnnl_cpu_isa_avx512_mic;
-            static bool has_avx512_bw = isa >= dnnl_cpu_isa_avx512_core;
+            static bool has_avx512_bw
+                    = dnnl::is_superset(isa, dnnl_cpu_isa_avx512_core);
+            static bool has_avx512_common = has_avx512_bw
+                    || dnnl::is_superset(isa, dnnl_cpu_isa_avx512_mic);
+
             bool is_int8 = prb->cfg[WEI].dt == dnnl_s8;
 
             bool pad_ok_f32 = prb->pw <= 1 && prb->ph <= 1 && prb->pw_r <= 1
@@ -675,14 +812,27 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
                 res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
                 return;
             }
-        } else if (engine_tgt_kind == dnnl_gpu) {
+#endif
+        } else if (is_gpu()) {
             bool shape_ok = prb->ndims == 4 && prb->g == 1 && prb->kh == 3
                     && prb->kw == 3 && prb->sh == 1 && prb->sw == 1
-                    && prb->dh == 0 && prb->dw == 0;
+                    && prb->dh == 0 && prb->dw == 0 && prb->pw < prb->kw
+                    && prb->pw_r < prb->kw && prb->ph < prb->kh
+                    && prb->ph_r < prb->kh;
             if (!shape_ok) {
                 res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                return;
             }
-            return;
+
+            const auto stag = normalize_tag(prb->stag, prb->ndims);
+            const bool stag_is_axb
+                    = stag == normalize_tag(tag::axb, prb->ndims);
+            bool is_axb_ok = (prb->ic % 16 == 0) && (prb->oc % 16 == 0);
+            if (stag_is_axb && !is_axb_ok) {
+                res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                return;
+            }
+
         } else {
             assert(!"Unknown Engine");
             res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
@@ -695,26 +845,17 @@ int doit(const prb_t *prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
     check_known_skipped_case(prb, res);
+    check_sum_post_ops(prb->attr, res, prb->cfg[DST].dt);
     if (res->state == SKIPPED) return OK;
 
-    dnnl_primitive_t c {};
-    // TODO: align init_pd interface with a common one which is used
-    // in the rest of the benchdnn drivers
-    auto init_pd = [&](dnnl_engine_t engine, const prb_t *prb,
-                           dnnl_primitive_desc_t &cpd, res_t *res, dir_t dir,
-                           const_dnnl_primitive_desc_t hint) {
-        SAFE(init_pd_custom(engine, prb, cpd, res), WARN);
-        return OK;
-    };
-
-    SAFE(init_prim(&c, init_pd, prb, res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
+    SAFE(init_prim(prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_pd), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_pd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
-        DNN_SAFE_V(dnnl_primitive_destroy(c));
+    if (check_mem_size(const_pd) != OK) {
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
 
@@ -750,26 +891,21 @@ int doit(const prb_t *prb, res_t *res) {
     const auto src_tag = tag::abx;
     const auto wei_tag = tag::abx;
 
-    // Try to use CPU primitive as the reference in GPU testing to reduce
-    // testing time
-    dnnl_primitive_t c_ref {};
+    // Use CPU prim as the reference in GPU testing to reduce testing time.
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim_ref;
+    SAFE(init_prim_ref(prim_ref, prb), WARN);
 
-    if (bench_mode & CORR && engine_tgt_kind == dnnl_gpu && fast_ref_gpu
-            && // TODO: temporary disable cpu as ref for testcases with binary post-ops
-            prb->attr.post_ops.binary_index() == -1) {
-        dnnl_primitive_desc_t cpd_ref = nullptr;
-        SAFE(init_pd_custom(get_cpu_engine(), prb, cpd_ref, nullptr, fp, fp, fp,
-                     fp, fp, src_tag, wei_tag, tag::x, src_tag),
-                WARN);
-        if (cpd_ref) {
-            DNN_SAFE(dnnl_primitive_create(&c_ref, cpd_ref), WARN);
-            BENCHDNN_PRINT(
-                    5, "%s\n", "benchdnn: use CPU primitive as the reference");
-            DNN_SAFE(dnnl_primitive_desc_destroy(cpd_ref), CRIT);
-        }
-    }
+    const_dnnl_primitive_desc_t const_pd_ref;
+    if (prim_ref)
+        DNN_SAFE(dnnl_primitive_get_primitive_desc(prim_ref, &const_pd_ref),
+                CRIT);
+    const auto q_ref = [&](int index = 0) -> const dnnl_memory_desc_t & {
+        return *dnnl_primitive_desc_query_md(
+                const_pd_ref, dnnl_query_exec_arg_md, index);
+    };
 
     const auto &test_engine = get_test_engine();
+    const auto &ref_engine = prim_ref ? get_cpu_engine() : get_test_engine();
 
     dnn_mem_t src_dt(src_md, test_engine);
     dnn_mem_t wei_dt(wei_md, test_engine);
@@ -781,26 +917,36 @@ int doit(const prb_t *prb, res_t *res) {
     dnn_mem_t dst_zero_points_m;
     std::vector<dnn_mem_t> binary_po_fp, binary_po_dt;
     std::vector<int> binary_po_args;
-    SAFE(binary::setup_binary_po(
-                 const_pd, binary_po_args, binary_po_dt, binary_po_fp),
+    SAFE(binary::setup_binary_po(const_pd, binary_po_args, binary_po_dt,
+                 binary_po_fp, ref_engine),
+            WARN);
+    std::vector<dnn_mem_t> prelu_po_fp, prelu_po_dt;
+    std::vector<int> prelu_po_args;
+    SAFE(prelu::setup_prelu_po(
+                 const_pd, prelu_po_args, prelu_po_fp, prelu_po_dt, ref_engine),
             WARN);
 
-    dnn_mem_t src_fp(src_md, fp, src_tag, test_engine);
-    dnn_mem_t wei_fp(wei_md, fp, wei_tag, test_engine);
-    dnn_mem_t dst_fp(dst_md, fp, src_tag, test_engine);
-    dnn_mem_t bia_fp(bia_md, fp, tag::x, test_engine);
+    dnn_mem_t src_fp(src_md, fp, src_tag, ref_engine);
+    dnn_mem_t wei_fp(wei_md, fp, wei_tag, ref_engine);
+    dnn_mem_t dst_fp(dst_md, fp, src_tag, ref_engine);
+    dnn_mem_t bia_fp(bia_md, fp, tag::x, ref_engine);
+    dnn_mem_t scratchpad_fp;
+    if (prim_ref)
+        scratchpad_fp = dnn_mem_t(q_ref(DNNL_ARG_SCRATCHPAD), ref_engine);
 
-    SAFE(fill_src(prb, src_dt, src_fp, res), WARN);
-    SAFE(fill_wei(prb, wei_dt, wei_fp, res), WARN);
-    SAFE(fill_dst(prb, dst_dt, dst_fp, res), WARN);
-    SAFE(fill_bia(prb, bia_dt, bia_fp, res), WARN);
-    maybe_prepare_runtime_scales(scales, prb->attr, prb->oc, prb->scales);
+    if (need_src_init(prb)) SAFE(fill_src(prb, src_dt, src_fp, res), WARN);
+    if (need_dst_init(prb)) SAFE(fill_dst(prb, dst_dt, dst_fp, res), WARN);
+    if (need_wei_init(prb)) SAFE(fill_wei(prb, wei_dt, wei_fp, res), WARN);
+    if (need_bia_init(prb)) SAFE(fill_bia(prb, bia_dt, bia_fp, res), WARN);
+
+    maybe_prepare_runtime_scales(
+            scales, prb->attr.oscale, prb->oc, prb->scales);
     maybe_prepare_runtime_zero_points(
             src_zero_points_m, prb->attr, DNNL_ARG_SRC, prb->ic, prb->src_zp);
     maybe_prepare_runtime_zero_points(
             dst_zero_points_m, prb->attr, DNNL_ARG_DST, prb->oc, prb->dst_zp);
 
-    args_t args;
+    args_t args, ref_args;
 
     if (prb->dir & FLAG_FWD) {
         args.set(DNNL_ARG_SRC, src_dt);
@@ -812,26 +958,38 @@ int doit(const prb_t *prb, res_t *res) {
         args.set(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC, src_zero_points_m);
         args.set(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST, dst_zero_points_m);
         args.set(binary_po_args, binary_po_dt);
+        args.set(prelu_po_args, prelu_po_dt);
 
-        SAFE(execute_and_wait(c, args), WARN);
+        SAFE(execute_and_wait(prim, args), WARN);
 
-        if (bench_mode & CORR) {
-            compute_ref_fwd(
-                    prb, c_ref, src_fp, wei_fp, bia_fp, binary_po_fp, dst_fp);
+        if (is_bench_mode(CORR)) {
+            ref_args.set(DNNL_ARG_SRC, src_fp);
+            ref_args.set(DNNL_ARG_WEIGHTS, wei_fp);
+            ref_args.set(DNNL_ARG_BIAS, bia_fp);
+            ref_args.set(DNNL_ARG_DST, dst_fp);
+            ref_args.set(DNNL_ARG_SCRATCHPAD, scratchpad_fp);
+            ref_args.set(binary_po_args, binary_po_fp);
+            ref_args.set(prelu_po_args, prelu_po_fp);
+
+            compute_ref_fwd(prb, prim_ref, ref_args);
             dnn_mem_t dst(dst_dt, fp, src_tag, test_engine);
             SAFE(compare_dst(prb, dst, dst_fp, res, true), WARN);
         }
     } else if (prb->dir == BWD_D) {
-        args.set(DNNL_ARG_DIFF_DST, dst_dt);
-        args.set(DNNL_ARG_WEIGHTS, wei_dt);
         args.set(DNNL_ARG_DIFF_SRC, src_dt);
+        args.set(DNNL_ARG_WEIGHTS, wei_dt);
+        args.set(DNNL_ARG_DIFF_DST, dst_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        SAFE(execute_and_wait(c, args), WARN);
+        SAFE(execute_and_wait(prim, args), WARN);
 
-        if (bench_mode & CORR) {
-            compute_ref_bwd_d(prb, c_ref, src_fp, wei_fp, bia_fp,
-                    std::vector<dnn_mem_t>(), dst_fp);
+        if (is_bench_mode(CORR)) {
+            ref_args.set(DNNL_ARG_DIFF_SRC, src_fp);
+            ref_args.set(DNNL_ARG_WEIGHTS, wei_fp);
+            ref_args.set(DNNL_ARG_DIFF_DST, dst_fp);
+            ref_args.set(DNNL_ARG_SCRATCHPAD, scratchpad_fp);
+
+            compute_ref_bwd_d(prb, prim_ref, ref_args);
             dnn_mem_t src(src_dt, fp, src_tag, test_engine);
             SAFE(compare_src(prb, src, src_fp, res, true), WARN);
         }
@@ -842,10 +1000,16 @@ int doit(const prb_t *prb, res_t *res) {
         args.set(DNNL_ARG_DIFF_BIAS, bia_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        SAFE(execute_and_wait(c, args), WARN);
+        SAFE(execute_and_wait(prim, args), WARN);
 
-        if (bench_mode & CORR) {
-            compute_ref_bwd_w(prb, c_ref, src_fp, wei_fp, bia_fp, dst_fp);
+        if (is_bench_mode(CORR)) {
+            ref_args.set(DNNL_ARG_SRC, src_fp);
+            ref_args.set(DNNL_ARG_DIFF_DST, dst_fp);
+            ref_args.set(DNNL_ARG_DIFF_WEIGHTS, wei_fp);
+            ref_args.set(DNNL_ARG_DIFF_BIAS, bia_fp);
+            ref_args.set(DNNL_ARG_SCRATCHPAD, scratchpad_fp);
+
+            compute_ref_bwd_w(prb, prim_ref, ref_args);
             dnn_mem_t wei(wei_dt, fp, wei_tag, test_engine);
             SAFE(compare_wei(prb, wei, wei_fp, res, true), WARN);
             if (prb->dir & FLAG_BIA) {
@@ -857,12 +1021,7 @@ int doit(const prb_t *prb, res_t *res) {
         SAFE(FAIL, CRIT);
     }
 
-    measure_perf(res->timer, c, args);
-
-    DNN_SAFE_V(dnnl_primitive_destroy(c));
-    DNN_SAFE_V(dnnl_primitive_destroy(c_ref));
-
-    return OK;
+    return measure_perf(res->timer, prim, args);
 }
 
 } // namespace conv
