@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,112 +14,23 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <iterator>
+
 #include <float.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
 
+#include "binary/binary.hpp"
 #include "conv/conv_dw_fusion.hpp"
 
 namespace conv_dw_fusion {
 
-// This function is copy-pasted from dnn_types.cpp. We want it here for now to
-// preserve current benchdnn attr semantics - what's set by user is remained
-// untouched - it saves the input state. When fusion is done, it's enough to
-// have only output data type, rest can be deduced from first conv.
-// TODO: consider merge with common code by changing benchdnn attr semantics.
-dnnl_primitive_attr_t create_dnnl_fusion_attr(
-        const prb_t *prb, int64_t scale_cnt) {
-    const attr_t &attr = prb->attr;
-
-    dnnl_primitive_attr_t dnnl_attr = nullptr;
-    DNN_SAFE_V(dnnl_primitive_attr_create(&dnnl_attr));
-
-    if (!attr.oscale.is_def()) {
-        int64_t count = attr.oscale.policy == policy_t::COMMON ? 1 : scale_cnt;
-        int scale_mask = attr.oscale.policy == policy_t::PER_OC ? 1 << 1 : 0;
-
-        float *scales = prb->scales;
-
-        const bool runtime = attr.oscale.runtime;
-        SAFE_V(scales == nullptr && runtime ? FAIL : OK);
-
-        float *gen_scs = nullptr;
-        if (scales == nullptr) {
-            gen_scs = (float *)zmalloc(count * sizeof(float), 64);
-            SAFE_V(gen_scs != nullptr ? OK : FAIL);
-            for (int64_t i = 0; i < count; ++i)
-                gen_scs[i] = attr.oscale.scale;
-            scales = gen_scs;
-        }
-
-        DNN_SAFE_V(dnnl_primitive_attr_set_output_scales(dnnl_attr,
-                runtime ? 1 : count, scale_mask,
-                runtime ? &DNNL_RUNTIME_F32_VAL : scales));
-        if (gen_scs) zfree(gen_scs);
-    }
-
-    if (!attr.post_ops.is_def()) {
-        const auto &po = attr.post_ops;
-        dnnl_post_ops_t ops;
-        DNN_SAFE_V(dnnl_post_ops_create(&ops));
-        for (int idx = 0; idx < po.len(); ++idx) {
-            const auto &e = po.entry[idx];
-            if (e.is_sum_kind()) {
-                DNN_SAFE_V(dnnl_post_ops_append_sum(ops, e.sum.scale));
-            } else if (e.is_convolution_kind()) {
-                const auto &os = e.convolution.oscale;
-                int count = 0, mask = 0;
-                const float *scales = &os.scale;
-                float *gen_scs = nullptr;
-                if (!os.is_def()) {
-                    count = os.policy == policy_t::COMMON ? 1 : scale_cnt;
-                    mask = os.policy == policy_t::PER_OC ? 1 << 1 : 0;
-                    if (mask > 0) {
-                        gen_scs = conv::generate_oscales(os, count);
-                        SAFE_V(gen_scs != nullptr ? OK : FAIL);
-                        scales = gen_scs;
-                    }
-                }
-                const auto wei_dt = prb->cfg[WEI].dt;
-                const auto bia_dt
-                        = prb->dir == FWD_B ? dnnl_f32 : dnnl_data_type_undef;
-
-                if (e.convolution.stride == 1) {
-                    DNN_SAFE_V(dnnl_post_ops_append_dw_k3s1p1(ops, wei_dt,
-                            bia_dt, e.convolution.dst_dt, count, mask, scales));
-                } else {
-                    DNN_SAFE_V(dnnl_post_ops_append_dw_k3s2p1(ops, wei_dt,
-                            bia_dt, e.convolution.dst_dt, count, mask, scales));
-                }
-                if (gen_scs) zfree(gen_scs);
-            } else if (e.is_eltwise_kind()) {
-                DNN_SAFE_V(dnnl_post_ops_append_eltwise(ops, e.eltwise.scale,
-                        e.eltwise.alg, e.eltwise.alpha, e.eltwise.beta));
-            } else {
-                assert(!"unknown attr::post_ops::kind");
-            }
-        }
-        DNN_SAFE_V(dnnl_primitive_attr_set_post_ops(dnnl_attr, ops));
-
-        const_dnnl_post_ops_t c_ops;
-        DNN_SAFE_V(dnnl_primitive_attr_get_post_ops(dnnl_attr, &c_ops));
-        SAFE_V(dnnl_post_ops_len(c_ops) == po.len() ? OK : FAIL);
-
-        DNN_SAFE_V(dnnl_post_ops_destroy(ops));
-    }
-
-    DNN_SAFE_V(dnnl_primitive_attr_set_scratchpad_mode(
-            dnnl_attr, attr.scratchpad_mode));
-
-    return dnnl_attr;
-}
-
 static int init_pd(dnnl_engine_t engine, const prb_t *prb,
         dnnl_primitive_desc_t &cpd, res_t *res, dir_t dir,
         const_dnnl_primitive_desc_t hint) {
@@ -129,6 +40,9 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     dnnl_dims_t src_1d_dims = {prb->mb, prb->ic, prb->iw};
     dnnl_dims_t src_2d_dims = {prb->mb, prb->ic, prb->ih, prb->iw};
     dnnl_dims_t src_3d_dims = {prb->mb, prb->ic, prb->id, prb->ih, prb->iw};
+    dnnl_dim_t *src_dims = prb->ndims == 5
+            ? src_3d_dims
+            : prb->ndims == 4 ? src_2d_dims : src_1d_dims;
 
     dnnl_dims_t wei_1d_dims
             = {prb->g, prb->oc / prb->g, prb->ic / prb->g, prb->kw};
@@ -136,45 +50,38 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
             = {prb->g, prb->oc / prb->g, prb->ic / prb->g, prb->kh, prb->kw};
     dnnl_dims_t wei_3d_dims = {prb->g, prb->oc / prb->g, prb->ic / prb->g,
             prb->kd, prb->kh, prb->kw};
+    dnnl_dim_t *wei_dims = prb->ndims == 5
+            ? &wei_3d_dims[!prb->has_groups]
+            : prb->ndims == 4 ? &wei_2d_dims[!prb->has_groups]
+                              : &wei_1d_dims[!prb->has_groups];
 
     dnnl_dims_t bia_dims = {prb->oc};
 
     dnnl_dims_t dst_1d_dims = {prb->mb, prb->oc, prb->ow};
     dnnl_dims_t dst_2d_dims = {prb->mb, prb->oc, prb->oh, prb->ow};
     dnnl_dims_t dst_3d_dims = {prb->mb, prb->oc, prb->od, prb->oh, prb->ow};
+    dnnl_dim_t *dst_dims = prb->ndims == 5
+            ? dst_3d_dims
+            : prb->ndims == 4 ? dst_2d_dims : dst_1d_dims;
 
     dnnl_data_type_t src_dt = prb->cfg[SRC].dt;
     dnnl_data_type_t wei_dt = prb->cfg[WEI].dt;
     dnnl_data_type_t bia_dt = prb->cfg[BIA].dt;
     dnnl_data_type_t dst_dt = prb->cfg[DST].dt;
     dnnl_data_type_t acc_dt = prb->cfg[ACC].dt;
-    std::string src_tag = prb->stag;
-    std::string wei_tag = prb->wtag;
     std::string bia_tag = tag::any;
     std::string dst_tag = prb->dtag;
 
-    SAFE(init_md(&src_d, prb->ndims,
-                 prb->ndims == 5 ? src_3d_dims
-                                 : prb->ndims == 3 ? src_1d_dims : src_2d_dims,
-                 src_dt, src_tag),
-            CRIT);
+    SAFE(init_md(&src_d, prb->ndims, src_dims, src_dt, prb->stag), WARN);
 
-    SAFE(init_md(&wei_d, prb->ndims + prb->has_groups,
-                 prb->ndims == 5
-                         ? &wei_3d_dims[!prb->has_groups]
-                         : prb->ndims == 3 ? &wei_1d_dims[!prb->has_groups]
-                                           : &wei_2d_dims[!prb->has_groups],
-                 wei_dt, wei_tag),
-            CRIT);
-
-    SAFE(init_md(&bia_d, 1, bia_dims, bia_dt, bia_tag), CRIT);
-
-    SAFE(init_md(&dst_d, prb->ndims,
-                 prb->ndims == 5 ? dst_3d_dims
-                                 : prb->ndims == 3 ? dst_1d_dims : dst_2d_dims,
-                 dst_dt, dst_tag),
+    SAFE(init_md(&wei_d, prb->ndims + prb->has_groups, wei_dims, wei_dt,
+                 prb->wtag),
             WARN);
 
+    SAFE(init_md(&bia_d, 1, bia_dims, bia_dt, bia_tag), WARN);
+
+    SAFE(init_md(&dst_d, prb->ndims, dst_dims, dst_dt, dst_tag), WARN);
+
     dnnl_dim_t strides_nd[] = {prb->sd, prb->sh, prb->sw};
     dnnl_dim_t dilates_nd[] = {prb->dd, prb->dh, prb->dw};
     dnnl_dim_t padding_nd[] = {prb->pd, prb->ph, prb->pw};
@@ -221,13 +128,19 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     DNN_SAFE(cd.accum_data_type == acc_dt ? dnnl_success : dnnl_unimplemented,
             CRIT);
 
-    auto dnnl_attr = create_dnnl_fusion_attr(prb, prb->oc);
+    attr_args_t attr_args;
+    attr_args.prepare_output_scales(prb->attr, prb->scales, prb->oc);
+
+    const auto dw_bia_dt = prb->dir == FWD_B ? dnnl_f32 : dnnl_data_type_undef;
+    attr_args.prepare_dw_post_op(
+            prb->attr, prb->cfg[WEI].dt, dw_bia_dt, prb->scales_dw, prb->oc);
+    attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, dst_dims);
+    auto dnnl_attr = make_benchdnn_dnnl_wrapper(
+            create_dnnl_attr(prb->attr, attr_args));
 
     dnnl_status_t init_status
             = dnnl_primitive_desc_create(&cpd, &cd, dnnl_attr, engine, nullptr);
 
-    dnnl_primitive_attr_destroy(dnnl_attr);
-
     if (init_status == dnnl_unimplemented) {
         if (res) res->state = UNIMPLEMENTED;
         return OK;
@@ -241,7 +154,6 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     if (maybe_skip(res->impl_name)) {
         BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
                 res->impl_name.c_str());
-        DNN_SAFE(dnnl_primitive_desc_destroy(cpd), WARN);
         return res->state = SKIPPED, res->reason = SKIP_IMPL_HIT, OK;
     } else {
         BENCHDNN_PRINT(
@@ -325,26 +237,32 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
     check_known_skipped_case_common(
             {prb->cfg[SRC].dt, prb->cfg[WEI].dt, prb->cfg[DST].dt}, prb->dir,
             res);
+
+    // GPU does not support depthwise fusion
+    if (is_gpu() && prb->attr.post_ops.convolution_index() != -1) {
+        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
 }
 
 int doit(const prb_t *prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
     check_known_skipped_case(prb, res);
+    check_sum_post_ops(prb->attr, res);
     if (res->state == SKIPPED) return OK;
 
     // Original problem with fusion attributes
-    dnnl_primitive_t c {};
-    SAFE(init_prim(&c, init_pd, prb, res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
+    SAFE(init_prim(prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_pd), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_pd), CRIT);
 
     // Check memory requirements only for original problem though it's broken
     // due to quering not by arg md.
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
-        DNN_SAFE_V(dnnl_primitive_destroy(c));
+    if (check_mem_size(const_pd) != OK) {
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
 
@@ -356,8 +274,9 @@ int doit(const prb_t *prb, res_t *res) {
     const auto adjust_alg = [](const_dnnl_primitive_desc_t pd, alg_t &alg) {
         if (alg == alg_t::AUTO) {
             dnnl_convolution_desc_t *temp_conv_desc = {nullptr};
-            DNN_SAFE_V(dnnl_primitive_desc_query(
-                    pd, dnnl_query_convolution_d, 0, &temp_conv_desc));
+            DNN_SAFE(dnnl_primitive_desc_query(
+                             pd, dnnl_query_convolution_d, 0, &temp_conv_desc),
+                    CRIT);
             alg = conv::alg_kind2alg(temp_conv_desc->alg_kind);
         }
         return OK;
@@ -397,16 +316,16 @@ int doit(const prb_t *prb, res_t *res) {
     dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     const auto fp = dnnl_f32;
-    const auto src_tag = tag::abx;
-    const auto wei_tag = tag::abx;
-    const auto fused_wei_tag = tag::abx;
-    dnn_mem_t src_fp(src_md, fp, src_tag, test_engine);
-    dnn_mem_t wei_fp(wei_md, fp, wei_tag, test_engine);
+    dnn_mem_t src_fp(src_md, fp, tag::abx, test_engine);
+    dnn_mem_t wei_fp(wei_md, fp, tag::abx, test_engine);
     dnn_mem_t bia_fp(bia_md, fp, tag::x, test_engine);
-    dnn_mem_t dst_fp(dst_md, fp, src_tag, test_engine);
-    dnn_mem_t fused_wei_fp(fused_wei_md, fp, fused_wei_tag, test_engine);
+    dnn_mem_t dst_fp(dst_md, fp, tag::abx, test_engine);
+    dnn_mem_t fused_wei_fp(fused_wei_md, fp, tag::abx, test_engine);
     dnn_mem_t fused_bia_fp(fused_bia_md, fp, tag::x, test_engine);
 
+    std::vector<dnn_mem_t> binary_po_dt;
+    std::vector<int> binary_po_args;
+
     // Current filling doesn't work for fused_wei due to relying on prb values,
     // which are different for fused conv. This can be fixed later by relying
     // on md values, rather than prb desc ones.
@@ -416,12 +335,12 @@ int doit(const prb_t *prb, res_t *res) {
     // Fill first convolution
     std::unique_ptr<prb_t> p0 = get_first_conv_prb(prb);
 
-    dnnl_primitive_t c0 {};
-    SAFE(init_prim(&c0, init_pd, p0.get(), res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim0;
+    SAFE(init_prim(prim0, init_pd, p0.get(), res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd0;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(c0, &const_pd0), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim0, &const_pd0), CRIT);
 
     const auto q0 = [&](int index = 0) -> const dnnl_memory_desc_t & {
         return *dnnl_primitive_desc_query_md(
@@ -444,19 +363,22 @@ int doit(const prb_t *prb, res_t *res) {
             = p0->dir & FLAG_BWD ? q0(DNNL_ARG_DIFF_DST) : q0(DNNL_ARG_DST);
     const auto &scratchpad_md0 = q0(DNNL_ARG_SCRATCHPAD);
 
-    const auto src_tag0 = tag::abx;
-    const auto wei_tag0 = tag::abx;
-
     dnn_mem_t src_dt0(src_md0, test_engine);
     dnn_mem_t wei_dt0(wei_md0, test_engine);
     dnn_mem_t bia_dt0(bia_md0, test_engine);
     dnn_mem_t dst_dt0(dst_md0, test_engine);
     dnn_mem_t scratchpad_dt0(scratchpad_md0, test_engine);
 
-    dnn_mem_t src_fp0(src_md0, fp, src_tag0, test_engine);
-    dnn_mem_t wei_fp0(wei_md0, fp, wei_tag0, test_engine);
+    dnn_mem_t src_fp0(src_md0, fp, tag::abx, test_engine);
+    dnn_mem_t wei_fp0(wei_md0, fp, tag::abx, test_engine);
     dnn_mem_t bia_fp0(bia_md0, fp, tag::x, test_engine);
-    dnn_mem_t dst_fp0(dst_md0, fp, src_tag0, test_engine);
+    dnn_mem_t dst_fp0(dst_md0, fp, tag::abx, test_engine);
+
+    std::vector<dnn_mem_t> binary_po_fp0, binary_po_dt0;
+    std::vector<int> binary_po_args0;
+    SAFE(binary::setup_binary_po(
+                 const_pd0, binary_po_args0, binary_po_dt0, binary_po_fp0),
+            WARN);
 
     SAFE(conv::fill_src(p0.get(), src_dt0, src_fp0, res), WARN);
     SAFE(conv::fill_wei(p0.get(), wei_dt0, wei_fp0, res), WARN);
@@ -467,12 +389,12 @@ int doit(const prb_t *prb, res_t *res) {
     std::unique_ptr<prb_t> p1 = get_fused_conv_prb(prb);
     if (!p1) SAFE(FAIL, CRIT);
 
-    dnnl_primitive_t c1 {};
-    SAFE(init_prim(&c1, init_pd, p1.get(), res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim1;
+    SAFE(init_prim(prim1, init_pd, p1.get(), res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd1;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(c1, &const_pd1), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim1, &const_pd1), CRIT);
 
     const auto q1 = [&](int index = 0) -> const dnnl_memory_desc_t & {
         return *dnnl_primitive_desc_query_md(
@@ -496,17 +418,21 @@ int doit(const prb_t *prb, res_t *res) {
             = prb->dir & FLAG_BWD ? q1(DNNL_ARG_DIFF_DST) : q1(DNNL_ARG_DST);
     const auto &scratchpad_md1 = q(DNNL_ARG_SCRATCHPAD);
 
-    const auto src_tag1 = tag::abx;
-    const auto wei_tag1 = tag::abx;
     dnn_mem_t src_dt1(src_md1, test_engine);
     dnn_mem_t wei_dt1(wei_md1, test_engine);
     dnn_mem_t bia_dt1(bia_md1, test_engine);
     dnn_mem_t dst_dt1(dst_md1, test_engine);
     dnn_mem_t scratchpad_dt1(scratchpad_md1, test_engine);
 
-    dnn_mem_t wei_fp1(wei_md1, fp, wei_tag1, test_engine);
+    dnn_mem_t wei_fp1(wei_md1, fp, tag::abx, test_engine);
     dnn_mem_t bia_fp1(bia_md1, fp, tag::x, test_engine);
-    dnn_mem_t dst_fp1(dst_md1, fp, src_tag1, test_engine);
+    dnn_mem_t dst_fp1(dst_md1, fp, tag::abx, test_engine);
+
+    std::vector<dnn_mem_t> binary_po_fp1, binary_po_dt1;
+    std::vector<int> binary_po_args1;
+    SAFE(binary::setup_binary_po(
+                 const_pd1, binary_po_args1, binary_po_dt1, binary_po_fp1),
+            WARN);
 
     SAFE(conv::fill_wei(p1.get(), wei_dt1, wei_fp1, res), WARN);
     SAFE(conv::fill_bia(p1.get(), bia_dt1, bia_fp1, res), WARN);
@@ -537,8 +463,9 @@ int doit(const prb_t *prb, res_t *res) {
         args0.set(DNNL_ARG_BIAS, bia_dt0);
         args0.set(DNNL_ARG_DST, dst_dt0);
         args0.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt0);
+        args0.set(binary_po_args0, binary_po_dt0);
 
-        SAFE(execute_and_wait(c0, args0), WARN);
+        SAFE(execute_and_wait(prim0, args0), WARN);
         SAFE(src_dt1.reorder(dst_dt0), WARN);
 
         args1.set(DNNL_ARG_SRC, src_dt1);
@@ -546,24 +473,51 @@ int doit(const prb_t *prb, res_t *res) {
         args1.set(DNNL_ARG_BIAS, bia_dt1);
         args1.set(DNNL_ARG_DST, dst_dt1);
         args1.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt1);
+        args1.set(binary_po_args1, binary_po_dt1);
 
-        SAFE(execute_and_wait(c1, args1), WARN);
+        SAFE(execute_and_wait(prim1, args1), WARN);
+
+        // Reverse engineer binary post-ops indices from second conv and update
+        // them in-place to follow fused conv enumaration.
+        const int dw_idx = prb->attr.post_ops.convolution_index();
+        const auto update_bin_po_args1_indices = [&](size_t i) {
+            auto &b = binary_po_args1[i];
+            const int orig_idx = b / DNNL_ARG_ATTR_MULTIPLE_POST_OP_BASE - 1;
+            b = DNNL_ARG_ATTR_MULTIPLE_POST_OP(orig_idx + dw_idx + 1)
+                    | DNNL_ARG_SRC_1;
+        };
+        for (size_t i = 0; i < binary_po_dt1.size(); ++i)
+            update_bin_po_args1_indices(i);
+
+        // As memory is not allowed to be copied, and binary post-op memories
+        // are read-only, we move them to main convolution execution and adjust
+        // arg indices to follow the library API.
+
+        // Move the content to binary_po_dt from separate convs.
+        std::move(binary_po_dt0.begin(), binary_po_dt0.end(),
+                std::back_inserter(binary_po_dt));
+        std::move(binary_po_dt1.begin(), binary_po_dt1.end(),
+                std::back_inserter(binary_po_dt));
+        // Move the content to binary_po_args from separate convs.
+        std::move(binary_po_args0.begin(), binary_po_args0.end(),
+                std::back_inserter(binary_po_args));
+        std::move(binary_po_args1.begin(), binary_po_args1.end(),
+                std::back_inserter(binary_po_args));
 
         args.set(DNNL_ARG_SRC, src_dt);
         args.set(DNNL_ARG_WEIGHTS, wei_dt);
-        if (bia_md.data_type != dnnl_data_type_undef)
-            args.set(DNNL_ARG_BIAS, bia_dt);
+        args.set(DNNL_ARG_BIAS, bia_dt);
         args.set(DNNL_ARG_DST, dst_dt);
         args.set(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS, fused_wei_dt);
-        if (fused_bia_md.data_type != dnnl_data_type_undef)
-            args.set(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS, fused_bia_dt);
+        args.set(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS, fused_bia_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
+        args.set(binary_po_args, binary_po_dt);
 
-        SAFE(execute_and_wait(c, args), WARN);
+        SAFE(execute_and_wait(prim, args), WARN);
 
-        if (bench_mode & CORR) {
-            dnn_mem_t dst_fused(dst_dt, fp, src_tag, test_engine);
-            dnn_mem_t dst_unfused(dst_dt1, fp, src_tag, test_engine);
+        if (is_bench_mode(CORR)) {
+            dnn_mem_t dst_fused(dst_dt, fp, tag::abx, test_engine);
+            dnn_mem_t dst_unfused(dst_dt1, fp, tag::abx, test_engine);
             // Used p1 to avoid writing separate compare function. Compare uses
             // prb->cfg which can be u8s8u8 while after fusion it may be u8s8s8,
             // thus, compare() will saturate values which is not correct.
@@ -571,16 +525,11 @@ int doit(const prb_t *prb, res_t *res) {
                     WARN);
         }
     } else {
+        assert(!"Backward is not supported");
         SAFE(FAIL, CRIT);
     }
 
-    measure_perf(res->timer, c, args);
-
-    DNN_SAFE_V(dnnl_primitive_destroy(c));
-    DNN_SAFE_V(dnnl_primitive_destroy(c0));
-    DNN_SAFE_V(dnnl_primitive_destroy(c1));
-
-    return OK;
+    return measure_perf(res->timer, prim, args);
 }
 
 } // namespace conv_dw_fusion
