@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,12 +20,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
 
 #include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
+#include "utils/compare.hpp"
 
 #include "binary/binary.hpp"
 #include "matmul/matmul.hpp"
@@ -61,13 +62,15 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
             = get_runtime_dims(prb->dst_dims(), prb->dst_runtime_dim_mask());
 
     SAFE(init_md(&src_d, prb->ndims, src_rt_dims.data(), prb->cfg[SRC].dt,
-                 prb->stag),
+                 prb->stag, prb->strides[STRIDES_SRC]),
             CRIT);
+
     SAFE(init_md(&wei_d, prb->ndims, weights_rt_dims.data(), prb->cfg[WEI].dt,
-                 prb->wtag),
+                 prb->wtag, prb->strides[STRIDES_WEI]),
             CRIT);
+
     SAFE(init_md(&dst_d, prb->ndims, dst_rt_dims.data(), prb->cfg[DST].dt,
-                 prb->dtag),
+                 prb->dtag, prb->strides[STRIDES_DST]),
             CRIT);
 
     if (prb->bia_dt != dnnl_data_type_undef) {
@@ -94,14 +97,14 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     attr_args_t attr_args;
     const auto &dst_dims = prb->dst_dims();
     attr_args.prepare_output_scales(prb->attr, prb->scales, prb->n, mask);
-    attr_args.prepare_binary_post_op_mds(
-            prb->attr, prb->ndims, dst_dims.data());
-    auto dnnl_attr = create_dnnl_attr(prb->attr, attr_args);
+    attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, dst_dims.data());
+    auto dnnl_attr = make_benchdnn_dnnl_wrapper(
+            create_dnnl_attr(prb->attr, attr_args));
 
-    dnnl_status_t init_status = dnnl_success;
-    init_status = dnnl_primitive_desc_create(
+    dnnl_status_t init_status = dnnl_primitive_desc_create(
             &mpd, &op_d, dnnl_attr, engine, nullptr);
-    dnnl_primitive_attr_destroy(dnnl_attr);
+
+    if (!res) return OK;
 
     if (init_status == dnnl_unimplemented)
         return res->state = UNIMPLEMENTED, OK;
@@ -112,87 +115,80 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     if (maybe_skip(res->impl_name)) {
         BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
                 res->impl_name.c_str());
-        DNN_SAFE(dnnl_primitive_desc_destroy(mpd), WARN);
         return res->state = SKIPPED, res->reason = SKIP_IMPL_HIT, OK;
     } else {
         BENCHDNN_PRINT(
                 5, "oneDNN implementation: %s\n", res->impl_name.c_str());
     }
 
+    SAFE(check_pd_w_and_wo_attr(res, prb->attr, op_d), WARN);
+
     return OK;
 }
 
-int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
-        dnn_mem_t &mem_fp, res_t *res) {
-    const auto nelems = mem_dt.nelems();
-    if (nelems == 0) return res->state = PASSED, OK;
+int init_prim_ref(
+        benchdnn_dnnl_wrapper_t<dnnl_primitive_t> &prim_ref, const prb_t *prb) {
+    if (!(is_bench_mode(CORR) && is_gpu() && fast_ref_gpu)) return OK;
 
-    res->total = nelems;
+    // Create a new copy of prb to avoid potentially corrupting the test by
+    // modifying prb in place.
+    const auto cpu_bia_dt = prb->bia_dt == dnnl_data_type_undef
+            ? dnnl_data_type_undef
+            : dnnl_f32;
+    const auto cpu_bia_mask
+            = prb->bia_dt == dnnl_data_type_undef ? 0 : prb->bia_mask;
+    auto cpu_attr = prb->attr;
+    update_cpu_ref_attrs(cpu_attr);
+    prb_t prb_cpu {*prb, conf_f32, tag::abx, tag::abx, tag::abx,
+            {strides_t(STRIDES_SIZE)}, false, false, false, false, cpu_bia_dt,
+            cpu_bia_mask, {0, 0, 0}, cpu_attr};
 
-    int64_t non_zero = 0;
-    const char *skind = data_kind2str(kind);
+    dnnl_primitive_desc_t pd_ref_ {};
+    SAFE(init_pd(get_cpu_engine(), &prb_cpu, pd_ref_, nullptr, FLAG_FWD,
+                 nullptr),
+            WARN);
+    auto pd_ref = make_benchdnn_dnnl_wrapper(pd_ref_);
 
-    for (int64_t i = 0; i < nelems; ++i) {
-        const float dt = mem_dt.get_elem(i);
-        const float fp0 = mem_fp.get_elem(i);
-        const float fp = round_to_nearest_representable(prb->cfg[kind].dt, fp0);
-
-        const float diff = fabsf(fp - dt);
-        const float rel_diff = diff / (fabsf(fp) > FLT_MIN ? fabsf(fp) : 1);
-        bool ok = (fabs(fp) > 1e-5 ? rel_diff : diff) <= prb->cfg[kind].eps;
-
-        // XXX: if reference fp0 value is nan, allow to return anything from the
-        // library for integral target data types.
-        if (!ok) ok = std::isnan(fp0) && is_integral_dt(prb->cfg[kind].dt);
-
-        res->errors += !ok;
-
-        const bool dump = false || (!ok && (res->errors < 10 || verbose >= 10))
-                || (verbose >= 50 && i < 30) || (verbose >= 99);
-        if (dump) {
-            BENCHDNN_PRINT(0,
-                    "[%4ld][%s]"
-                    "fp:%8g fp0:%8g dt:%8g diff:%8g rdiff:%8g\n",
-                    (long)i, skind, fp, fp0, dt, diff, rel_diff);
-        }
-        non_zero += fp != 0;
+    dnnl_primitive_t prim_ref_ {};
+    if (pd_ref) {
+        DNN_SAFE(dnnl_primitive_create(&prim_ref_, pd_ref), WARN);
+        BENCHDNN_PRINT(
+                5, "%s\n", "benchdnn: use CPU primitive as the reference");
     }
-
-    const double trust_nz = (double)non_zero / res->total;
-    bool no_trust = trust_nz < 0.1;
-    if (no_trust) {
-        res->state = MISTRUSTED;
-        const char *skind = data_kind2str(kind);
-        BENCHDNN_PRINT(0,
-                "@@@ [%s] test-bug: trust is too low."
-                " Nonzeros in output: %.2f\n",
-                skind, trust_nz);
-    }
-
-    if (res->errors) res->state = FAILED;
-
-    if (res->state == UNTESTED) res->state = PASSED; /* optimism */
-
-    return res->state == FAILED ? FAIL : OK;
+    prim_ref.reset(prim_ref_);
+    return OK;
 }
 
 int fill_data(data_kind_t kind, const prb_t *prb, dnn_mem_t &mem_dt,
-        dnn_mem_t &mem_fp, res_t *res) {
+        dnn_mem_t &mem_fp, res_t *res,
+        dnnl_data_type_t sum_dt = dnnl_data_type_undef) {
+
     const auto nelems = mem_dt.nelems();
     if (nelems == 0) return OK;
 
     assert(mem_dt.nelems() == mem_fp.nelems());
 
     const auto &c = prb->cfg[kind];
-    float c_f_min = c.f_min, c_f_max = c.f_max;
+    float c_f_min = c.f_min, c_f_max = c.f_max, c_f_scale = c.f_scale;
 
     if (kind == BIA && mem_dt.dt() == dnnl_u8) c_f_min = 0;
 
+    const bool dst_with_diff_sum_dt = kind == DST
+            && sum_dt != dnnl_data_type_undef && sum_dt != mem_dt.dt();
+    if (dst_with_diff_sum_dt) {
+        mem_dt.set_dt(sum_dt);
+        if (sum_dt == dnnl_s8 || sum_dt == dnnl_u8) {
+            c_f_min = lowest_dt(sum_dt);
+            c_f_max = max_dt(sum_dt);
+        }
+        if (sum_dt == dnnl_s32) c_f_scale = 1;
+    }
+
     /* Do fixed partitioning to have same filling for any number of threads */
     const int64_t n_chunks = 16;
     const int64_t chunk_size = div_up(nelems, n_chunks);
 
-    dnnl::impl::parallel_nd(n_chunks, [&](int idx_chunk) {
+    dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx_chunk) {
         int64_t idx_start = idx_chunk * chunk_size;
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         // Note: we use a different seed for each chunk to avoid
@@ -209,28 +205,33 @@ int fill_data(data_kind_t kind, const prb_t *prb, dnn_mem_t &mem_dt,
             float val = 0;
             while (val == 0)
                 val = (float)gen(msr);
-            mem_fp.set_elem(0, val * c.f_scale);
+            mem_fp.set_elem(0, val * c_f_scale);
             idx_start += 1;
         }
 
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
-            auto val = (float)gen(msr) * c.f_scale;
+            auto val = (float)gen(msr) * c_f_scale;
             mem_fp.set_elem(idx, val);
         }
     });
 
     // work-around mistrusted when A > 0 && B < 0  && C.dt = u8 (or relu)
     if (kind == WEI && nelems == 1 && prb->cfg[SRC].dt == dnnl_u8) {
-        if (c.f_max >= 1) mem_fp.set_elem(0, c.f_scale);
+        if (c.f_max >= 1) mem_fp.set_elem(0, c_f_scale);
     }
 
     SAFE(mem_dt.reorder(mem_fp), WARN);
+    if (dst_with_diff_sum_dt) mem_dt.set_dt(prb->cfg[DST].dt);
     return OK;
 }
 
 void check_known_skipped_case(const prb_t *prb, res_t *res) {
     check_known_skipped_case_common(
-            {prb->cfg[SRC].dt, prb->cfg[WEI].dt, prb->cfg[DST].dt}, FWD_D, res);
+            {prb->cfg[SRC].dt, prb->cfg[WEI].dt, prb->bia_dt, prb->cfg[DST].dt},
+            FWD_D, res);
+    if (res->state == SKIPPED) return;
+
+    check_sum_post_ops(prb->attr, res, prb->cfg[DST].dt);
     if (res->state == SKIPPED) return;
 
     // zero points for non-integral data type does not make sense
@@ -277,6 +278,87 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
             return;
         }
     }
+
+    if (is_nvidia_gpu()) {
+        const auto &po = prb->attr.post_ops;
+        bool post_ops_ok = true;
+        for (int i = 0; i < po.len(); ++i) {
+            const auto &e = po.entry[i];
+            if (e.is_sum_kind())
+                continue;
+            else if (e.is_eltwise_kind())
+                post_ops_ok = post_ops_ok && is_nvidia_eltwise_ok(FLAG_FWD, e);
+            else if (e.is_binary_kind() || e.is_convolution_kind())
+                post_ops_ok = false;
+            else
+                assert(!"unknown post-op type");
+        }
+
+        const bool oscale_ok = prb->attr.oscale.policy == policy_t::COMMON;
+        const bool zp_ok = prb->attr.zero_points.is_def();
+        const bool dims_ok = prb->ndims <= 3;
+        const bool batch_bcast_ok = IMPLICATION(
+                prb->ndims == 3, prb->src_dims()[0] == prb->weights_dims()[0]);
+
+        if (!post_ops_ok || !oscale_ok || !zp_ok || !dims_ok
+                || !batch_bcast_ok) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+
+    // skip gpu testing for zero points policy other than COMMON
+    if (is_gpu()) {
+        if (prb->attr.zero_points.get(DNNL_ARG_SRC).policy != policy_t::COMMON
+                || prb->attr.zero_points.get(DNNL_ARG_DST).policy
+                        != policy_t::COMMON) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+
+    // skip gpu testing for non-default sum_dt
+    if (is_gpu()) {
+        const auto &po = prb->attr.post_ops;
+        const int sum_idx = po.find(attr_t::post_ops_t::kind_t::SUM);
+        if (sum_idx != -1 && po.entry[sum_idx].sum.dt != dnnl_data_type_undef) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+
+    // skip gpu testing for int8 with bf16 dst_dt with:
+    // * dst zero-point
+    // * any runtime dimensions
+    // * batched problem
+    if (is_gpu()) {
+        const bool is_s8_wei = prb->cfg[WEI].dt == dnnl_s8;
+        const bool is_bf16_dst = prb->cfg[DST].dt == dnnl_bf16;
+        const bool rt_dims_are_none = src_rt_mask.none() && wei_rt_mask.none()
+                && dst_rt_mask.none();
+        if (is_s8_wei && is_bf16_dst
+                && (!prb->attr.zero_points.get(DNNL_ARG_DST).is_def()
+                        || !rt_dims_are_none || prb->ndims > 2)) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+
+    // skip bf16 bias for:
+    // * any batch and non-bf16 config
+    // * 2+D batch and any config
+    if (is_gpu()) {
+        const bool is_bf16 = prb->cfg[SRC].dt == dnnl_bf16
+                && prb->cfg[WEI].dt == dnnl_bf16
+                && (prb->cfg[DST].dt == dnnl_bf16
+                        || prb->cfg[DST].dt == dnnl_f32);
+        if (prb->bia_dt == dnnl_bf16
+                && ((prb->ndims > 2 && !is_bf16)
+                        || (prb->ndims > 3 && is_bf16))) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
 }
 
 int doit(const prb_t *prb, res_t *res) {
@@ -285,15 +367,14 @@ int doit(const prb_t *prb, res_t *res) {
     check_known_skipped_case(prb, res);
     if (res->state == SKIPPED) return OK;
 
-    dnnl_primitive_t m {};
-    SAFE(init_prim(&m, init_pd, prb, res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
+    SAFE(init_prim(prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(m, &const_pd), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_pd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
-        DNN_SAFE(dnnl_primitive_destroy(m), CRIT);
+    if (check_mem_size(const_pd) != OK) {
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
 
@@ -316,15 +397,15 @@ int doit(const prb_t *prb, res_t *res) {
     if (dnnl_memory_desc_equal(&src_md, &def_md)) {
         assert(prb->stag != tag::any);
         SAFE(init_md(&src_md, prb->ndims, src_dims.data(), prb->cfg[SRC].dt,
-                     prb->stag),
+                     prb->stag, prb->strides[STRIDES_SRC]),
                 WARN);
     }
 
     const auto &weights_dims = prb->weights_dims();
     if (dnnl_memory_desc_equal(&wei_md, &def_md)) {
-        assert(prb->wtag != "any");
+        assert(prb->wtag != tag::any);
         SAFE(init_md(&wei_md, prb->ndims, weights_dims.data(), prb->cfg[WEI].dt,
-                     prb->wtag),
+                     prb->wtag, prb->strides[STRIDES_WEI]),
                 WARN);
     }
 
@@ -332,7 +413,7 @@ int doit(const prb_t *prb, res_t *res) {
     if (dnnl_memory_desc_equal(&dst_md, &def_md)) {
         assert(prb->dtag != tag::any);
         SAFE(init_md(&dst_md, prb->ndims, dst_dims.data(), prb->cfg[DST].dt,
-                     prb->dtag),
+                     prb->dtag, prb->strides[STRIDES_DST]),
                 WARN);
     }
     if (prb->bia_dt != dnnl_data_type_undef) {
@@ -345,7 +426,21 @@ int doit(const prb_t *prb, res_t *res) {
 
     const auto &scratchpad_md = q(DNNL_ARG_SCRATCHPAD);
 
+    // Use CPU prim as the reference in GPU testing to reduce testing time.
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim_ref;
+    SAFE(init_prim_ref(prim_ref, prb), WARN);
+
+    const_dnnl_primitive_desc_t const_pd_ref;
+    if (prim_ref)
+        DNN_SAFE(dnnl_primitive_get_primitive_desc(prim_ref, &const_pd_ref),
+                CRIT);
+    const auto q_ref = [&](int index = 0) -> const dnnl_memory_desc_t & {
+        return *dnnl_primitive_desc_query_md(
+                const_pd_ref, dnnl_query_exec_arg_md, index);
+    };
+
     const auto &test_engine = get_test_engine();
+    const auto &ref_engine = prim_ref ? get_cpu_engine() : get_test_engine();
 
     dnn_mem_t src_dt(src_md, test_engine);
     dnn_mem_t wei_dt(wei_md, test_engine);
@@ -356,36 +451,45 @@ int doit(const prb_t *prb, res_t *res) {
     dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     const auto fp = dnnl_f32;
-    dnn_mem_t src_fp(prb->ndims, src_md.dims, fp, nullptr, test_engine);
-    dnn_mem_t wei_fp(prb->ndims, wei_md.dims, fp, nullptr, test_engine);
-    dnn_mem_t dst_fp(prb->ndims, dst_md.dims, fp, nullptr, test_engine);
+    dnn_mem_t src_fp(src_md, fp, tag::abx, ref_engine);
+    dnn_mem_t wei_fp(wei_md, fp, tag::abx, ref_engine);
+    dnn_mem_t dst_fp(dst_md, fp, tag::abx, ref_engine);
     dnn_mem_t bia_fp;
     if (prb->bia_dt != dnnl_data_type_undef)
-        bia_fp = dnn_mem_t(prb->ndims, bia_md.dims, fp, nullptr, test_engine);
+        bia_fp = dnn_mem_t(bia_md, fp, tag::abx, ref_engine);
+    dnn_mem_t scratchpad_fp;
+    if (prim_ref)
+        scratchpad_fp = dnn_mem_t(q_ref(DNNL_ARG_SCRATCHPAD), ref_engine);
 
     SAFE(fill_data(SRC, prb, src_dt, src_fp, res), WARN);
     SAFE(fill_data(WEI, prb, wei_dt, wei_fp, res), WARN);
-    SAFE(fill_data(DST, prb, dst_dt, dst_fp, res), WARN);
+    const int sum_idx = prb->attr.post_ops.find(attr_t::post_ops_t::SUM);
+    if (sum_idx >= 0) {
+        const auto sum_dt = prb->attr.post_ops.entry[sum_idx].sum.dt;
+        SAFE(fill_data(DST, prb, dst_dt, dst_fp, res, sum_dt), WARN);
+    }
     if (prb->bia_dt != dnnl_data_type_undef)
         SAFE(fill_data(BIA, prb, bia_dt, bia_fp, res), WARN);
 
     dnn_mem_t scales;
     dnn_mem_t src_zero_points_m, wei_zero_points_m, dst_zero_points_m;
-    maybe_prepare_runtime_scales(scales, prb->attr, prb->n, prb->scales);
+    const auto &wei_zero_point_val
+            = prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).value;
+    maybe_prepare_runtime_scales(scales, prb->attr.oscale, prb->n, prb->scales);
     maybe_prepare_runtime_zero_points(
-            src_zero_points_m, prb->attr, DNNL_ARG_SRC);
+            src_zero_points_m, prb->attr, DNNL_ARG_SRC, prb->k, prb->src_zp);
+    maybe_prepare_runtime_zero_points(wei_zero_points_m, prb->attr,
+            DNNL_ARG_WEIGHTS, 1, &(wei_zero_point_val));
     maybe_prepare_runtime_zero_points(
-            wei_zero_points_m, prb->attr, DNNL_ARG_WEIGHTS);
-    maybe_prepare_runtime_zero_points(
-            dst_zero_points_m, prb->attr, DNNL_ARG_DST);
+            dst_zero_points_m, prb->attr, DNNL_ARG_DST, prb->n, prb->dst_zp);
 
     std::vector<dnn_mem_t> binary_po_fp, binary_po_dt;
     std::vector<int> binary_po_args;
-    SAFE(binary::setup_binary_po(
-                 const_pd, binary_po_args, binary_po_dt, binary_po_fp),
+    SAFE(binary::setup_binary_po(const_pd, binary_po_args, binary_po_dt,
+                 binary_po_fp, ref_engine, false, true),
             WARN);
 
-    args_t args;
+    args_t args, ref_args;
 
     args.set(DNNL_ARG_SRC, src_dt);
     args.set(DNNL_ARG_WEIGHTS, wei_dt);
@@ -398,20 +502,26 @@ int doit(const prb_t *prb, res_t *res) {
     args.set(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST, dst_zero_points_m);
     args.set(binary_po_args, binary_po_dt);
 
-    SAFE(execute_and_wait(m, args), WARN);
+    SAFE(execute_and_wait(prim, args), WARN);
 
-    if (bench_mode & CORR) {
-        compute_ref(
-                test_engine, prb, src_fp, wei_fp, bia_fp, binary_po_fp, dst_fp);
-        dnn_mem_t c(dst_dt, fp, tag::abx, test_engine);
-        SAFE(compare_dat(prb, DST, c, dst_fp, res), WARN);
+    if (is_bench_mode(CORR)) {
+        ref_args.set(DNNL_ARG_SRC, src_fp);
+        ref_args.set(DNNL_ARG_WEIGHTS, wei_fp);
+        if (prb->bia_dt != dnnl_data_type_undef)
+            ref_args.set(DNNL_ARG_BIAS, bia_fp);
+        ref_args.set(DNNL_ARG_DST, dst_fp);
+        ref_args.set(DNNL_ARG_SCRATCHPAD, scratchpad_fp);
+        ref_args.set(binary_po_args, binary_po_fp);
+
+        compute_ref(prb, prim_ref, ref_args);
+        compare::compare_t cmp;
+        cmp.set_threshold(prb->cfg[DST].eps);
+        cmp.set_data_kind(DST);
+        cmp.set_zero_trust_percent(90.f); // TODO: why so bad filling?
+        SAFE(cmp.compare(dst_fp, dst_dt, prb->attr, res), WARN);
     }
 
-    measure_perf(res->timer, m, args);
-
-    DNN_SAFE_V(dnnl_primitive_destroy(m));
-
-    return OK;
+    return measure_perf(res->timer, prim, args);
 }
 
 } // namespace matmul
