@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,8 +19,13 @@
 #include <random>
 #include <stdio.h>
 #include <stdlib.h>
+#include <type_traits>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+#include "tests/test_isa_common.hpp"
+#endif
 
 #include "tests/test_thread.hpp"
 
@@ -31,18 +36,66 @@
 #include "rnn/rnn.hpp"
 #include "rnn/rnn_aux.hpp"
 
-#define COMPARE_DAT(rc, kind, a, lay) \
+#define COMPARE_DAT(kind, a, lay) \
     do { \
         dnn_mem_t CONCAT2(a, _dt_plain)( \
                 CONCAT2(a, _dt), fp, lay, test_engine); \
-        (rc) |= compare_dat( \
-                prb, kind, CONCAT2(a, _dt_plain), CONCAT2(a, _fp), res, true); \
+        SAFE(compare_dat(prb, kind, CONCAT2(a, _dt_plain), CONCAT2(a, _fp), \
+                     res, true), \
+                WARN); \
     } while (0)
 
 // Using hidden attr API for testing RNN
 dnnl_status_t dnnl_primitive_attr_set_rnn_tparams(dnnl_primitive_attr_t attr,
         bool mode, dnnl_dim_t ngates, const float *scales, float cscale);
 
+namespace {
+
+// In order to have consistent filling across compilers and operating systems,
+// we implement the equivalent of std::normal_distribution using the so-called
+// Marsaglia polar method.
+template <typename T>
+class normal_distribution_t {
+public:
+    normal_distribution_t(T mean, T stddev)
+        : gen(-1.f, 1.f)
+        , is_odd_(false)
+        , odd_(1.f)
+        , mean_(mean)
+        , stddev_(stddev) {
+        static_assert(std::is_floating_point<T>::value,
+                "T must be a floating point type.");
+    }
+    template <typename URNG>
+    T operator()(URNG &g) {
+        T r, r2, x, y;
+        if (is_odd_) {
+            is_odd_ = false;
+            return odd_;
+        }
+        is_odd_ = true;
+        do {
+            x = gen(g); // x E [-1, 1)
+            y = gen(g); // y E [-1, 1)
+            r2 = x * x + y * y;
+        } while (0.f == r2 || 1.f < r2); // r2 E (0, 1]
+        r = stddev_ * std::sqrt(-2.f * std::log(r2) / r2);
+        x = mean_ + x * r;
+        y = mean_ + y * r;
+        odd_ = x;
+        return y;
+    }
+
+private:
+    std::uniform_real_distribution<T> gen;
+    bool is_odd_;
+    T odd_;
+    const T mean_;
+    const T stddev_;
+};
+
+} // namespace
+
 namespace rnn {
 
 void create_dnnl_rnn_attr(const prb_t &prb, dnnl_primitive_attr_t *dnnl_attr) {
@@ -55,6 +108,11 @@ void create_dnnl_rnn_attr(const prb_t &prb, dnnl_primitive_attr_t *dnnl_attr) {
     DNN_SAFE_V(dnnl_primitive_attr_set_rnn_weights_qparams(
             *dnnl_attr, prb.wei_nscales, prb.wei_scales_mask, prb.wei_scales));
 
+    if (prb.is_lstm_projection() && prb.is_int8())
+        DNN_SAFE_V(dnnl_primitive_attr_set_rnn_weights_projection_qparams(
+                *dnnl_attr, prb.wei_proj_nscales, prb.wei_proj_scales_mask,
+                prb.wei_proj_scales));
+
     if (prb.data_scale != 1.0 || prb.data_shift != 0.0)
         DNN_SAFE_V(dnnl_primitive_attr_set_rnn_data_qparams(
                 *dnnl_attr, prb.data_scale, prb.data_shift));
@@ -63,10 +121,51 @@ void create_dnnl_rnn_attr(const prb_t &prb, dnnl_primitive_attr_t *dnnl_attr) {
             *dnnl_attr, prb.attr.scratchpad_mode));
 }
 
-int check_s8s8_reorder(
-        const prb_t &prb, const dnn_mem_t &mem_dt, const dnn_mem_t &mem_fp) {
+int check_ldoi_s8_reorder(const prb_t &prb, data_kind_t kind,
+        const dnn_mem_t &mem_dt, const dnn_mem_t &mem_fp,
+        const_dnnl_primitive_attr_t attr = nullptr) {
     // TODO: enable for all cpu_kind when supported
-    if (engine_tgt_kind != dnnl_cpu) return OK;
+    if (is_gpu()) return OK;
+
+    // we compare ldio_f32 -> ldio_s8 to ldio_f32 -> ldoi_f32 -> ldio_s8
+    // fp is in ldio
+    dnn_mem_t mem_ldoi_f32(mem_fp.md_, dnnl_f32, "ldoi", get_test_engine());
+    dnn_mem_t mem_s8_src(mem_dt.md_, dnnl_s8, get_test_engine());
+
+    mem_ldoi_f32.reorder(mem_fp);
+    mem_s8_src.reorder(mem_ldoi_f32, attr);
+
+    dnn_mem_t mem_s8_dst(mem_dt.md_, dnnl_s8, get_test_engine());
+    mem_s8_dst.reorder(mem_fp, attr);
+
+    // we check that the two are identical
+    auto sz = mem_dt.size();
+    uint8_t *s8_src_handle = (uint8_t *)mem_s8_src;
+    uint8_t *s8_dst_handle = (uint8_t *)mem_s8_dst;
+
+    // check that both have the same size
+    assert(mem_s8_src.size() == mem_s8_dst.size());
+    // check that both have the same alignment modulo align_data in
+    // gemm_pack_storage.hpp
+    assert((uint64_t)s8_src_handle % 0x1000
+            == (uint64_t)s8_dst_handle % 0x1000);
+    for (size_t i = 0; i < sz; ++i) {
+        if (s8_src_handle[i] != s8_dst_handle[i]) { return FAIL; }
+    }
+
+    return OK;
+}
+
+int check_s8s8_reorder(const prb_t &prb, data_kind_t kind,
+        const dnn_mem_t &mem_dt, const dnn_mem_t &mem_fp) {
+    // TODO: enable for all cpu_kind when supported
+    if (is_gpu()) return OK;
+
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_DPCPP
+    // DPC++ does not provide a simple way to access the underlying
+    // buffer alignment.
+    return OK;
+#endif
 
     // In the main test, we fill buffers with f32 and reorder to s8
     // with quantization.
@@ -93,19 +192,37 @@ int check_s8s8_reorder(
     auto nelems = mem_fp.nelems();
     const int64_t n_chunks = 16;
     const int64_t chunk_size = div_up(nelems, n_chunks);
-    const auto quantize = [&](const float *scales, int nscales, int idx_chunk) {
+    const auto quantize = [&](const float *scales, int nscales, float shift,
+                                  int idx_chunk) {
         int64_t idx_start = idx_chunk * chunk_size;
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
             const float current_scale = scales[idx % nscales];
             float val_f32 = mem_fp.get_elem(idx);
-            int8_t val_s8
-                    = saturate_and_round<dnnl_s8>(val_f32 * current_scale);
+            int8_t val_s8 = saturate_and_round<dnnl_s8>(
+                    val_f32 * current_scale + shift);
             mem_s8_src.set_elem(idx, val_s8);
         }
     };
-    dnnl::impl::parallel_nd(n_chunks,
-            [&](int idx) { quantize(prb.wei_scales, prb.wei_nscales, idx); });
+    switch (kind) {
+        case WEIGHTS_LAYER:
+        case WEIGHTS_ITER:
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
+                quantize(prb.wei_scales, prb.wei_nscales, 0, idx);
+            });
+            break;
+        case WEIGHTS_PROJECTION:
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
+                quantize(prb.wei_proj_scales, prb.wei_proj_nscales, 0, idx);
+            });
+            break;
+        case SRC_LAYER:
+        case SRC_ITER:
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
+                quantize(&(prb.data_scale), 1, prb.data_shift, idx);
+            });
+        default: assert(!"unsupported kind");
+    }
 
     /* 2. compute s8_plain_quantized --reorder--> s8_packed_quantized */
     mem_s8_dst.reorder(mem_s8_src);
@@ -163,7 +280,7 @@ int fill_memory(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         std::minstd_rand msr;
         msr.seed(idx_start + kind);
-        std::normal_distribution<float> gen(mean, stddev);
+        normal_distribution_t<float> gen(mean, stddev);
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
             float val = round_to_nearest_representable(dt, gen(msr));
             val = MAX2(MIN2(val, max), min);
@@ -181,20 +298,26 @@ int fill_memory(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
         }
     };
     switch (kind) {
+        case WEIGHTS_PROJECTION:
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
+                fill_chunk(
+                        prb.wei_proj_scales, prb.wei_proj_nscales, 0.0f, idx);
+            });
+            break;
         case WEIGHTS_LAYER:
         case WEIGHTS_ITER:
-            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
                 fill_chunk(prb.wei_scales, prb.wei_nscales, 0.0f, idx);
             });
             break;
         case SRC_LAYER:
         case SRC_ITER:
-            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
                 fill_chunk(&(prb.data_scale), 1, prb.data_shift, idx);
             });
             break;
         default: // we do no scale/shift
-            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+            dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
                 fill_chunk(default_scales, 1, default_shift, idx);
             });
     }
@@ -202,7 +325,11 @@ int fill_memory(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
     // 3. We reorder the data for the DNNL RNN primitive
     mem_dt.reorder(mem_fp, reorder_attr);
     if ((reorder_attr != nullptr) && (dt == dnnl_s8))
-        if (check_s8s8_reorder(prb, mem_dt, mem_fp) != OK) return FAIL;
+        if (check_s8s8_reorder(prb, kind, mem_dt, mem_fp) != OK) return FAIL;
+    if ((kind == WEIGHTS_PROJECTION) && (dt == dnnl_s8))
+        if (check_ldoi_s8_reorder(prb, kind, mem_dt, mem_fp, reorder_attr)
+                != OK)
+            return FAIL;
 
     // Bullet 4.a holds: quantize weights for int8 benchdnn reference RNN
     if (prb.is_int8()) {
@@ -220,10 +347,16 @@ int fill_memory(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
         switch (kind) {
             case WEIGHTS_LAYER:
             case WEIGHTS_ITER:
-                dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+                dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
                     quantize_chunk(prb.wei_scales, prb.wei_nscales, idx);
                 });
                 break;
+            case WEIGHTS_PROJECTION:
+                dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx) {
+                    quantize_chunk(
+                            prb.wei_proj_scales, prb.wei_proj_nscales, idx);
+                });
+                break;
             default: // Nothing to do
                 break;
         }
@@ -357,6 +490,11 @@ int fill_weights(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
         mem_pure_fp.set_elem(i, 0);
     }
 
+    auto scales = (kind == WEIGHTS_PROJECTION) ? prb.wei_proj_scales
+                                               : prb.wei_scales;
+    auto n_scales = (kind == WEIGHTS_PROJECTION) ? prb.wei_proj_nscales
+                                                 : prb.wei_nscales;
+
     // Fill weights sparsely to avoid accumulation errors. Using two memories:
     // one is quantized for reference, another is for a reorder.
     for_(int64_t l = 0; l < L; l++)
@@ -367,7 +505,7 @@ int fill_weights(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
         int64_t off = (((l * D + d) * I + i_off) * G + g) * O + o;
         float val = gate_factor;
         mem_pure_fp.set_elem(off, val);
-        if (prb.is_int8()) val *= prb.wei_scales[off % prb.wei_nscales];
+        if (prb.is_int8()) val *= scales[off % n_scales];
         mem_fp.set_elem(off, round_to_nearest_representable(c.dt, val));
     }
 
@@ -378,7 +516,7 @@ int fill_weights(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
 
     // Test that s8 -> s8 reorder works correctly
     if ((reorder_attr != nullptr) && (c.dt == dnnl_s8))
-        return check_s8s8_reorder(prb, mem_dt, mem_pure_fp);
+        return check_s8s8_reorder(prb, kind, mem_dt, mem_pure_fp);
     return OK;
 }
 
@@ -393,7 +531,7 @@ int fill_bias(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
     auto O = dims[3];
 
     std::minstd_rand msr;
-    std::normal_distribution<float> gen(
+    normal_distribution_t<float> gen(
             prb.cfg[kind].f_mean, prb.cfg[kind].f_stddev);
     msr.seed(kind);
 
@@ -417,7 +555,8 @@ static int init_pd(dnnl_engine_t engine, const prb_t *p_ptr,
     dnnl_rnn_desc_t rd;
     dnnl_prop_kind_t fwd_prop = dnnl_prop_kind_undef;
     switch (prb.prop) {
-        case dnnl_forward: fwd_prop = dnnl_forward_inference; break;
+        case dnnl_forward_training: fwd_prop = dnnl_forward_training; break;
+        case dnnl_forward_inference: fwd_prop = dnnl_forward_inference; break;
         // If we are testing backward, we have to run forward training first
         // in order to generate a valid workspace.
         case dnnl_backward: fwd_prop = dnnl_forward_training; break;
@@ -646,7 +785,7 @@ void check_known_skipped_case(const prb_t &prb, res_t *res) {
         return;
     }
 
-#if !defined(DNNL_X64)
+#if !defined(DNNL_X64) || DNNL_X64 == 0
     // int8 is not supported altogether since RNN relies on packed IGEMM
     // FIXME: this will disable int8 RNN testing if the library is built with
     //        Intel MKL that does have packed IGEMM
@@ -669,14 +808,31 @@ void check_known_skipped_case(const prb_t &prb, res_t *res) {
         }
     }
 
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+    // only AMX LSTM cell kinds support signed int8 so far;
+    if (prb.is_s8()) {
+        static auto isa = dnnl_get_effective_cpu_isa();
+        if (prb.alg != VANILLA_LSTM
+                || !dnnl::is_superset(isa, dnnl_cpu_isa_avx512_core_amx)) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+#endif
+
     // LSTM w/ projection is not supported for bf16
     if (prb.is_lstm_projection() && prb.cfg[SRC_LAYER].dt == dnnl_bf16) {
         res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
         return;
     }
 
+    if (is_nvidia_gpu()) {
+        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
+
     // GPU limitations for RNN
-    if (engine_tgt_kind == dnnl_gpu) {
+    if (is_gpu()) {
         if (prb.is_lstm_projection() || prb.is_lstm_peephole()) {
             res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
             return;
@@ -685,6 +841,12 @@ void check_known_skipped_case(const prb_t &prb, res_t *res) {
             res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
             return;
         }
+        // Implemented only for CPU
+        if (prb.cfg[BIAS].dt == dnnl_bf16 || prb.cfg[SRC_ITER_C].dt == dnnl_bf16
+                || prb.cfg[DST_ITER_C].dt == dnnl_bf16) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
     }
 }
 
@@ -692,21 +854,17 @@ int doit(const prb_t &prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
     check_known_skipped_case(prb, res);
+    check_sum_post_ops(prb.attr, res);
     if (res->state == SKIPPED) return OK;
 
-    dnnl_primitive_t c {};
-    SAFE(init_prim(&c, init_pd, &prb, res), WARN);
+    benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
+    SAFE(init_prim(prim, init_pd, &prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
-    auto cleanup = [&]() {
-        DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
-        return OK;
-    };
 
     const_dnnl_primitive_desc_t const_fpd;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_fpd), CRIT);
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_fpd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_fpd) != OK) {
-        DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
+    if (check_mem_size(const_fpd) != OK) {
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
 
@@ -799,7 +957,7 @@ int doit(const prb_t &prb, res_t *res) {
                  weights_peephole_fp),
             WARN);
     SAFE(fill_weights(prb, WEIGHTS_PROJECTION, weights_projection_dt,
-                 weights_projection_fp),
+                 weights_projection_fp, rnn_attr),
             WARN);
     SAFE(fill_memory(prb, BIAS, bias_dt, bias_fp), WARN);
     SAFE(fill_activation(prb, DST_LAYER, dst_layer_dt, dst_layer_fp), WARN);
@@ -824,36 +982,30 @@ int doit(const prb_t &prb, res_t *res) {
     args.set(DNNL_ARG_WORKSPACE, workspace_dt);
     args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-    SAFE_CLEAN(execute_and_wait(c, args), WARN, cleanup);
+    SAFE(execute_and_wait(prim, args), WARN);
 
-    if ((prb.prop == dnnl_forward) && (bench_mode & CORR)) {
-        compute_ref_fwd(prb, src_layer_fp, src_iter_fp, src_iter_c_fp,
-                weights_layer_fp, weights_iter_fp, weights_peephole_fp,
-                weights_projection_fp, bias_fp, dst_layer_fp, dst_iter_fp,
-                dst_iter_c_fp);
+    if (prb.prop != dnnl_backward) {
+        if (is_bench_mode(CORR)) {
+            compute_ref_fwd(prb, src_layer_fp, src_iter_fp, src_iter_c_fp,
+                    weights_layer_fp, weights_iter_fp, weights_peephole_fp,
+                    weights_projection_fp, bias_fp, dst_layer_fp, dst_iter_fp,
+                    dst_iter_c_fp);
 
-        int compare_status = OK;
-        COMPARE_DAT(compare_status, DST_LAYER, dst_layer, tag::abx /*tnc*/);
-        COMPARE_DAT(compare_status, DST_ITER, dst_iter, tag::abx /*ldnc*/);
-        if (prb.alg == VANILLA_LSTM)
-            COMPARE_DAT(
-                    compare_status, DST_ITER_C, dst_iter_c, tag::abx /*ldnc*/);
-        SAFE_CLEAN(compare_status, WARN, cleanup);
-    }
-
-    if (prb.prop == dnnl_backward) {
-        dnnl_primitive_t bwd_p {};
-        int status = init_prim(&bwd_p, init_pd, &prb, res, FLAG_BWD);
-        dnnl_primitive_destroy(c);
-        if (status != OK) return status;
+            COMPARE_DAT(DST_LAYER, dst_layer, tag::abx /*tnc*/);
+            COMPARE_DAT(DST_ITER, dst_iter, tag::abx /*ldnc*/);
+            if (prb.alg == VANILLA_LSTM)
+                COMPARE_DAT(DST_ITER_C, dst_iter_c, tag::abx /*ldnc*/);
+        }
+    } else {
+        benchdnn_dnnl_wrapper_t<dnnl_primitive_t> tmp_prim;
+        SAFE(init_prim(tmp_prim, init_pd, &prb, res, FLAG_BWD), WARN);
         if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
-        c = bwd_p;
+        prim.reset(tmp_prim.release());
 
         const_dnnl_primitive_desc_t const_bpd;
-        DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_bpd), CRIT);
+        DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_bpd), CRIT);
 
-        if (dnn_mem_t::check_mem_size(const_bpd) != OK) {
-            DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
+        if (check_mem_size(const_bpd) != OK) {
             return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
         }
 
@@ -985,9 +1137,9 @@ int doit(const prb_t &prb, res_t *res) {
         args.set(DNNL_ARG_DIFF_BIAS, diff_bias_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        SAFE_CLEAN(execute_and_wait(c, args), WARN, cleanup);
+        SAFE(execute_and_wait(prim, args), WARN);
 
-        if (bench_mode & CORR) {
+        if (is_bench_mode(CORR)) {
             compute_ref_bwd(prb, src_layer_fp, src_iter_fp, src_iter_c_fp,
                     diff_dst_layer_fp, diff_dst_iter_fp, diff_dst_iter_c_fp,
                     weights_layer_fp, weights_iter_fp, weights_peephole_fp,
@@ -997,46 +1149,32 @@ int doit(const prb_t &prb, res_t *res) {
                     diff_weights_iter_fp, diff_weights_peephole_fp,
                     diff_weights_projection_fp, diff_bias_fp);
 
-            int compare_fwd_status = OK;
-            COMPARE_DAT(
-                    compare_fwd_status, DST_LAYER, dst_layer, tag::abx /*tnc*/);
-            COMPARE_DAT(
-                    compare_fwd_status, DST_ITER, dst_iter, tag::abx /*ldnc*/);
+            COMPARE_DAT(DST_LAYER, dst_layer, tag::abx /*tnc*/);
+            COMPARE_DAT(DST_ITER, dst_iter, tag::abx /*ldnc*/);
             if (prb.alg == VANILLA_LSTM)
-                COMPARE_DAT(compare_fwd_status, DST_ITER_C, dst_iter_c,
-                        tag::abx /*ldnc*/);
-            SAFE_CLEAN(compare_fwd_status, WARN, cleanup);
+                COMPARE_DAT(DST_ITER_C, dst_iter_c, tag::abx /*ldnc*/);
 
-            int compare_bwd_data_status = OK;
-            COMPARE_DAT(compare_bwd_data_status, DIFF_SRC_LAYER, diff_src_layer,
-                    tag::abx /*tnc*/);
-            COMPARE_DAT(compare_bwd_data_status, DIFF_SRC_ITER, diff_src_iter,
-                    tag::abx /*ldnc*/);
+            COMPARE_DAT(DIFF_SRC_LAYER, diff_src_layer, tag::abx /*tnc*/);
+            COMPARE_DAT(DIFF_SRC_ITER, diff_src_iter, tag::abx /*ldnc*/);
             if (prb.alg == VANILLA_LSTM)
-                COMPARE_DAT(compare_bwd_data_status, DIFF_SRC_ITER_C,
-                        diff_src_iter_c, tag::abx /*ldnc*/);
-            SAFE_CLEAN(compare_bwd_data_status, WARN, cleanup);
+                COMPARE_DAT(
+                        DIFF_SRC_ITER_C, diff_src_iter_c, tag::abx /*ldnc*/);
 
-            int compare_bwd_weights_status = OK;
-            COMPARE_DAT(compare_bwd_weights_status, DIFF_WEIGHTS_LAYER,
-                    diff_weights_layer, tag::abx /*ldigo*/);
-            COMPARE_DAT(compare_bwd_weights_status, DIFF_WEIGHTS_ITER,
-                    diff_weights_iter, tag::abx /*ldigo*/);
+            COMPARE_DAT(
+                    DIFF_WEIGHTS_LAYER, diff_weights_layer, tag::abx /*ldigo*/);
+            COMPARE_DAT(
+                    DIFF_WEIGHTS_ITER, diff_weights_iter, tag::abx /*ldigo*/);
             if (prb.is_lstm_peephole())
-                COMPARE_DAT(compare_bwd_weights_status, DIFF_WEIGHTS_PEEPHOLE,
-                        diff_weights_peephole, tag::abx /*ldgo*/);
+                COMPARE_DAT(DIFF_WEIGHTS_PEEPHOLE, diff_weights_peephole,
+                        tag::abx /*ldgo*/);
             if (prb.is_lstm_projection())
-                COMPARE_DAT(compare_bwd_weights_status, DIFF_WEIGHTS_PROJECTION,
-                        diff_weights_projection, tag::abx /*ldio*/);
-            COMPARE_DAT(compare_bwd_weights_status, DIFF_BIAS, diff_bias,
-                    tag::abx /*ldgo*/);
-            SAFE_CLEAN(compare_bwd_weights_status, WARN, cleanup);
+                COMPARE_DAT(DIFF_WEIGHTS_PROJECTION, diff_weights_projection,
+                        tag::abx /*ldio*/);
+            COMPARE_DAT(DIFF_BIAS, diff_bias, tag::abx /*ldgo*/);
         }
     }
 
-    measure_perf(res->timer, c, args);
-    cleanup();
-
-    return OK;
+    return measure_perf(res->timer, prim, args);
 }
+
 } // namespace rnn
