@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,18 +32,29 @@
 #define collapse(x)
 #endif
 
-#include "dnnl.hpp"
-#include "dnnl_debug.h"
-#include "dnnl_test_macros.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
+#include "oneapi/dnnl/dnnl_debug.h"
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
+#if DNNL_CPU_THREADING_RUNTIME == DNNL_RUNTIME_THREADPOOL
+#include "oneapi/dnnl/dnnl_threadpool.hpp"
+#endif
+
+#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL || defined(DNNL_WITH_SYCL)
 #include "dnnl_test_common_ocl.hpp"
 #endif
 
+#ifdef DNNL_WITH_SYCL
+#include "oneapi/dnnl/dnnl_sycl.hpp"
+#endif
+
+// Don't move it higher than library public headers
+#include "dnnl_test_macros.hpp"
+
 #include "src/common/bfloat16.hpp"
 #include "src/common/float16.hpp"
 #include "src/common/memory_desc_wrapper.hpp"
 #include "src/common/nstl.hpp"
+#include "src/common/primitive_cache.hpp"
 #include "tests/gtests/test_malloc.hpp"
 #include "tests/test_thread.hpp"
 
@@ -68,6 +79,13 @@ using dnnl::impl::f16_support::float16_t;
     } while (0)
 #endif
 
+// XXX: Using EXPECT_NE in 'if' statement raises a warning when GCC compiler is
+// used: suggest explicit braces to avoid ambiguous 'else'
+#define GTEST_EXPECT_NE(val1, val2) \
+    do { \
+        EXPECT_NE(val1, val2); \
+    } while (0)
+
 using memory = dnnl::memory;
 
 bool is_current_test_failed();
@@ -77,14 +95,48 @@ dnnl::engine::kind get_test_engine_kind();
 dnnl::engine get_test_engine();
 #endif
 
-inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
-    dnnl::engine::kind kind = eng.get_kind();
+inline int get_vendor_id(const std::string &vendor) {
+    if (vendor == "nvidia") {
+        return 0x10DE;
+    } else if (vendor == "intel") {
+        return 0x8086;
+    } else {
+        return -1;
+    }
+}
 
+inline bool is_nvidia_gpu(const dnnl::engine &eng) {
+#ifdef DNNL_WITH_SYCL
+    if (eng.get_kind() != dnnl::engine::kind::gpu) return false;
+    const uint32_t nvidia_vendor_id = get_vendor_id("nvidia");
+    const auto device = dnnl::sycl_interop::get_device(eng);
+    const auto eng_vendor_id
+            = device.get_info<cl::sycl::info::device::vendor_id>();
+    return eng_vendor_id == nvidia_vendor_id;
+#endif
+    return false;
+}
+
+inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
     bool supported = true; // optimism
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+    dnnl::engine::kind kind = eng.get_kind();
     if (kind == dnnl::engine::kind::cpu)
         supported = dnnl::impl::cpu::platform::has_data_type_support(
                 memory::convert_to_c(dt));
+#endif
 
+#ifdef DNNL_SYCL_CUDA
+    if (is_nvidia_gpu(eng)) {
+        switch (dt) {
+            case memory::data_type::f32: return false;
+            case memory::data_type::f16: return false;
+            case memory::data_type::s8: return false;
+            default: return true;
+        }
+    }
+#endif
     return !supported;
 }
 
@@ -211,6 +263,7 @@ struct mapped_ptr_t {
 
     operator T *() { return ptr_; }
     operator const T *() const { return ptr_; }
+    operator bool() const { return ptr_ != nullptr; }
 
 private:
     const memory *mem_;
@@ -333,6 +386,15 @@ static void fill_data(const memory::dim size, const memory &mem,
     fill_data<data_t>(size, data_ptr, sparsity, init_negs);
 }
 
+template <typename data_t>
+static void remove_zeroes(const memory &mem) {
+    size_t nelems = mem.get_desc().get_size() / sizeof(data_t);
+    auto data_ptr = map_memory<data_t>(mem);
+    dnnl::impl::parallel_nd(nelems, [&](memory::dim n) {
+        if (data_ptr[n] == data_t(0)) data_ptr[n] += data_t(1);
+    });
+}
+
 template <typename data_t>
 static void compare_data(
         const memory &ref, const memory &dst, data_t threshold = (data_t)1e-4) {
@@ -513,7 +575,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
                 // Print unimplemented but do not treat as error
                 std::cout << "[  UNIMPL  ] "
                           << "Implementation not found" << std::endl;
-                reset_malloc_counter();
+                reset_failed_malloc_counter();
                 return true;
             } else if (test_out_of_memory()
                     && (e.status == dnnl_out_of_memory
@@ -525,7 +587,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
                 // gemm_pack_storage_shell_t ctor makes it unable to use the
                 // reference RNN impl, and the iterator produces an
                 // `dnnl_unimplemented` error.
-                increment_malloc_counter();
+                increment_failed_malloc_counter();
                 return catch_expected_failures(f, expect_to_fail,
                         expected_status, ignore_unimplemented);
             } else {
@@ -540,7 +602,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
         // Return normally if the failure is expected. Reset failed malloc
         // counter to zero before performing a new test.
         if (expect_to_fail) {
-            reset_malloc_counter();
+            reset_failed_malloc_counter();
             return true;
         }
     }
@@ -554,10 +616,51 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
     }
 
     // Reset failed malloc counter to zero before performing a new test.
-    reset_malloc_counter();
+    reset_failed_malloc_counter();
     return false;
 }
 
+namespace test {
+inline dnnl::memory make_memory(
+        const dnnl::memory::desc &md, const dnnl::engine &eng) {
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_SYCL
+    if (eng.get_kind() == dnnl::engine::kind::cpu) {
+        return dnnl::memory(md, eng);
+    }
+#endif
+
+#if defined(TEST_DNNL_OCL_USM)
+    return dnnl::ocl_interop::make_memory(
+            md, eng, dnnl::ocl_interop::memory_kind::usm);
+#elif defined(TEST_DNNL_DPCPP_BUFFER)
+    return dnnl::sycl_interop::make_memory(
+            md, eng, dnnl::sycl_interop::memory_kind::buffer);
+#else
+    return dnnl::memory(md, eng);
+#endif
+}
+
+inline dnnl::memory make_memory(
+        const dnnl::memory::desc &md, const dnnl::engine &eng, void *handle) {
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_SYCL
+    if (eng.get_kind() == dnnl::engine::kind::cpu) {
+        return dnnl::memory(md, eng, handle);
+    }
+#endif
+
+#if defined(TEST_DNNL_OCL_USM)
+    return dnnl::ocl_interop::make_memory(
+            md, eng, dnnl::ocl_interop::memory_kind::usm, handle);
+#elif defined(TEST_DNNL_DPCPP_BUFFER)
+    return dnnl::sycl_interop::make_memory(
+            md, eng, dnnl::sycl_interop::memory_kind::buffer, handle);
+#else
+    return dnnl::memory(md, eng, handle);
+#endif
+}
+} // namespace test
+
 #define TEST_MALLOC_OFFSET 8
 static char *test_malloc(size_t size) {
     void *ptr;
@@ -585,19 +688,21 @@ static void test_free(char *ptr) {
 class test_memory {
 public:
     test_memory(const memory::desc &d, const dnnl::engine &e) {
-        bool is_cpu_native = (e.get_kind() == dnnl::engine::kind::cpu);
+        bool is_cpu_native = (e.get_kind() == dnnl::engine::kind::cpu)
+                && DNNL_CPU_RUNTIME != DNNL_RUNTIME_SYCL;
 
         size_ = d.get_size();
         if (is_cpu_native) {
             data_.reset(test_malloc(size_), test_free);
-            mem_ = memory(d, e, data_.get());
+            mem_ = test::make_memory(d, e, data_.get());
         } else {
-            mem_ = memory(d, e);
+            mem_ = test::make_memory(d, e);
         }
         // Fill with a magic number to catch possible uninitialized access
         mapped_ptr_t<char> ptr(&mem_);
-        memset(ptr, 0xFF, size_);
+        if (ptr) memset(ptr, 0xFF, size_);
     }
+
     size_t get_size() const { return size_; }
     const memory &get() const { return mem_; }
 
@@ -626,6 +731,18 @@ inline std::string to_string(dnnl_engine_kind_t engine_kind) {
     return ss.str();
 }
 
+inline std::string to_string(dnnl_stream_flags_t stream_flags) {
+    std::stringstream ss;
+    if (stream_flags & dnnl_stream_default_flags)
+        ss << "default";
+    else if (stream_flags & dnnl_stream_in_order)
+        ss << "in_order";
+    else if (stream_flags & dnnl_stream_out_of_order)
+        ss << "out_of_order";
+
+    return ss.str();
+}
+
 // testing all available C++ primitive descriptor constructors
 struct allows_attr_t {
     bool oscale;
@@ -710,18 +827,10 @@ void test_fwd_pd_attr_zp(
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr_scales(
         const op_desc_t &op_desc, const engine &eng, bool supports_scales) {
-    using dt_t = memory::data_type;
-
     dnnl::primitive_attr attr_scales;
     attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
 
-    pd_t new_pd(op_desc, eng);
-    const auto dt = new_pd.src_desc().data.data_type;
-    const bool dt_is_integral
-            = dt == dt_t::s32 || dt == dt_t::s8 || dt == dt_t::u8;
-
-    // Scales are supposed to work on int8 data type only.
-    if (supports_scales && dt_is_integral) {
+    if (supports_scales) { // Currently only used with binary ops
         EXPECT_NO_THROW(pd_t pd(op_desc, attr_scales, eng));
 
         // Check oscale and scales don't work together
@@ -880,14 +989,18 @@ void test_bwd_pd_constructors(const op_desc_t &op_desc, const pd_t &pd,
 inline dnnl::stream make_stream(dnnl::engine engine,
         dnnl::stream::flags flags = dnnl::stream::flags::default_flags) {
 #if DNNL_CPU_THREADING_RUNTIME == DNNL_RUNTIME_THREADPOOL
-    if (engine.get_kind() != dnnl::engine::kind::cpu)
-        return dnnl::stream(engine, flags);
-    dnnl::stream_attr stream_attr(dnnl::engine::kind::cpu);
-    stream_attr.set_threadpool(dnnl::testing::get_threadpool());
-    return dnnl::stream(engine, flags, stream_attr);
-#else
-    return dnnl::stream(engine, flags);
+    if (engine.get_kind() == dnnl::engine::kind::cpu)
+        return dnnl::threadpool_interop::make_stream(
+                engine, dnnl::testing::get_threadpool());
 #endif
+    return dnnl::stream(engine, flags);
+}
+
+inline int get_primitive_cache_size() {
+    int result = 0;
+    auto status = dnnl::impl::get_primitive_cache_size(&result);
+    if (status != dnnl::impl::status::success) return -1;
+    return result;
 }
 
 #endif
