@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,11 +14,13 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <memory>
+
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
-#include "dnnl_debug.h"
+#include "oneapi/dnnl/dnnl.hpp"
+#include "oneapi/dnnl/dnnl_debug.h"
 namespace dnnl {
 using fmt = memory::format_tag;
 struct deconvolution_test_params_t {
@@ -131,12 +133,46 @@ private:
 
 protected:
     void SetUp() override {
+        memory::data_type data_type = data_traits<data_t>::data_type;
+        SKIP_IF(unsupported_data_type(data_type),
+                "Engine does not support this data type.");
+
         auto p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
+
+        SKIP_IF_CUDA(
+                !(cuda_check_format_tags(p.formats.src_format, data_type)
+                        && cuda_check_format_tags(
+                                p.formats.dst_format, data_type)
+                        && cuda_check_src_wei_format_tags(p.formats.src_format,
+                                p.formats.weights_format, p.sizes.ng > 1)),
+                "Format is not supported.");
+
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
+    bool cuda_check_format_tags(memory::format_tag tag, memory::data_type dt) {
+        return ((impl::utils::one_of(tag, memory::format_tag::ab,
+                        memory::format_tag::abc, memory::format_tag::abcd,
+                        memory::format_tag::abcde, memory::format_tag::abcdef,
+                        memory::format_tag::acb, memory::format_tag::acdb,
+                        memory::format_tag::acdeb))
+                || (dt == memory::data_type::s8
+                        && impl::utils::one_of(tag, memory::format_tag::aBcd4b,
+                                memory::format_tag::aBcde4b)));
+    }
+
+    bool cuda_check_src_wei_format_tags(
+            memory::format_tag src, memory::format_tag wei, bool is_grouped) {
+        if (src == memory::format_tag::abcd) return true;
+        if (src == memory::format_tag::acdb)
+            return wei
+                    != (is_grouped ? memory::format_tag::abcde
+                                   : memory::format_tag::abcd);
+        return false;
+    }
+
     void Test() {
         auto p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
@@ -167,29 +203,31 @@ protected:
         else
             bias_dims = {};
 
-        dec_src_desc.reset(
-                new memory::desc(src_dims, data_type, p.formats.src_format));
-        dec_dst_desc.reset(
-                new memory::desc(dst_dims, data_type, p.formats.src_format));
-        dec_weights_desc.reset(new memory::desc(
-                weights_dims, data_type, p.formats.weights_format));
-        dec_bias_desc.reset(
-                new memory::desc(bias_dims, data_type, p.formats.bias_format));
+        dec_src_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.formats.src_format);
+        dec_dst_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.formats.src_format);
+        dec_weights_desc = std::make_shared<memory::desc>(
+                weights_dims, data_type, p.formats.weights_format);
+        dec_bias_desc = std::make_shared<memory::desc>(
+                bias_dims, data_type, p.formats.bias_format);
 
-        con_src_desc.reset(
-                new memory::desc(dst_dims, data_type, p.formats.src_format));
-        con_dst_desc.reset(
-                new memory::desc(src_dims, data_type, p.formats.src_format));
-        con_weights_desc.reset(new memory::desc(
-                c_weights_dims, data_type, p.formats.weights_format));
+        con_src_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.formats.src_format);
+        con_dst_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.formats.src_format);
+        con_weights_desc = std::make_shared<memory::desc>(
+                c_weights_dims, data_type, p.formats.weights_format);
 
-        src.reset(new test_memory(*dec_src_desc, eng));
-        weights.reset(new test_memory(*dec_weights_desc, eng));
-        bias.reset(new test_memory(*dec_bias_desc, eng));
-        dst.reset(new test_memory(*dec_dst_desc, eng));
+        src = std::make_shared<test_memory>(*dec_src_desc, eng);
+        weights = std::make_shared<test_memory>(*dec_weights_desc, eng);
+        bias = std::make_shared<test_memory>(*dec_bias_desc, eng);
+        dst = std::make_shared<test_memory>(*dec_dst_desc, eng);
 
         padR = {right_padding(dd.oh, dd.ih, dd.kh, dd.padh, dd.strh, dd.dilh),
                 right_padding(dd.ow, dd.iw, dd.kw, dd.padw, dd.strw, dd.dilw)};
+        SKIP_IF_CUDA(p.sizes.padh < padR[0] || p.sizes.padw < padR[1],
+                "Padding not supported");
         Forward();
         BackwardData();
         BackwardWeights();
@@ -209,7 +247,7 @@ protected:
             fill_data<data_t>(bias->get_size() / sizeof(data_t), bias->get());
         }
 
-        auto weights_tr = memory(*con_weights_desc, eng);
+        auto weights_tr = test::make_memory(*con_weights_desc, eng);
         transpose_wei<data_t>(dd, weights->get(), weights_tr);
         auto deconv_desc = with_bias
                 ? deconvolution_forward::desc(aprop_kind,
@@ -285,7 +323,7 @@ protected:
 
         fill_data<data_t>(dst->get_size() / sizeof(data_t), dst->get());
 
-        auto weights_tr = memory(*con_weights_desc, eng);
+        auto weights_tr = test::make_memory(*con_weights_desc, eng);
         transpose_wei<data_t>(dd, weights->get(), weights_tr);
 
         auto deconv_desc = deconvolution_forward::desc(
@@ -347,7 +385,7 @@ protected:
                 deconvolution_test_params_t>::GetParam();
         auto conv_src = dst;
         auto conv_dst = src;
-        auto conv_weights = memory(*con_weights_desc, eng);
+        auto conv_weights = test::make_memory(*con_weights_desc, eng);
         test_convolution_sizes_t dd = p.sizes;
 
         fill_data<data_t>(src->get_size() / sizeof(data_t), src->get());
@@ -417,13 +455,13 @@ protected:
                                 {DNNL_ARG_DIFF_WEIGHTS, conv_weights}});
         strm.wait();
 
-        auto weights_tr = memory(*con_weights_desc, eng);
+        auto weights_tr = test::make_memory(*con_weights_desc, eng);
         transpose_wei<data_t>(dd, weights->get(), weights_tr);
 
         compare_data<data_t>(weights_tr, conv_weights);
 
         if (with_bias) {
-            auto ref_bias = memory(*dec_bias_desc, eng);
+            auto ref_bias = test::make_memory(*dec_bias_desc, eng);
             compute_bias_bwd<data_t>(dd, dst->get(), ref_bias);
             compare_data<data_t>(ref_bias, bias->get());
         }
@@ -522,8 +560,13 @@ GPU_INST_TEST_CASE(SimpleSmall_NCHW,
         PARAMS(nhwc, hwio, x, nhwc, 2, 1, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
         PARAMS(nhwc, hwio, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
         PARAMS(nhwc, goihw, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 0, 0, 1, 1),
-        PARAMS(nhwc, hwigo, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1)
-
-);
+        PARAMS(nhwc, hwigo, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1));
 
+GPU_INST_TEST_CASE(SimpleSmall_NHWC,
+        PARAMS(nchw, oihw, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
+        PARAMS(nhwc, ohwi, x, nhwc, 2, 1, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nhwc, ohwi, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
+        PARAMS(nchw, goihw, x, nchw, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nchw, goihw, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nhwc, gohwi, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1));
 } // namespace dnnl
