@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,10 +14,12 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <memory>
+
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
 
 namespace dnnl {
 
@@ -348,10 +350,30 @@ protected:
                         && (data_type == memory::data_type::s32
                                 || data_type == memory::data_type::s8),
                 "oneDNN only supports relu w/ slope=0 for integers");
+        SKIP_IF_CUDA(p.alg_kind != algorithm::eltwise_relu
+                        && p.alg_kind != algorithm::eltwise_bounded_relu
+                        && p.alg_kind != algorithm::eltwise_tanh
+                        && p.alg_kind != algorithm::eltwise_elu
+                        && p.alg_kind != algorithm::eltwise_logistic,
+                "Unsupported algorithm type for CUDA");
+        SKIP_IF_CUDA(p.alg_kind == algorithm::eltwise_relu && p.alpha != 0.0,
+                "DNNL only supports relu w/ slope=0 for integers");
+        SKIP_IF_CUDA(!cuda_check_format_tag(p.data_format),
+                "Unsupported format tag");
+        SKIP_IF_CUDA(!cuda_check_format_tag(p.diff_format),
+                "Unsupported format tag");
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
+    bool cuda_check_format_tag(memory::format_tag tag) {
+        // Blocking is not supported by cuDNN
+        return (tag != memory::format_tag::aBcd8b
+                && tag != memory::format_tag::aBcd16b
+                && tag != memory::format_tag::aBcde8b
+                && tag != memory::format_tag::aBcde16b);
+    }
+
     void Test() {
         p = ::testing::TestWithParam<eltwise_test_params_t>::GetParam();
 
@@ -366,10 +388,11 @@ protected:
     }
 
     void Forward() {
-        data_desc.reset(new memory::desc(p.dims, data_type, p.data_format));
-        src = memory(*data_desc, eng);
-        memory dst(*data_desc, eng);
-        memory ref_dst(*data_desc, eng);
+        data_desc = std::make_shared<memory::desc>(
+                p.dims, data_type, p.data_format);
+        src = test::make_memory(*data_desc, eng);
+        auto dst = test::make_memory(*data_desc, eng);
+        auto ref_dst = test::make_memory(*data_desc, eng);
 
         data_t data_median = data_t(0);
         data_t data_deviation = (p.alg_kind == algorithm::eltwise_elu
@@ -404,9 +427,15 @@ protected:
     }
 
     void Backward() {
+        SKIP_IF_CUDA(p.alg_kind != algorithm::eltwise_relu
+                        && p.alg_kind != algorithm::eltwise_bounded_relu,
+                "Unsupported algorithm");
+        SKIP_IF_CUDA(p.diff_format != p.data_format,
+                "CUDA does not support different data formats for data and "
+                "diff vectors");
         memory::desc diff_data_desc(p.dims, data_type, p.diff_format);
-        memory diff_src(diff_data_desc, eng);
-        memory diff_dst(diff_data_desc, eng);
+        auto diff_src = test::make_memory(diff_data_desc, eng);
+        auto diff_dst = test::make_memory(diff_data_desc, eng);
 
         data_t data_median = data_t(0);
         data_t data_deviation = p.alg_kind == algorithm::eltwise_elu
