@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
+* Copyright 2020-2021 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +18,8 @@
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
+#include "tests/test_isa_common.hpp"
 
 namespace dnnl {
 
@@ -28,6 +30,29 @@ protected:
     void SetUp() override {}
 };
 
+TEST_F(attr_test_t, TestFPMathMode) {
+    dnnl::primitive_attr attr;
+    ASSERT_EQ(attr.get_fpmath_mode(), fpmath_mode::strict);
+
+    for (auto m : {fpmath_mode::strict, fpmath_mode::bf16, fpmath_mode::f16,
+                 fpmath_mode::any}) {
+        attr.set_fpmath_mode(m);
+        ASSERT_EQ(m, attr.get_fpmath_mode());
+    }
+}
+
+TEST_F(attr_test_t, TestFPMathModeDefault) {
+    ASSERT_EQ(fpmath_mode::strict, get_default_fpmath_mode());
+
+    for (auto m : {fpmath_mode::strict, fpmath_mode::bf16, fpmath_mode::f16,
+                 fpmath_mode::any}) {
+        set_default_fpmath_mode(m);
+        ASSERT_EQ(m, get_default_fpmath_mode());
+        dnnl::primitive_attr attr;
+        ASSERT_EQ(m, attr.get_fpmath_mode());
+    }
+}
+
 TEST_F(attr_test_t, TestScratchpadMode) {
     dnnl::primitive_attr attr;
     for (auto m : {scratchpad_mode::library, scratchpad_mode::user}) {
@@ -54,9 +79,6 @@ TEST_F(attr_test_t, TestScratchpadModeEx) {
         auto mem_consumption
                 = (long)softmax_pd.query_s64(query::memory_consumption_s64);
 
-        // printf("scratchpad_size: %ld\n", scratchpad_size);
-        // printf("mem consumption: %ld\n", mem_consumption);
-
         if (m == scratchpad_mode::library) {
             ASSERT_EQ(scratchpad_size, 0L);
         } else {
@@ -65,10 +87,41 @@ TEST_F(attr_test_t, TestScratchpadModeEx) {
     }
 }
 
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScratchpadArg) {
+    engine eng = get_test_engine();
+
+    const memory::dim N = 2, C = 2, W = 2;
+
+    memory::desc data_md(
+            {N, C, W}, memory::data_type::f32, memory::format_tag::ncw);
+
+    dnnl::primitive_attr attr;
+    auto softmax_d
+            = softmax_forward::desc(prop_kind::forward_inference, data_md, 1);
+    for (auto m : {scratchpad_mode::library, scratchpad_mode::user}) {
+        attr.set_scratchpad_mode(m);
+        auto softmax_pd = softmax_forward::primitive_desc(softmax_d, attr, eng);
+
+        auto src = test::make_memory(softmax_pd.src_desc(), eng);
+        auto dst = test::make_memory(softmax_pd.dst_desc(), eng);
+        auto scratchpad = test::make_memory(softmax_pd.scratchpad_desc(), eng);
+
+        fill_data<float>(src.get_desc().get_size() / sizeof(float), src);
+
+        stream s(eng);
+
+        softmax_forward softmax_p(softmax_pd);
+        softmax_p.execute(s,
+                {{DNNL_ARG_SRC, src}, {DNNL_ARG_DST, dst},
+                        {DNNL_ARG_SCRATCHPAD, scratchpad}});
+        s.wait();
+    }
+}
+
 HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestIntOutputScales) {
     dnnl::primitive_attr attr;
 
-    int mask;
+    int mask = INT_MAX;
     std::vector<float> scales;
 
     // default scales
@@ -101,7 +154,7 @@ TEST_F(attr_test_t, TestZeroPoints) {
             = {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST};
     const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
             DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
-    int zero_points_mask;
+    int zero_points_mask = INT_MAX;
     std::vector<int> zero_points;
 
     // default zero points
@@ -156,7 +209,7 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScales) {
     const std::vector<int> supported_args = {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1};
     const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
             DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
-    int scales_mask;
+    int scales_mask = INT_MAX;
     std::vector<float> scales;
 
     // default scales
@@ -184,6 +237,81 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScales) {
     ASSERT_EQ(scales[2], 3.);
 }
 
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNDataQuantization) {
+    dnnl::primitive_attr attr;
+
+    float scale = NAN, shift = NAN;
+
+    // default scale and shift
+    attr.get_rnn_data_qparams(scale, shift);
+    ASSERT_EQ(scale, 1.f);
+    ASSERT_EQ(shift, 0.f);
+
+    // non-default
+    attr.set_rnn_data_qparams(0.5f, 2.f);
+    attr.get_rnn_data_qparams(scale, shift);
+    ASSERT_EQ(scale, 0.5f);
+    ASSERT_EQ(shift, 2.f);
+}
+
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNWeightsQuantization) {
+    dnnl::primitive_attr attr;
+
+    int scales_mask = INT_MAX;
+    std::vector<float> scales;
+
+    // default scale and shift
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 1.f);
+
+    // single non-default scales
+    attr.set_rnn_weights_qparams(0, {2.f});
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 2.f);
+
+    // multiple scales
+    attr.set_rnn_weights_qparams(1 << 1, {1.f, 2.f, 4.f});
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 1 << 1);
+    ASSERT_EQ(scales.size(), 3U);
+    ASSERT_EQ(scales[0], 1.f);
+    ASSERT_EQ(scales[1], 2.f);
+    ASSERT_EQ(scales[2], 4.f);
+}
+
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNProjWeightsQuantization) {
+    dnnl::primitive_attr attr;
+
+    int scales_mask = INT_MAX;
+    std::vector<float> scales;
+
+    // default scale and shift
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 1.f);
+
+    // single non-default scales
+    attr.set_rnn_weights_projection_qparams(0, {2.f});
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 2.f);
+
+    // multiple scales
+    attr.set_rnn_weights_projection_qparams(1 << 1, {1.f, 2.f, 4.f});
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 1 << 1);
+    ASSERT_EQ(scales.size(), 3U);
+    ASSERT_EQ(scales[0], 1.f);
+    ASSERT_EQ(scales[1], 2.f);
+    ASSERT_EQ(scales[2], 4.f);
+}
+
 TEST_F(attr_test_t, TestScalesExpectFailure) {
     dnnl::primitive_attr attr;
     const int unsupported_arg = DNNL_ARG_MEAN;
@@ -197,19 +325,23 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestPostOps) {
     dnnl::primitive_attr attr;
     dnnl::post_ops ops;
 
-    algorithm alg;
-    float scale, alpha, beta;
+    algorithm alg = algorithm::undef;
+    float scale = NAN, alpha = NAN, beta = NAN;
+    int32_t sum_zp = INT_MAX;
+    data_type dt = data_type::undef;
 
     ASSERT_EQ(ops.len(), 0);
     ASSERT_EQ(attr.get_post_ops().len(), 0);
 
-    ops.append_sum(1.1f);
+    ops.append_sum(1.1f, 1, data_type::f32);
     attr.set_post_ops(ops);
 
     ASSERT_EQ(attr.get_post_ops().len(), 1);
     ASSERT_EQ(attr.get_post_ops().kind(0), primitive::kind::sum);
-    attr.get_post_ops().get_params_sum(0, scale);
+    attr.get_post_ops().get_params_sum(0, scale, sum_zp, dt);
     ASSERT_FLOAT_EQ(scale, 1.1f);
+    ASSERT_EQ(1, sum_zp);
+    ASSERT_EQ(data_type::f32, dt);
 
     ops.append_eltwise(2.2f, algorithm::eltwise_bounded_relu, 3.3f, 4.4f);
     attr.set_post_ops(ops);
@@ -235,10 +367,22 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestPostOps) {
     attr.get_post_ops().get_params_binary(2, alg, src1_md_out);
     ASSERT_EQ(alg, algorithm::binary_add);
     ASSERT_EQ(src1_md, src1_md_out);
+
+    const int prelu_mask = 1;
+    ops.append_prelu(prelu_mask);
+    attr.set_post_ops(ops);
+    ASSERT_EQ(attr.get_post_ops().len(), 4);
+    ASSERT_EQ(attr.get_post_ops().kind(0), primitive::kind::sum);
+    ASSERT_EQ(attr.get_post_ops().kind(1), primitive::kind::eltwise);
+    ASSERT_EQ(attr.get_post_ops().kind(2), primitive::kind::binary);
+    ASSERT_EQ(attr.get_post_ops().kind(3), primitive::kind::prelu);
+    int mask = INT_MAX;
+    attr.get_post_ops().get_params_prelu(3, mask);
+    ASSERT_EQ(mask, prelu_mask);
 }
 
 TEST_F(attr_test_t, TestPostOpsCheckLimit) {
-    dnnl::post_ops ops_sum, ops_eltwise, ops_binary;
+    dnnl::post_ops ops_sum, ops_eltwise, ops_binary, ops_prelu;
 
     for (int i = 0; i < 32; i++) {
         EXPECT_NO_THROW(ops_sum.append_sum(i + 1.f));
@@ -246,7 +390,10 @@ TEST_F(attr_test_t, TestPostOpsCheckLimit) {
                 i, algorithm::eltwise_relu, 2 * i, 0.f));
         EXPECT_NO_THROW(ops_binary.append_binary(algorithm::binary_add,
                 memory::desc({i}, data_type::s8, memory::format_tag::a)));
+        EXPECT_NO_THROW(ops_prelu.append_prelu(1));
     }
+
+    EXPECT_ANY_THROW(ops_prelu.append_prelu(1));
     EXPECT_ANY_THROW(ops_sum.append_sum(1.f));
     EXPECT_ANY_THROW(
             ops_eltwise.append_eltwise(1.f, algorithm::eltwise_relu, 1.f, 0.f));
@@ -258,9 +405,11 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, DepthwiseFusionPostop) {
     dnnl::primitive_attr attr;
     dnnl::post_ops ops;
 
-    int scales_mask;
+    int scales_mask = INT_MAX;
     std::vector<float> scales_in, scales_out;
-    memory::data_type wei_dt, bias_dt, dst_dt;
+    data_type wei_dt = data_type::undef;
+    data_type bias_dt = data_type::undef;
+    data_type dst_dt = data_type::undef;
 
     ASSERT_EQ(ops.len(), 0);
     ASSERT_EQ(attr.get_post_ops().len(), 0);
@@ -302,6 +451,9 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, DepthwiseFusion) {
     auto engine_kind = get_test_engine_kind();
     SKIP_IF(engine_kind != engine::kind::cpu,
             "Depthwise fusion is only supported on CPU engine");
+#if DNNL_AARCH64
+    SKIP_IF(true, "Depthwise fusion is not supported on AArch64 at this time");
+#endif
 
     engine e {engine_kind, 0};
 
@@ -352,6 +504,43 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, DepthwiseFusion) {
     }
 }
 
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, InnerProdBlockedWeights) {
+    auto engine_kind = get_test_engine_kind();
+    bool skip_test = !DNNL_X64 || (DNNL_CPU_RUNTIME == DNNL_RUNTIME_NONE)
+            || (engine_kind != engine::kind::cpu);
+#if DNNL_X64 && (DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE)
+    skip_test = skip_test || !dnnl::mayiuse(cpu_isa::avx512_core);
+#endif
+    SKIP_IF(skip_test,
+            "Inner product blocked weights test is supported only on "
+            "avx512_core CPU");
+
+    engine e {engine_kind, 0};
+
+    std::vector<memory::format_tag> blocked_weights_tags {
+            memory::format_tag::OIhw16i64o, memory::format_tag::OIhw16i32o,
+            memory::format_tag::OIhw16i16o};
+
+    for (const auto &weights_tag : blocked_weights_tags) {
+        memory::desc src_md {{1024, 512, 1, 1}, memory::data_type::f32,
+                memory::format_tag::any};
+        memory::desc wei_md {
+                {256, 512, 1, 1}, memory::data_type::f32, weights_tag};
+        memory::desc bia_md {
+                {256}, memory::data_type::f32, memory::format_tag::any};
+        memory::desc dst_md {
+                {1024, 256}, memory::data_type::f32, memory::format_tag::any};
+
+        auto ip_desc = inner_product_forward::desc(
+                prop_kind::forward_training, src_md, wei_md, bia_md, dst_md);
+        auto pd = inner_product_forward::primitive_desc(ip_desc, e);
+
+        std::string impl_info;
+        ASSERT_NO_THROW(impl_info = pd.impl_info_str(););
+        ASSERT_NE(impl_info, "ref:any");
+    }
+}
+
 HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestGetAttr) {
     auto engine_kind = get_test_engine_kind();
     SKIP_IF(engine_kind != engine::kind::cpu,
