@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,11 +15,12 @@
 *******************************************************************************/
 
 #include <cmath>
+#include <memory>
 
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
 
 namespace dnnl {
 
@@ -208,21 +209,37 @@ protected:
     void SetUp() override {
         data_type = data_traits<data_t>::data_type;
 
-        SKIP_IF(data_type == memory::data_type::bf16
-                        && get_test_engine_kind() == engine::kind::gpu,
-                "GPU does not support bf16 data type.");
         SKIP_IF(unsupported_data_type(data_type),
                 "Engine does not support this data type.");
 
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
-
+        SKIP_IF_CUDA(!cuda_check_format_tags(p.data_format, p.diff_data_format),
+                "Unsupported format tag");
+        SKIP_IF_CUDA(p.aalgorithm != algorithm::lrn_across_channels,
+                "Unsupported algorithm");
         ASSERT_TRUE(p.aalgorithm == algorithm::lrn_across_channels
                 || p.aalgorithm == algorithm::lrn_within_channel);
 
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
+    bool cuda_check_format_tags(memory::format_tag data_format,
+            memory::format_tag diff_data_format) {
+        bool data_ok = data_format == memory::format_tag::ncdhw
+                || data_format == memory::format_tag::nchw
+                || data_format == memory::format_tag::nhwc
+                || data_format == memory::format_tag::ncw
+                || data_format == memory::format_tag::nwc
+                || data_format == memory::format_tag::any;
+        bool diff_data_ok = diff_data_format == memory::format_tag::ncdhw
+                || diff_data_format == memory::format_tag::nchw
+                || diff_data_format == memory::format_tag::nhwc
+                || diff_data_format == memory::format_tag::ncw
+                || diff_data_format == memory::format_tag::nwc
+                || diff_data_format == memory::format_tag::any;
 
+        return data_ok && diff_data_ok;
+    }
     void Test() {
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
 
@@ -235,14 +252,15 @@ protected:
 
         test_lrn_desc_t ld = p.test_ld;
 
-        src_desc.reset(new memory::desc(
-                {ld.mb, ld.c, ld.h, ld.w}, data_type, p.data_format));
-        dst_desc.reset(new memory::desc(
-                {ld.mb, ld.c, ld.h, ld.w}, data_type, p.data_format));
-        diff_src_desc.reset(new memory::desc(
-                {ld.mb, ld.c, ld.h, ld.w}, data_type, p.diff_data_format));
-        diff_dst_desc.reset(new memory::desc(
-                {ld.mb, ld.c, ld.h, ld.w}, data_type, p.diff_data_format));
+        auto dims = {ld.mb, ld.c, ld.h, ld.w};
+        src_desc = std::make_shared<memory::desc>(
+                dims, data_type, p.data_format);
+        dst_desc = std::make_shared<memory::desc>(
+                dims, data_type, p.data_format);
+        diff_src_desc = std::make_shared<memory::desc>(
+                dims, data_type, p.diff_data_format);
+        diff_dst_desc = std::make_shared<memory::desc>(
+                dims, data_type, p.diff_data_format);
 
         Forward();
         Backward();
@@ -254,8 +272,8 @@ protected:
                 p.test_ld.k);
         lrn_fwd_prim_desc = lrn_forward::primitive_desc(lrn_desc, eng);
 
-        src.reset(new test_memory(*src_desc, eng));
-        dst.reset(new test_memory(*dst_desc, eng));
+        src = std::make_shared<test_memory>(*src_desc, eng);
+        dst = std::make_shared<test_memory>(*dst_desc, eng);
 
         fill_data<data_t>(src->get_size() / sizeof(data_t), src->get());
         fill_data<data_t>(dst->get_size() / sizeof(data_t), dst->get());
@@ -267,7 +285,7 @@ protected:
         std::unordered_map<int, memory> args
                 = {{DNNL_ARG_SRC, src->get()}, {DNNL_ARG_DST, dst->get()}};
         auto workspace_md = lrn_fwd_prim_desc.workspace_desc();
-        workspace = memory(workspace_md, eng);
+        workspace = test::make_memory(workspace_md, eng);
         args.insert({DNNL_ARG_WORKSPACE, workspace});
         l.execute(strm, args);
         strm.wait();
@@ -278,9 +296,9 @@ protected:
                 *diff_dst_desc, p.test_ld.local_size, p.test_ld.alpha,
                 p.test_ld.beta, p.test_ld.k);
 
-        src.reset(new test_memory(*src_desc, eng));
-        diff_src.reset(new test_memory(*diff_src_desc, eng));
-        diff_dst.reset(new test_memory(*diff_dst_desc, eng));
+        src = std::make_shared<test_memory>(*src_desc, eng);
+        diff_src = std::make_shared<test_memory>(*diff_src_desc, eng);
+        diff_dst = std::make_shared<test_memory>(*diff_dst_desc, eng);
 
         auto lrn_prim_desc = lrn_backward::primitive_desc(
                 lrn_desc, eng, lrn_fwd_prim_desc);
@@ -404,16 +422,6 @@ static auto CaffeNHWC_cases = [](algorithm lk) {
             fmt::nhwc, fmt::nhwc, {2, 4, 5, 5, 5, 1.0f, 0.75f, 1.0f}});
 };
 
-static auto Caffe_nChw8c_cases = [](algorithm lk) {
-    return ::testing::Values(lrn_test_params_t {prop_kind::forward_training, lk,
-            fmt::nChw8c, fmt::nChw8c, {2, 96, 55, 55, 3, 1.0f, 0.75f, 1.0f}});
-};
-
-static auto Caffe_nChw16c_cases = [](algorithm lk) {
-    return ::testing::Values(lrn_test_params_t {prop_kind::forward_training, lk,
-            fmt::nChw16c, fmt::nChw16c, {2, 96, 55, 55, 3, 1.0f, 0.75f, 1.0f}});
-};
-
 static auto AlexnetNCHW_cases = [](algorithm lk) {
     return ::testing::Values(
             lrn_test_params_t {prop_kind::forward_training, lk, fmt::nchw,
@@ -430,22 +438,6 @@ static auto AlexnetNHWC_cases = [](algorithm lk) {
                     fmt::nhwc, {2, 256, 27, 27, 5, 1.0e-4f, 0.75f, 1.0f}});
 };
 
-static auto Alexnet_nChw8c_cases = [](algorithm lk) {
-    return ::testing::Values(
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw8c,
-                    fmt::nChw8c, {2, 96, 55, 55, 5, 1.0e-4f, 0.75f, 1.0f}},
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw8c,
-                    fmt::nChw8c, {2, 256, 27, 27, 5, 1.0e-4f, 0.75f, 1.0f}});
-};
-
-static auto Alexnet_nChw16c_cases = [](algorithm lk) {
-    return ::testing::Values(
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw16c,
-                    fmt::nChw16c, {2, 96, 55, 55, 5, 1.0e-4f, 0.75f, 1.0f}},
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw16c,
-                    fmt::nChw16c, {2, 256, 27, 27, 5, 1.0e-4f, 0.75f, 1.0f}});
-};
-
 static auto GoogleNetV1NCHW_cases = [](algorithm lk) {
     return ::testing::Values(
             lrn_test_params_t {prop_kind::forward_training, lk, fmt::nchw,
@@ -454,22 +446,6 @@ static auto GoogleNetV1NCHW_cases = [](algorithm lk) {
                     fmt::nchw, {2, 192, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}});
 };
 
-static auto GoogleNetV1_nChw8c_cases = [](algorithm lk) {
-    return ::testing::Values(
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw8c,
-                    fmt::nChw8c, {2, 64, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}},
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw8c,
-                    fmt::nChw8c, {2, 192, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}});
-};
-
-static auto GoogleNetV1_nChw16c_cases = [](algorithm lk) {
-    return ::testing::Values(
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw16c,
-                    fmt::nChw16c, {2, 64, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}},
-            lrn_test_params_t {prop_kind::forward_training, lk, fmt::nChw16c,
-                    fmt::nChw16c, {2, 192, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}});
-};
-
 static auto RegressionWeightFormat_cases = [](algorithm lk) {
     return ::testing::Values(lrn_test_params_t {prop_kind::forward_training, lk,
             fmt::oihw, fmt::oihw, {2, 64, 56, 56, 5, 1.0e-4f, 0.75f, 1.0f}});
@@ -489,19 +465,10 @@ static auto RegressionWeightFormat_cases = [](algorithm lk) {
     INSTANTIATE_TEST_SUITE_P(nChw16c, test, nChw16c_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P(CaffeNCHW, test, CaffeNCHW_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P(CaffeNHWC, test, CaffeNHWC_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P(Caffe_nChw8c, test, Caffe_nChw8c_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P(Caffe_nChw16c, test, Caffe_nChw16c_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P(AlexnetNCHW, test, AlexnetNCHW_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P(AlexnetNHWC, test, AlexnetNHWC_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P(Alexnet_nChw8c, test, Alexnet_nChw8c_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P( \
-            Alexnet_nChw16c, test, Alexnet_nChw16c_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P( \
             GoogleNetV1NCHW, test, GoogleNetV1NCHW_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P( \
-            GoogleNetV1_nChw8c, test, GoogleNetV1_nChw8c_cases(lk)); \
-    INSTANTIATE_TEST_SUITE_P( \
-            GoogleNetV1_nChw16c, test, GoogleNetV1_nChw16c_cases(lk)); \
     INSTANTIATE_TEST_SUITE_P(RegressionWeightFormat, test, \
             RegressionWeightFormat_cases( \
                     lk)); // This tests compatibility with Intel MKL-DNN v0.14
