@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
 
 #include <vector>
 
@@ -95,6 +95,26 @@ protected:
 
         SKIP_IF(unsupported_data_type(p.base.src.dt),
                 "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.weights.dt),
+                "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.dst.dt),
+                "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.bia_dt),
+                "Engine does not support this data type.");
+        SKIP_IF(get_test_engine_kind() == engine::kind::gpu
+                        && ((p.attr.zero_points.src & P::PER_N)
+                                || (p.attr.zero_points.dst & P::PER_N)),
+                "Per dimensional zero points are not supported on GPU");
+        SKIP_IF(get_test_engine_kind() == engine::kind::cpu
+                        && p.base.src.tag == impl::format_tag::AB8a4b,
+                "Don't test blocked formats on CPU");
+
+        SKIP_IF_CUDA((p.attr.zero_points.src != 0 || p.attr.zero_points.dst != 0
+                             || p.attr.zero_points.weights != 0),
+                "Zero points not supported for CUDA");
+
+        SKIP_IF_CUDA((p.attr.scale_flags & P::MASK_MASK) == P::PER_N,
+                "Per dimensional scaling is not supported for CUDA");
 
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status, false);
@@ -149,9 +169,10 @@ protected:
             if (p.attr.scale_flags & P::RUNTIME) {
                 attr.set_output_scales(mask, {DNNL_RUNTIME_F32_VAL});
 
-                scales_m = memory(
+                scales_m = test::make_memory(
                         {{scale_size}, memory::data_type::f32, {1}}, eng);
                 auto s = map_memory<float>(scales_m);
+                GTEST_EXPECT_NE(s, nullptr);
                 for (memory::dim i = 0; i < scale_size; ++i)
                     s[i] = 2.f;
             } else {
@@ -191,9 +212,10 @@ protected:
 
             if (flags & P::RUNTIME) {
                 attr.set_zero_points(arg, mask, {DNNL_RUNTIME_S32_VAL});
-                zero_points_m = memory(
+                zero_points_m = test::make_memory(
                         {{zero_points_size}, memory::data_type::s32, {1}}, eng);
                 auto z = map_memory<int32_t>(zero_points_m);
+                GTEST_EXPECT_NE(z, nullptr);
                 for (memory::dim i = 0; i < zero_points_size; ++i)
                     z[i] = (arg % 7) - 3;
             } else {
@@ -250,7 +272,8 @@ protected:
             tag bia_tag = bia_dims.size() == 2 ? tag::ab : tag::abc;
             bia_md = init_md({bia_dims, p.base.bia_dt, bia_tag,
                     p.base.dst.flags & P::RUNTIME});
-            bia_m = memory(init_md({bia_dims, p.base.bia_dt, bia_tag}), eng);
+            bia_m = test::make_memory(
+                    init_md({bia_dims, p.base.bia_dt, bia_tag}), eng);
         }
 
         auto matmul_d = matmul::desc(src_md, weights_md, bia_md, dst_md);
@@ -274,14 +297,15 @@ protected:
 
         auto matmul_p = matmul(matmul_pd);
 
-        memory src_m(init_md(p.base.src, true), eng);
-        memory weights_m(init_md(p.base.weights, true), eng);
-        memory dst_m(init_md(p.base.dst, true), eng);
+        auto src_m = test::make_memory(init_md(p.base.src, true), eng);
+        auto weights_m = test::make_memory(init_md(p.base.weights, true), eng);
+        auto dst_m = test::make_memory(init_md(p.base.dst, true), eng);
 
         // Initialize memory to make sanitizers happy
         auto set_to_zero = [](memory &m) {
             if (m) {
                 auto p = map_memory<char>(m);
+                GTEST_EXPECT_NE(p, nullptr);
                 memset(p, 0, m.get_desc().get_size());
             }
         };
@@ -308,6 +332,64 @@ protected:
     }
 };
 
+struct attr_test_t
+    : public ::testing::TestWithParam<std::tuple<memory::dims, memory::dims,
+              memory::format_tag, memory::data_type, int>> {};
+
+HANDLE_EXCEPTIONS_FOR_TEST_P(
+        attr_test_t, TestMatmulShouldCallSameImplementationWithAttributes) {
+    auto engine_kind = get_test_engine_kind();
+    SKIP_IF(!DNNL_X64 || engine_kind != engine::kind::cpu,
+            "Binary impl_info_str should be same only on x64 CPU");
+    engine e {engine_kind, 0};
+
+    const auto &tensor_dims = std::get<0>(GetParam());
+    const auto format_tag = std::get<2>(GetParam());
+
+    auto src_md = memory::desc(tensor_dims, memory::data_type::u8, format_tag);
+    auto weights_md
+            = memory::desc(tensor_dims, memory::data_type::s8, format_tag);
+    auto dst_md = memory::desc(tensor_dims, memory::data_type::s8, format_tag);
+    auto bia_md = memory::desc();
+
+    auto matmul_d = matmul::desc(src_md, weights_md, bia_md, dst_md);
+
+    std::string impl_info_no_postops;
+    auto matmul_pd = matmul::primitive_desc(matmul_d, e);
+    ASSERT_NO_THROW(impl_info_no_postops = matmul_pd.impl_info_str(););
+
+    dnnl::primitive_attr attr;
+    const float scale = 1.f;
+    const float alpha = 1.f;
+    const float beta = 1.f;
+    const float oscale = 1.5f;
+
+    const int ndims = std::get<4>(GetParam());
+    // per-channel output scales
+    std::vector<float> oscales(tensor_dims[1], oscale);
+    attr.set_output_scales(1 << (ndims - 1), oscales);
+
+    dnnl::post_ops ops;
+    ops.append_sum(1.0);
+    ops.append_eltwise(scale, algorithm::eltwise_relu, alpha, beta);
+
+    const auto &binary_po_tensor_dims = std::get<1>(GetParam());
+    const auto &binary_po_mem_dt = std::get<3>(GetParam());
+    SKIP_IF(unsupported_data_type(binary_po_mem_dt),
+            "Engine does not support this data type.");
+    memory::desc src1_po_md(
+            binary_po_tensor_dims, binary_po_mem_dt, format_tag);
+    ops.append_binary(algorithm::binary_add, src1_po_md);
+
+    attr.set_post_ops(ops);
+
+    std::string impl_info_with_postops;
+
+    matmul_pd = matmul::primitive_desc(matmul_d, attr, e);
+    ASSERT_NO_THROW(impl_info_with_postops = matmul_pd.impl_info_str(););
+    ASSERT_EQ(impl_info_no_postops, impl_info_with_postops);
+}
+
 /********************************* TEST CASES *********************************/
 
 using iface = matmul_iface_test_t;
@@ -366,16 +448,25 @@ static auto cases_ef = []() {
             dnnl_unimplemented});
 
     // unimplemented data types
-    cases.push_back(
-            {{{{10, 1}, data_type::f32, tag::ab},
-                     {{1, 20}, data_type::f32, tag::ab},
-                     {{10, 20}, data_type::f32, tag::ab}, data_type::u8},
-                    {}, true, dnnl_unimplemented});
+    if (get_test_engine_kind() == engine::kind::cpu) {
+        cases.push_back(
+                {{{{10, 1}, data_type::f32, tag::ab},
+                         {{1, 20}, data_type::f32, tag::ab},
+                         {{10, 20}, data_type::f32, tag::ab}, data_type::u8},
+                        {}, true, dnnl_unimplemented});
+    }
     // XXX: disable assert in type_helpers.hpp: default_accum_data_type(...)
-    //cases.push_back({{{{10, 1}, data_type::u8, tag::ab}, {{1, 20}, data_type::u8, tag::ab},
+    // cases.push_back({{{{10, 1}, data_type::u8, tag::ab}, {{1, 20},
+    // data_type::u8, tag::ab},
     //                           {{10, 20}, data_type::u8, tag::ab}},
     //        {}, true, dnnl_unimplemented});
 
+    // unimplemented formats (GPU only)
+    cases.push_back({{{{16, 16}, data_type::f32, tag::AB8a4b},
+                             {{16, 16}, data_type::f32, tag::AB8a4b},
+                             {{16, 16}, data_type::f32, tag::AB8a4b}},
+            {}, true, dnnl_unimplemented});
+
     return ::testing::ValuesIn(cases);
 };
 INSTANTIATE_TEST_SUITE_P(EF, iface, cases_ef());
@@ -496,6 +587,13 @@ static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
                                     P::ZERO_POINTS | P::DST | P::COMMON
                                             | P::RUNTIME}}});
 
+    // per_dim_1 zero points + runtime
+    cases.push_back({{{{10, 2}, src_dt, tag::ba},
+                             {{2, 20}, data_type::s8, tag::ab},
+                             {{10, 20}, dst_dt, tag::ab}, data_type::f32},
+            {P::SCALES | P::COMMON | P::RUNTIME,
+                    {P::ZERO_POINTS | P::SRC | P::PER_N | P::RUNTIME, P::NONE,
+                            P::ZERO_POINTS | P::DST | P::PER_N | P::RUNTIME}}});
     // post-ops
     cases.push_back({{{{10, 1}, src_dt, tag::ab},
                              {{1, 20}, data_type::s8, tag::ab},
@@ -537,4 +635,21 @@ INSTANTIATE_TEST_SUITE_P(
 INSTANTIATE_TEST_SUITE_P(
         Generic_u8s8u8, iface, cases_x8(data_type::u8, data_type::u8));
 
+INSTANTIATE_TEST_SUITE_P(TensorDims, attr_test_t,
+        ::testing::Values(
+                // {{src0, src1, dst same_dim}, { binary post-op dim }},
+                // format_tag, post-op data type, ndims
+                std::make_tuple(memory::dims {3, 2, 16, 16},
+                        memory::dims {3, 1, 16, 16}, tag::abcd,
+                        memory::data_type::f32, 4),
+                std::make_tuple(memory::dims {9, 9, 64, 64},
+                        memory::dims {9, 1, 64, 64}, tag::abcd,
+                        memory::data_type::f32, 4),
+                std::make_tuple(memory::dims {3, 2, 16, 16},
+                        memory::dims {3, 2, 16, 16}, tag::abcd,
+                        memory::data_type::f32, 4),
+                std::make_tuple(memory::dims {2, 10, 10, 10},
+                        memory::dims {2, 10, 10, 10}, tag::abcd,
+                        memory::data_type::bf16, 4)));
+
 } // namespace dnnl
