@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,10 +14,12 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <memory>
+
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "dnnl.hpp"
+#include "oneapi/dnnl/dnnl.hpp"
 
 namespace dnnl {
 
@@ -215,8 +217,19 @@ private:
     stream strm;
 
 protected:
+    bool cuda_supported_format_tag(memory::format_tag tag) {
+        return impl::utils::one_of(
+                tag, dnnl_abc, dnnl_abcd, dnnl_acb, dnnl_acdb);
+    }
     void SetUp() override {
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
+        SKIP_IF_CUDA(p.aalgorithm == algorithm::resampling_nearest,
+                "nearet algorithm is not supported for cudnn backend");
+        SKIP_IF_CUDA(p.ndims == 5,
+                "cudnn resampling backend does not support 5d tensor");
+        SKIP_IF_CUDA(!cuda_supported_format_tag(p.src_format),
+                "Unsupported format tag");
+
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
@@ -251,8 +264,10 @@ protected:
         }
 
         memory::data_type data_type = data_traits<data_t>::data_type;
-        src_desc.reset(new memory::desc(src_dims, data_type, p.src_format));
-        dst_desc.reset(new memory::desc(dst_dims, data_type, p.src_format));
+        src_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.src_format);
+        dst_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.src_format);
 
         Forward();
         Backward();
@@ -276,9 +291,9 @@ protected:
                     resampling_pd.dst_desc(), resampling_pd_no_dst.dst_desc());
         }
 
-        memory src(resampling_pd.src_desc(), eng);
-        memory dst(resampling_pd.dst_desc(), eng);
-        memory dst_ref(resampling_pd.dst_desc(), eng);
+        auto src = test::make_memory(resampling_pd.src_desc(), eng);
+        auto dst = test::make_memory(resampling_pd.dst_desc(), eng);
+        auto dst_ref = test::make_memory(resampling_pd.dst_desc(), eng);
 
         fill_data<data_t>(src.get_desc().get_size() / sizeof(data_t), src);
         check_zero_tail<data_t>(1, src);
@@ -301,9 +316,12 @@ protected:
         auto resampling_bwd_pd = resampling_backward::primitive_desc(
                 resampling_bwd_desc, eng, resampling_pd);
 
-        memory diff_src(resampling_bwd_pd.diff_src_desc(), eng);
-        memory diff_dst(resampling_bwd_pd.diff_dst_desc(), eng);
-        memory diff_src_ref(resampling_bwd_pd.diff_src_desc(), eng);
+        auto diff_src
+                = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
+        auto diff_dst
+                = test::make_memory(resampling_bwd_pd.diff_dst_desc(), eng);
+        auto diff_src_ref
+                = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
 
         fill_data<data_t>(
                 diff_dst.get_desc().get_size() / sizeof(data_t), diff_dst);
@@ -365,6 +383,26 @@ INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainLinear, resampling_test_float,
                         EXPAND_SIZES_3D(
                                 1, 16, 5, 10, 1, 10, 5, 1, 2.f, 0.5f, 1.f)}));
 
+GPU_INSTANTIATE_TEST_SUITE_P(TestResamplePlainLinear, resampling_test_float,
+        ::testing::Values(
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(1, 1, 5, 10, 2.f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(1, 1, 525, 5, 0.01f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(13, 10, 7, 13, 1.99f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(10, 16, 7, 13, 1.9f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::nchw,
+                        EXPAND_SIZES_2D(32, 10, 14, 7, 29, 5, 2.1f, 0.72f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::nhwc,
+                        EXPAND_SIZES_2D(2, 14, 5, 5, 2, 3, 0.5f, 0.6f)}));
 INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedLinear,
         resampling_test_float,
         ::testing::Values(
