@@ -99,7 +99,9 @@ class Arg:
         'double or None':'dmlc::optional<double>',\
         'Shape or None':'dmlc::optional<Shape>',\
         'string':'const std::string&',\
-        'tuple of <float>':'nnvm::Tuple<mx_float>'}
+        'tuple of <float>':'nnvm::Tuple<mx_float>',\
+        'tuple of <>':'mxnet::cpp::Shape',\
+        '':'index_t'}
     name = ''
     type = ''
     description = ''
