@@ -19,4 +19,4 @@
 
 # install libraries for mxnet's perl package on ubuntu
 apt-get update && apt-get install -y libmouse-perl pdl cpanminus swig libgraphviz-perl
-cpanm -q Function::Parameters Hash::Ordered PDL::CCS
+cpanm -q Function::Parameters Hash::Ordered PDL~2.064 PDL::CCS
